import AppKit


@NSApplicationMain
class AppDelegate: NSObject, NSApplicationDelegate, NSWindowDelegate, HoverViewDelegate {
    // Main outlets
    @IBOutlet var mainWindow            : MainWindow!
    
    @IBOutlet var hoverView             : HoverView!
    @IBOutlet var totalTimeFld          : NSTextField?
    // Controller outlets
    @IBOutlet var playlistLoader        : PlaylistLoader!
    @IBOutlet var playlistController    : PlaylistController!
    @IBOutlet var playbackController    : PlaybackController!
    @IBOutlet var sideViewController    : SideViewController!
    // Buttons
    @IBOutlet var playbackButtons       : NSSegmentedControl!
    @IBOutlet var fileButton            : NSButton!
    @IBOutlet var kmeterButton          : NSButton!
    @IBOutlet var shuffleButton         : NSButton!
    @IBOutlet var repeatButton          : NSButton!
    @IBOutlet var recordButton          : NSButton!
    // Screen positioning buttons
    @IBOutlet var centerBtn             : NSButton?
    @IBOutlet var upLeftBtn             : NSButton?
    @IBOutlet var upRightBtn            : NSButton?
    @IBOutlet var dnLeftBtn             : NSButton?
    @IBOutlet var dnRightBtn            : NSButton?
    // Timedisplay field & Wavepositionslider
    @IBOutlet var timeDisplay           : NSTextField!
    @IBOutlet var wavepositionSlider    : WaveSlider!
    // OpenURLPanel outlets
    @IBOutlet var openurlpanel          : NSPanel!
    @IBOutlet var mUrlComboBox          : NSComboBox!
    // Radio Search
    @IBOutlet var cuteradiobutton       : NSButton!
    @IBOutlet var shoutCASTbutton       : NSButton!
    
    var mUrlArray                       = [AnyHashable]()
    var mIsSheet                        = false
    
    let preferencesWindowController = PreferencesWindowController(
        viewControllers: [
            GeneralPreferenceViewController(),
            OutputPreferenceViewController(),
        ]
    )
    
    @IBAction func showPreferences(_ sender: Any) {
        preferencesWindowController.showWindow()
    }
    
    // Do not remove: It's for enabling dropping files/folders onto the Dock icon of Phonix
    func application(_ sender: NSApplication, openFile filename: String) -> Bool {
        let urls = [URL(fileURLWithPath: filename)]
        if let entries = playlistLoader.addURLs(urls, sort: false) {
            _ = playbackController?.play(entry: entries[0] )
        }
        return true
    }
    
    var sideViewIsHidden = false
    
    func windowWillExitFullScreen(_ notification: Notification) {
        //
        // workaround for constraint error from sideview, while exiting full screen.
        //
        if sideViewController.sideViewIsHidden() == false {
            sideViewIsHidden = false
            sideViewController.showSideView(false)
        } else {
            sideViewIsHidden = true
        }
    }
    
    func windowDidExitFullScreen(_ notification: Notification) {
        //
        // workaround for constraint error from sideview, while exitting full screen.
        //
        if sideViewIsHidden == false {
            sideViewController.showSideView(true)
        }
    }
    
    override init() {
        cacheKmeter()
        super.init()
        transFormers()
        createPhonixFolders()
        registerFont()
        
        let fontSizetoLineHeightTransformer = FontSizetoLineHeightTransformer()
        ValueTransformer.setValueTransformer(fontSizetoLineHeightTransformer, forName: NSValueTransformerName("FontSizetoLineHeightTransformer"))
    }
    
    override func awakeFromNib() {
        if #available(OSX 10.14, *) {
            mainWindow.appearance = NSAppearance(named: .darkAqua)
        }
        else
        {
            mainWindow.appearance = NSAppearance(named: .vibrantDark)
        }
        mainWindow.level = .normal
        mainWindow.titleVisibility = .hidden
        timeDisplay.toolTip = TimeDisplayTooltip
        timeDisplay.font = timeDisplayFont
        timeDisplay.textColor = MeterBarColor
        wavepositionSlider.toolTip = PositionSliderTooltip
        wavepositionSlider.setKnobImage(NSImage(named: "Transparantvoid")!)
        wavepositionSlider.sendAction(on: [.leftMouseDown, .leftMouseDragged])
        fileButton.toolTip = FileButtonTooltip
        shuffleButton.toolTip = ShuffleButtonTooltip
        repeatButton.toolTip = RepeatButtonTooltip
        kmeterButton.toolTip = KmeterButtonTooltip
        recordButton.toolTip = RecordButtonTooltip
        shoutCASTbutton.toolTip = shoutCastButtonTooltip
        cuteradiobutton.toolTip = cuteRadioButtonTooltip
        playbackButtons.setToolTip(PrevButtonTooltip, forSegment: 0)
        playbackButtons.setToolTip(PlayButtonTooltip, forSegment: 1)
        playbackButtons.setToolTip(NextButtonTooltip, forSegment: 2)
        playbackButtons.segmentStyle = .rounded
        playbackButtons.cell?.isBezeled = false
        playbackButtons.wantsLayer = true
        repeatButton.bezelColor = NSColor.white
        repeatButton.wantsLayer = true
        repeatButton.showsBorderOnlyWhileMouseInside = true
        shuffleButton.bezelColor = NSColor.white
        shuffleButton.wantsLayer = true
        shuffleButton.showsBorderOnlyWhileMouseInside = true
        if #available(OSX 10.14, *) {
            let exposureFilter = CIFilter(name: "CIExposureAdjust")
            exposureFilter!.setDefaults()
            exposureFilter!.setValue(-1.0, forKey: "inputEV")
            fileButton.contentFilters = [exposureFilter!]
            shoutCASTbutton.contentFilters = [exposureFilter!]
            cuteradiobutton.contentFilters = [exposureFilter!]
            fileButton.contentFilters = [exposureFilter!]
        }
        else {
            let image = NSImage.init(named: "NavigatorTemplate")
            let imageRef = image!.cgImage(forProposedRect: nil, context: nil, hints: nil)
            let beginImage = CIImage(cgImage: imageRef!)
            let filter = CIFilter(name: "CIColorInvert")
            filter!.setValue(beginImage, forKey: kCIInputImageKey)
            let rep: NSCIImageRep = NSCIImageRep(ciImage: filter!.outputImage!)
            let nsImage: NSImage = NSImage(size: rep.size)
            nsImage.addRepresentation(rep)
            fileButton.image = nsImage
            
            let image2 = NSImage.init(named: "RepeatModeOffTemplate")
            let imageRef2 = image2!.cgImage(forProposedRect: nil, context: nil, hints: nil)
            let beginImage2 = CIImage(cgImage: imageRef2!)
            let filter2 = CIFilter(name: "CIColorInvert")
            filter2!.setValue(beginImage2, forKey: kCIInputImageKey)
            let rep2: NSCIImageRep = NSCIImageRep(ciImage: filter2!.outputImage!)
            let nsImage2: NSImage = NSImage(size: rep2.size)
            nsImage2.addRepresentation(rep2)
            repeatButton.image = nsImage2
            
            let image3 = NSImage.init(named: "ShuffleOffTemplate")
            let imageRef3 = image3!.cgImage(forProposedRect: nil, context: nil, hints: nil)
            let beginImage3 = CIImage(cgImage: imageRef3!)
            let filter3 = CIFilter(name: "CIColorInvert")
            filter3!.setValue(beginImage3, forKey: kCIInputImageKey)
            let rep3: NSCIImageRep = NSCIImageRep(ciImage: filter3!.outputImage!)
            let nsImage3: NSImage = NSImage(size: rep3.size)
            nsImage3.addRepresentation(rep3)
            shuffleButton.image = nsImage3
        }
        
        fileButton.wantsLayer = true
        fileButton.showsBorderOnlyWhileMouseInside = true
        
        shoutCASTbutton.showsBorderOnlyWhileMouseInside = true
        cuteradiobutton.showsBorderOnlyWhileMouseInside = true
        
        recordButton.wantsLayer = true
        recordButton.isHidden = true
        
        if kMeter() == nil {
            kmeterButton.isEnabled = false
        }
        
        hoverView.delegate = self
        mouseInside(false)
    }
    
    func applicationDidFinishLaunching(_ aNotification: Notification) {
        playbackController?.autoplayStart()
    }
    
    func applicationWillTerminate(_ aNotification: Notification) {
        phonixWillTerminate = true
        let folder = PhonixTabsFolder.path
        //
        // Create Tabs folder if non existent
        //
        if !PhonixFileManager.fileExists(atPath: folder) {
            createPhonixFolders()
            mainWindow.saveTabFiles(toPath: (folder as NSString))
        } else {
            let regex = try? NSRegularExpression(pattern: "^*.m3u$", options: .caseInsensitive)
            removeFiles(regex!, inPath: folder as NSString)
            mainWindow.saveTabFiles(toPath: (folder as NSString))
        }
        //
        // store the last selected tab and Index
        //
        let tabIndex: Int = mainWindow.playingTabIndex()
        //
        UserDefaults.standard.set(tabIndex, forKey: "selectedTabIndex")
        //
        //
        // save the current playlist controller with index of last selection
        //
        //
        // autoplay store last track, playback status and
        //
        UserDefaults.standard.set(playbackController.playbackStatus,            forKey: "lastPlaybackStatus")
        UserDefaults.standard.set(playlistController.currentEntry?.peindex,     forKey: "lastTrackPlaying")
        UserDefaults.standard.set(playbackController.position,                  forKey: "lastTrackPosition")
        playbackController.stopStreamRecorder()
        if UserDefaults.standard.bool(forKey: "useAUeffects") && playbackController.auController != nil {
        playbackController.auController.saveAuSettings()
        }
        
        if playbackController.kMeterController != nil {
            playbackController.kMeterController!.saveKmeterSettings()
        }
        
        UserDefaults.standard.set(playbackController.volumeSlider.doubleValue, forKey: "volume")
        let appBuildString = Bundle.main.object(forInfoDictionaryKey: "CFBundleVersion") as? String
        UserDefaults.standard.setValue(appBuildString, forKey: "PhonixVersion")
        //
        // and just before the end...synchronise
        //
        UserDefaults.standard.synchronize()
    }
    
    func applicationShouldHandleReopen(_ sender: NSApplication, hasVisibleWindows flag: Bool) -> Bool {
        if flag {
            mainWindow.orderFront(self)
        } else {
            mainWindow.makeKeyAndOrderFront(self)
        }
        return true
    }
    
    func setURLArray(_ urlLArray: [AnyHashable]) {
        mUrlArray = urlLArray
    }
    
    @IBAction func doOpenURL(_ sender: NSButton) {
        if sender.tag == NSApplication.ModalResponse.OK.rawValue {
            // validate the URL
            var urlString: String
            urlString = mUrlComboBox.stringValue
            
            guard let url = URL(string: urlString) else { return }
            
            if      url.isFileURL
                || (url.scheme != nil && url.host != nil)
            {
                // save the url
                if !mUrlArray.contains(urlString) {
                    // save the url
                    mUrlArray.append(urlString)
                    // add the url to the combo box
                    mUrlComboBox.addItem(withObjectValue: urlString)
                    // remove the oldest url if the maximum has been exceeded
                    if mUrlArray.count > 15 { 
                        mUrlArray.remove(at: 0)
                        mUrlComboBox.removeItem(at: 0)
                    }
                } else {
                    // move the url to the bottom of the list
                    while let elementIndex = mUrlArray.firstIndex(of: urlString) { mUrlArray.remove(at: elementIndex) }
                    mUrlArray.append(urlString)
                    mUrlComboBox.removeItem(withObjectValue: urlString)
                    mUrlComboBox.addItem(withObjectValue: urlString)
                }
                
                let addedEntries = playlistLoader.addURLs([url], sort: false)
                playbackController.play(addedEntries![0])
                
            } else {
                if mIsSheet {
                    let a = NSAlert()
                    a.messageText = "The URL is not valid"
                    a.informativeText = "Invalid URL"
                    a.addButton(withTitle: "OK")
                    a.alertStyle = NSAlert.Style.warning
                    a.beginSheetModal(for: mainWindow, completionHandler: { (modalResponse) -> Void in
                        if modalResponse == NSApplication.ModalResponse.alertFirstButtonReturn {
                        }
                    })
                }
            }
        }
        openurlpanel.performClose(self)
        mainWindow.endSheet(openurlpanel)
        openurlpanel.orderOut(self)
    }
    
    @IBAction func showTheSheet(_ sender: Any) {
        mainWindow.beginSheet(openurlpanel, completionHandler: { (_ returnCode: NSApplication.ModalResponse) -> Void in
            NSApp.stopModal(withCode: returnCode)
        })
        NSApp.runModal(for: openurlpanel)
    }
    
    @IBAction func endTheSheet(_ sender: Any) {
        mainWindow.endSheet(openurlpanel)
    }
    
    @IBAction func delEntries(_ sender: Any) {
        playlistController.remove(self)
    }
    
    @IBAction func increaseFontSize(_ sender: Any) {
        changeFontSize(1)
    }
    
    @IBAction func decreaseFontSize(_ sender: Any) {
        changeFontSize(-1)
    }
    
    @IBAction func openFiles(_ sender: Any) {
        //print("ApD_openFiles")
        var p: NSOpenPanel?
        p = NSOpenPanel()
        p?.canChooseDirectories = true
        p?.allowsMultipleSelection = true
        p?.beginSheetModal(for: mainWindow, completionHandler: { result in
            if result.rawValue == NSApplication.ModalResponse.OK.rawValue {
                let loadURL = p?.urls
                if let entries = self.playlistLoader.addURLs(loadURL!, sort: true) {
                    _ = self.playbackController?.play(entry: entries[0])
                }
            } else {
                p?.close()
            }
        })
    }
    
    func openURL(url: URL) {
        let urls = [url]
        if let entries = playlistLoader.addURLs(urls, sort: false) {
            _ = playbackController?.play(entry: entries[0] )
        }
    }
    
    @IBAction func savePlaylist(_ sender: Any) {
        var savePanel: NSSavePanel!
        savePanel = NSSavePanel()
        savePanel.allowedFileTypes = acceptablePlaylistTypes()
        savePanel.beginSheetModal(for: mainWindow, completionHandler: { (result) -> Void in
            if result.rawValue == NSApplication.ModalResponse.OK.rawValue {
                _ = self.playlistLoader.saveM3u(savePanel.url!.path)
            } else {
                savePanel.close()
            }
        })
    }
    
    @IBAction func openPhonixSite(_ sender: Any) {
        NSWorkspace.shared.open((URL(string: "https://phonix.nl.eu.org"))!)
    }
    
    @IBAction func openSHOUTcast(_ sender: Any) {
        NSWorkspace.shared.open((URL(string: "https://www.shoutcast.com"))!)
    }
    
    //
    // window positioning
    //
    @IBAction func positionCenter(_ sender: NSButton) {
        mainWindow.center()
    }
    
    @IBAction func postionUpLeft(_ sender: NSButton) {
        mainWindow.setFrameTopLeftPoint(NSPoint(x: NSScreen.main!.visibleFrame.origin.x,
                                                y: NSScreen.main!.frame.size.height))
    }
    
    @IBAction func postionDnLeft(_ sender: NSButton) {
        mainWindow.setFrameOrigin(NSPoint(x: NSScreen.main!.visibleFrame.origin.x,
                                          y: NSScreen.main!.visibleFrame.origin.y))
    }
    
    @IBAction func postionUpRight(_ sender: NSButton) {
        mainWindow.setFrameTopLeftPoint(NSPoint(x: NSScreen.main!.visibleFrame.origin.x + NSScreen.main!.visibleFrame.size.width - mainWindow.frame.size.width,
                                                y: NSScreen.main!.frame.size.height))
    }
    
    @IBAction func postionDnRight(_ sender: NSButton) {
        mainWindow.setFrameOrigin(NSPoint(x: NSScreen.main!.visibleFrame.origin.x + NSScreen.main!.visibleFrame.size.width - mainWindow.frame.size.width,
                                          y: NSScreen.main!.visibleFrame.origin.y))
    }
    
    func mouseInside(_ state: Bool) {
        let hidden = !state
        totalTimeFld?.animator().isHidden   = state
        centerBtn?.animator().isHidden      = hidden
        upLeftBtn?.animator().isHidden      = hidden
        upRightBtn?.animator().isHidden     = hidden
        dnLeftBtn?.animator().isHidden      = hidden
        dnRightBtn?.animator().isHidden     = hidden
    }
}

