#include "PhonixEngine.h"
// SFB Core
#import "SFBAudioEngine/SFBAudioEngine/Player/AudioPlayer.h"
#import "SFBAudioEngine/SFBAudioEngine/Output/CoreAudioOutput.h"
#import "SFBAudioEngine/SFBAudioEngine/Metadata/AudioMetadata.h"
// SFB Metadata formats
#import "SFBAudioEngine/SFBAudioEngine/Metadata/AIFFMetadata.h"
#import "SFBAudioEngine/SFBAudioEngine/Metadata/DSDIFFMetadata.h"
#import "SFBAudioEngine/SFBAudioEngine/Metadata/DSFMetadata.h"
#import "SFBAudioEngine/SFBAudioEngine/Metadata/FLACMetadata.h"
#import "SFBAudioEngine/SFBAudioEngine/Metadata/MODMetadata.h"
#import "SFBAudioEngine/SFBAudioEngine/Metadata/MonkeysAudioMetadata.h"
#import "SFBAudioEngine/SFBAudioEngine/Metadata/MP3Metadata.h"
#import "SFBAudioEngine/SFBAudioEngine/Metadata/MP4Metadata.h"
#import "SFBAudioEngine/SFBAudioEngine/Metadata/OggOpusMetadata.h"
#import "SFBAudioEngine/SFBAudioEngine/Metadata/OggVorbisMetadata.h"
#import "SFBAudioEngine/SFBAudioEngine/Metadata/TrueAudioMetadata.h"
#import "SFBAudioEngine/SFBAudioEngine/Metadata/WAVEMetadata.h"
#import "SFBAudioEngine/SFBAudioEngine/Metadata/WavPackMetadata.h"
// SFB Decoders
#import "SFBAudioEngine/SFBAudioEngine/Decoders/CoreAudioDecoder.h"
#import "SFBAudioEngine/SFBAudioEngine/Decoders/DSDIFFDecoder.h"
#import "SFBAudioEngine/SFBAudioEngine/Decoders/DSDPCMDecoder.h"
#import "SFBAudioEngine/SFBAudioEngine/Decoders/DSFDecoder.h"
#import "SFBAudioEngine/SFBAudioEngine/Decoders/LoopableRegionDecoder.h"
#import "SFBAudioEngine/SFBAudioEngine/Decoders/MODDecoder.h"
#import "SFBAudioEngine/SFBAudioEngine/Decoders/MonkeysAudioDecoder.h"
#import "SFBAudioEngine/SFBAudioEngine/Decoders/MPEGDecoder.h"
#import "SFBAudioEngine/SFBAudioEngine/Decoders/OggOpusDecoder.h"
#import "SFBAudioEngine/SFBAudioEngine/Decoders/OggVorbisDecoder.h"
#import "SFBAudioEngine/SFBAudioEngine/Decoders/TrueAudioDecoder.h"
#import "SFBAudioEngine/SFBAudioEngine/Decoders/WavPackDecoder.h"

// TagLib
#import <taglib/tag.h>
#import <taglib/fileref.h>

#import <taglib/id3v2tag.h>
#import <taglib/xiphcomment.h>

using namespace SFB;

Audio::Player *filePlayer;
AudioUnit effectUnit;
AudioUnit kMeterUnit;

void registerSFB() {
    filePlayer = new Audio::Player;
    SFB::Audio::Metadata::RegisterSubclass<SFB::Audio::AIFFMetadata>();
    SFB::Audio::Metadata::RegisterSubclass<SFB::Audio::DSDIFFMetadata>();
    SFB::Audio::Metadata::RegisterSubclass<SFB::Audio::DSFMetadata>();
    SFB::Audio::Metadata::RegisterSubclass<SFB::Audio::FLACMetadata>();
    SFB::Audio::Metadata::RegisterSubclass<SFB::Audio::MODMetadata>();
    SFB::Audio::Metadata::RegisterSubclass<SFB::Audio::MonkeysAudioMetadata>();
    SFB::Audio::Metadata::RegisterSubclass<SFB::Audio::MP3Metadata>();
    SFB::Audio::Metadata::RegisterSubclass<SFB::Audio::MP4Metadata>();
    SFB::Audio::Metadata::RegisterSubclass<SFB::Audio::OggOpusMetadata>();
    SFB::Audio::Metadata::RegisterSubclass<SFB::Audio::OggVorbisMetadata>();
    SFB::Audio::Metadata::RegisterSubclass<SFB::Audio::TrueAudioMetadata>();
    SFB::Audio::Metadata::RegisterSubclass<SFB::Audio::WAVEMetadata>();
    SFB::Audio::Metadata::RegisterSubclass<SFB::Audio::WavPack>();
    // Register MPEGDecoder BEFORE CoreAudioDecoder takes over MP3!
    SFB::Audio::Decoder::RegisterSubclass<SFB::Audio::MPEGDecoder>();
    SFB::Audio::Decoder::RegisterSubclass<SFB::Audio::CoreAudioDecoder>();
    SFB::Audio::Decoder::RegisterSubclass<SFB::Audio::DSDIFFDecoder>();
    SFB::Audio::Decoder::RegisterSubclass<SFB::Audio::DSFDecoder>();
    SFB::Audio::Decoder::RegisterSubclass<SFB::Audio::MODDecoder>();
    SFB::Audio::Decoder::RegisterSubclass<SFB::Audio::MonkeysAudioDecoder>();
    SFB::Audio::Decoder::RegisterSubclass<SFB::Audio::OggOpusDecoder>();
    SFB::Audio::Decoder::RegisterSubclass<SFB::Audio::OggVorbisDecoder>();
    SFB::Audio::Decoder::RegisterSubclass<SFB::Audio::TrueAudioDecoder>();
    SFB::Audio::Decoder::RegisterSubclass<SFB::Audio::WavPackDecoder>();
    // Register FileExtensions & MIMETypes
    SFB::Audio::Decoder::CreateSupportedMIMETypes();
    SFB::Audio::Decoder::CreateSupportedFileExtensions();
}

AUGraph sfbAUGraph() {
    return static_cast<Audio::CoreAudioOutput&>(filePlayer->GetOutput())._GetAUgraph();
}

AUNode sfbConverterNode(){
    return static_cast<Audio::CoreAudioOutput&>(filePlayer->GetOutput())._GetConverterNode();
}

AudioUnit kMeter () {
    return kMeterUnit;
}

AudioUnit mixerUnit() {
    return static_cast<Audio::CoreAudioOutput&>(filePlayer->GetOutput())._GetMixerUnit();
}

AudioUnit fxUnit() {
    return effectUnit;
}

AudioDeviceID currentDeviceID() {
    AudioDeviceID deviceID = 0;
    static_cast<Audio::CoreAudioOutput&>(filePlayer->GetOutput()).GetDeviceID(deviceID);
    return deviceID;
}

bool setOutputDeviceTo(const char* deviceID) {
    CFStringRef deviceUID;
    deviceUID = CFStringCreateWithCString(kCFAllocatorDefault,  deviceID, kCFStringEncodingUTF8);
    return static_cast<Audio::CoreAudioOutput&>(filePlayer->GetOutput()).SetDeviceUID(deviceUID);
}

bool setMixerVolume(float volume) {
    return static_cast<Audio::CoreAudioOutput&>(filePlayer->GetOutput()).SetMixerVolume(sliderToMixer(volume));
}

bool muteFileConverter(bool mute) {
    double value = mute == true ? 0.0 : 1.0;
    return static_cast<Audio::CoreAudioOutput&>(filePlayer->GetOutput()).SetFileVolume(Float32(value));
}

bool getMeterLevels(Float32 *levels) {
    return static_cast<Audio::CoreAudioOutput&>(filePlayer->GetOutput()).GetMeterLevels(levels);
}

void printAUGraph() {
    static_cast<Audio::CoreAudioOutput&>(filePlayer->GetOutput()).PrintAUGraph();
}

Float64 deviceSampleRate() {
    Float64 sampleRate;
    if (static_cast<Audio::CoreAudioOutput&>(filePlayer->GetOutput()).GetDeviceSampleRate(sampleRate)) {
        return sampleRate;
    } else {
        return -1.0;
    }
}

Float64 sampleRateFromMetaDataForURL(const char* url) {
    std::string str(url);
    CFURLRef urlref = 0;
    CFErrorRef err = nil;
    urlref = CFURLCreateFromFileSystemRepresentation(kCFAllocatorDefault, (const UInt8 *)str.c_str() , (long)str.length(), false);
    auto metadatareader = Audio::Metadata::CreateMetadataForURL(urlref,&err);
    Float64 samplerate = 44100.0;
    if (metadatareader->GetSampleRate()) CFNumberGetValue(metadatareader->GetSampleRate(), kCFNumberFloat64Type, &samplerate);
    return samplerate;
}

bool setDeviceSampleRate(Float64 samplerate) {
    return static_cast<Audio::CoreAudioOutput&>(filePlayer->GetOutput()).SetDeviceSampleRate(samplerate);
}

int getDeviceSampleRates(Float64 *sampleRates) {
    std::vector<AudioValueRange> nominalSampleRanges;
    bool success = static_cast<Audio::CoreAudioOutput&>(filePlayer->GetOutput()).GetDeviceAvailableNominalSampleRates(nominalSampleRanges);
    if (success == false) {
        return 0;
    }
    int sampleRateCount = (int)nominalSampleRanges.size();
    
    for (int i = 0; i < sampleRateCount ; i++) {
        sampleRates[i] = nominalSampleRanges[(unsigned long)i].mMinimum;
    }
    return sampleRateCount;
}

bool setupForFormat(AudioStreamBasicDescription format) {
    return static_cast<Audio::CoreAudioOutput&>(filePlayer->GetOutput())._SetupForFormat(format);
}

CFTimeInterval SFBcurrentTime() {
    CFTimeInterval time = 0.0;
    filePlayer->GetCurrentTime(time);
    return time;
}

CFTimeInterval SFBtotalTime() {
    CFTimeInterval time = 0.0;
    filePlayer->GetTotalTime(time);
    return time;
}

Float32 SFBprogress() {
    SInt64 currentFrame;
    SInt64 totalFrames;
    filePlayer->GetPlaybackPosition(currentFrame, totalFrames);
    return (Float32)currentFrame/(Float32)totalFrames;
}

bool SFBplayURL(const char *url) {
    std::string str(url);
    CFURLRef urlref = 0;
    urlref = CFURLCreateFromFileSystemRepresentation(kCFAllocatorDefault, (const UInt8 *)str.c_str(), (long)str.length(), false);
    return filePlayer->Play(urlref);
}

bool SFBenqueue(const char *url) {
    std::string str(url);
    CFURLRef urlref = 0;
    urlref = CFURLCreateFromFileSystemRepresentation(kCFAllocatorDefault, (const UInt8 *)str.c_str() , (long)str.length(), false);
    return filePlayer->Enqueue(urlref);
}

void SFBplayPause()                     { filePlayer->PlayPause(); }
void SFBstop()                          { filePlayer->Stop(); }
bool SFBseekForward (double seconds)    { return filePlayer->SeekForward(seconds); }
bool SFBseekBackward(double seconds)    { return filePlayer->SeekBackward(seconds); }
bool SFBisPlaying()                     { return filePlayer->IsPlaying(); }
bool SFBisPaused()                      { return filePlayer->IsPaused(); }
bool SFBisStopped()                     { return filePlayer->IsStopped(); }

// Cue
bool enqueueCueURL(const char *url, SInt64 startingFrame, UInt32 frames) {
    std::string str(url);
    CFURLRef urlref = 0;
    urlref = CFURLCreateFromFileSystemRepresentation(kCFAllocatorDefault, (const UInt8 *)str.c_str() , (long)str.length(), false);
    auto decoder = SFB::Audio::LoopableRegionDecoder::CreateForURLRegion(urlref, startingFrame, frames, nullptr);
    CFRelease(urlref);
    return filePlayer->Enqueue(decoder);
}

bool playCueURL   (const char *url, SInt64 startingFrame, UInt32 frames) {
    if (nil == url) return false;
    std::string str(url);
    CFURLRef urlref = 0;
    urlref = CFURLCreateFromFileSystemRepresentation(kCFAllocatorDefault, (const UInt8 *)str.c_str() , (long)str.length(), false);
    auto decoder = SFB::Audio::LoopableRegionDecoder::CreateForURLRegion(urlref, startingFrame, frames, nullptr);
    CFRelease(urlref);
    return filePlayer->Play(decoder);
}

// DSD / DSF
bool playDSDURL(const char *url) {
    if (nil == url) return false;
    std::string str(url);
    CFURLRef urlref = 0;
    urlref = CFURLCreateFromFileSystemRepresentation(kCFAllocatorDefault, (const UInt8 *)str.c_str() , (long)str.length(), false);
    auto decoder = SFB::Audio::DSDPCMDecoder::CreateForURL(urlref, nullptr);
    CFRelease(urlref);
    return filePlayer->Play(decoder);
}

void SFBseek(double position) {
    filePlayer->SeekToTime(position);
}

bool removeEffect() {
    if (static_cast<Audio::CoreAudioOutput&>(filePlayer->GetOutput()).RemoveEffect(effectUnit) == true ) {
        AudioUnitReset(effectUnit, kAudioUnitScope_Global, 0);
        AudioUnitUninitialize(effectUnit);
        AudioComponentInstanceDispose(effectUnit);
        effectUnit = nil;
        return true;
    } else {
        return false;
    }
}

bool removeEffectWhilePlaying() {
    SFBplayPause();
    
    if (removeEffect()) {
        SFBplayPause();
        return true;
    }
    
    SFBplayPause();
    return false;
}

bool addEffect(AudioComponentDescription inDescription) {
    return static_cast<Audio::CoreAudioOutput&>(filePlayer->GetOutput()).AddEffect1(inDescription.componentSubType,
                                                                                    inDescription.componentManufacturer,
                                                                                    0,
                                                                                    0,
                                                                                    &effectUnit);
}

bool addMeter(AudioComponentDescription inDescription) {
    return static_cast<Audio::CoreAudioOutput&>(filePlayer->GetOutput()).AddEffect2(inDescription.componentSubType,
                                                                                    inDescription.componentManufacturer,
                                                                                    0,
                                                                                    0,
                                                                                    &kMeterUnit);
}

// Properties & Metadata
struct phonixProperties SFBpropertiesForURL(CFURLRef url) {
    struct phonixProperties properties;
    properties = {.channels = 2, .bitspersample = 16, .samplerate = 44100.0, .totalframes = 0, .fileformat = nil};
    
    CFErrorRef err = nil;
    //
    // we do this 2x, once for props and once for metadata, we should try to combine this
    //
    auto propertyreader = Audio::Metadata::CreateMetadataForURL(url, &err);
    if (err == nil) {
        
        CFNumberRef channels = propertyreader->GetChannelsPerFrame();
        if (channels != nil) {
            CFNumberGetValue(channels, kCFNumberNSIntegerType, &properties.channels);
        }
        
        CFNumberRef bits = propertyreader->GetBitsPerChannel();
        if (bits != nil) {
            CFNumberGetValue(bits, kCFNumberNSIntegerType, &properties.bitspersample);
        }
        else { properties.bitspersample = 16; }
        
        CFNumberRef sampleRate = propertyreader->GetSampleRate();
        if (sampleRate != nil) {
            CFNumberGetValue(sampleRate, kCFNumberFloatType, &properties.samplerate);
        }
        else { properties.samplerate = 44100.0; }
        
        CFNumberRef totalFrames = propertyreader->GetTotalFrames();
        if (totalFrames != nil) {
            CFNumberGetValue(totalFrames, kCFNumberNSIntegerType, &properties.totalframes);
        }
        
        CFStringRef format = propertyreader->GetFormatName();
        if (format != nil) {
            properties.fileformat = (const char*)CFStringGetCStringPtr(format,  kCFStringEncodingUTF8);
        }
    }
    propertyreader.release();
    return properties;
}

struct phonixMetaPropsData SFBmetaAndPropsForURL(CFURLRef url) {
    struct phonixMetaPropsData data;
    data = {.title = nil, .artist = nil, .album = nil, .genre = nil, .year = nil, .track = 0, .artwork = nil , .imagelength = 0, .channels = 0, .bitspersample = 0, .samplerate = 0.0, .totalframes = 0, .fileformat = nil};
    
    CFErrorRef err = nil;
    //
    // we do this 2x, once for props and once for metadata, we should try to combine this
    //
    auto dataReader = Audio::Metadata::CreateMetadataForURL(url, &err);
    if (err == nil && dataReader != nil) {
        
        auto pictures = dataReader->GetAttachedPictures();
        if (pictures.size() >= 1) {
            data.artwork = (const UInt8*)CFDataGetBytePtr(pictures.front()->GetData());
            data.imagelength = CFDataGetLength(pictures.front()->GetData());
        }
        
        CFStringRef artist = dataReader->GetArtist();
        if (artist != nil) {
            CFIndex length = CFStringGetLength(artist);
            CFIndex maxlen = CFStringGetMaximumSizeForEncoding (length, kCFStringEncodingUTF8);
            char *artistbuffer = (char *)malloc((unsigned long)maxlen+1);
            Boolean success = CFStringGetCString(artist, (char *) artistbuffer, maxlen, kCFStringEncodingUTF8);
            if (success) { data.artist = strdup(artistbuffer); }
            free(artistbuffer);
        }
        
        CFStringRef title = dataReader->GetTitle();
        if (title != nil) {
            CFIndex length = CFStringGetLength(title);
            CFIndex maxlen = CFStringGetMaximumSizeForEncoding (length, kCFStringEncodingUTF8);
            char *titlebuffer = (char *)malloc((unsigned long)maxlen+1);
            Boolean success = CFStringGetCString(title, (char *) titlebuffer, maxlen, kCFStringEncodingUTF8);
            if (success) { data.title = strdup(titlebuffer); }
            free(titlebuffer);
        }
        
        CFStringRef album = dataReader->GetAlbumTitle();
        if (album != nil) {
            CFIndex length = CFStringGetLength(album);
            CFIndex maxlen = CFStringGetMaximumSizeForEncoding (length, kCFStringEncodingUTF8);
            char *albumbuffer = (char *)malloc((unsigned long)maxlen+1);
            Boolean success = CFStringGetCString(album, (char *) albumbuffer, maxlen, kCFStringEncodingUTF8);
            if (success) { data.album = strdup(albumbuffer); }
            free(albumbuffer);
        }
        
        CFStringRef genre = dataReader->GetGenre();
        if (genre != nil) {
            CFIndex length = CFStringGetLength(genre);
            CFIndex maxlen = CFStringGetMaximumSizeForEncoding (length, kCFStringEncodingUTF8);
            char *genrebuffer = (char *)malloc((unsigned long)maxlen+1);
            Boolean success = CFStringGetCString(genre, (char *) genrebuffer, maxlen, kCFStringEncodingUTF8);
            if (success) { data.genre = strdup(genrebuffer); }
            free(genrebuffer);
        }
        
        CFStringRef date = dataReader->GetReleaseDate();
        if (date != nil) {
            CFIndex length = CFStringGetLength(date);
            CFIndex maxlen = CFStringGetMaximumSizeForEncoding (length, kCFStringEncodingUTF8);
            char *yearbuffer = (char *)malloc((unsigned long)maxlen+1);
            Boolean success = CFStringGetCString(date, (char *) yearbuffer, maxlen, kCFStringEncodingUTF8);
            if (success) { data.year = strdup(yearbuffer); }
            free(yearbuffer);
        }
        
        CFNumberRef track = dataReader->GetTrackNumber();
        if (track != nil) {
            CFNumberGetValue(track, kCFNumberNSIntegerType, &data.track);
        }
        
        CFNumberRef channels = dataReader->GetChannelsPerFrame();
        if (channels != nil) {
            CFNumberGetValue(channels, kCFNumberNSIntegerType, &data.channels);
        }
        
        CFNumberRef bits = dataReader->GetBitsPerChannel();
        if (bits != nil) {
            CFNumberGetValue(bits, kCFNumberNSIntegerType, &data.bitspersample);
        }
        
        CFNumberRef sampleRate = dataReader->GetSampleRate();
        if (sampleRate != nil) {
            CFNumberGetValue(sampleRate, kCFNumberFloatType, &data.samplerate);
        }
        
        CFNumberRef totalFrames = dataReader->GetTotalFrames();
        if (totalFrames != nil) {
            CFNumberGetValue(totalFrames, kCFNumberNSIntegerType, &data.totalframes);
        }
        
        CFStringRef format = dataReader->GetFormatName();
        if (format != nil) {
            data.fileformat = (const char*)CFStringGetCStringPtr(format,  kCFStringEncodingUTF8);
        }
    }
    dataReader.release();
    return data;
}

struct phonixMetadata SFBmetadataForURL(CFURLRef url) {
    struct phonixMetadata metadata;
    metadata = {.title = nil, .artist = nil, .album = nil, .genre = nil, .year = nil, .track = 0, .artwork = nil , .imagelength = 0};
    
    CFErrorRef err = nil;
    //
    // we do this 2x, once for props and once for metadata, we should try to combine this : DONE
    //
    auto metadatareader = Audio::Metadata::CreateMetadataForURL(url, &err);
    if (err == nil && metadatareader != nil) {
        
        auto pictures = metadatareader->GetAttachedPictures();
        if (pictures.size() >= 1) {
            metadata.artwork = (const UInt8*)CFDataGetBytePtr(pictures.front()->GetData());
            metadata.imagelength = CFDataGetLength(pictures.front()->GetData());
        }

        CFStringRef artist = metadatareader->GetArtist();
        if (artist != nil) {
            CFIndex length = CFStringGetLength(artist);
            CFIndex maxlen = CFStringGetMaximumSizeForEncoding (length, kCFStringEncodingUTF8);
            char *artistbuffer = (char *)malloc((unsigned long)maxlen+1);
            Boolean success = CFStringGetCString(artist, (char *) artistbuffer, maxlen, kCFStringEncodingUTF8);
            if (success) { metadata.artist = strdup(artistbuffer); }
            free(artistbuffer);
        }
        
        CFStringRef title = metadatareader->GetTitle();
        if (title != nil) {
            CFIndex length = CFStringGetLength(title);
            CFIndex maxlen = CFStringGetMaximumSizeForEncoding (length, kCFStringEncodingUTF8);
            char *titlebuffer = (char *)malloc((unsigned long)maxlen+1);
            Boolean success = CFStringGetCString(title, (char *) titlebuffer, maxlen, kCFStringEncodingUTF8);
            if (success) { metadata.title = strdup(titlebuffer); }
            free(titlebuffer);
        }
        
        CFStringRef album = metadatareader->GetAlbumTitle();
        if (album != nil) {
            CFIndex length = CFStringGetLength(album);
            CFIndex maxlen = CFStringGetMaximumSizeForEncoding (length, kCFStringEncodingUTF8);
            char *albumbuffer = (char *)malloc((unsigned long)maxlen+1);
            Boolean success = CFStringGetCString(album, (char *) albumbuffer, maxlen, kCFStringEncodingUTF8);
            if (success) { metadata.album = strdup(albumbuffer); }
            free(albumbuffer);
        }
        
        CFStringRef genre = metadatareader->GetGenre();
        if (genre != nil) {
            CFIndex length = CFStringGetLength(genre);
            CFIndex maxlen = CFStringGetMaximumSizeForEncoding (length, kCFStringEncodingUTF8);
            char *genrebuffer = (char *)malloc((unsigned long)maxlen+1);
            Boolean success = CFStringGetCString(genre, (char *) genrebuffer, maxlen, kCFStringEncodingUTF8);
            if (success) { metadata.genre = strdup(genrebuffer); }
            free(genrebuffer);
        }
        
        CFStringRef date = metadatareader->GetReleaseDate();
        if (date != nil) {
            CFIndex length = CFStringGetLength(date);
            CFIndex maxlen = CFStringGetMaximumSizeForEncoding (length, kCFStringEncodingUTF8);
            char *yearbuffer = (char *)malloc((unsigned long)maxlen+1);
            Boolean success = CFStringGetCString(date, (char *) yearbuffer, maxlen, kCFStringEncodingUTF8);
            if (success) { metadata.year = strdup(yearbuffer); }
            free(yearbuffer);
        }
        
        CFNumberRef track = metadatareader->GetTrackNumber();
        if (track != nil) {
            CFNumberGetValue(track, kCFNumberNSIntegerType, &metadata.track);
        }
    }
    metadatareader.release();
    return metadata;
}

// TabLib
void TagLibsetTitle(const char* url, const char* title)
{
    TagLib::FileRef f(url, false);
    if (!f.isNull()) {
        const TagLib::Tag *tag = f.tag();
        if (tag) {
            if (title != NULL) {
                f.tag()->setTitle(title);
                f.save();
            }
        }
    }
}

void TagLibsetArtist(const char* url, const char* artist)
{
    TagLib::FileRef f(url, false);
    if (!f.isNull()) {
        const TagLib::Tag *tag = f.tag();
        if (tag) {
            if (artist != NULL) {
                f.tag()->setArtist(artist);
                f.save();
            }
        }
    }
}

void TagLibsetAlbum(const char* url, const char* album)
{
    TagLib::FileRef f(url, false);
    if (!f.isNull()) {
        const TagLib::Tag *tag = f.tag();
        if (tag) {
            if (album != NULL) {
                f.tag()->setAlbum(album);
                f.save();
            }
        }
    }
}

void TagLibsetGenre(const char* url, const char* genre)
{
    TagLib::FileRef f(url, false);
    if (!f.isNull()) {
        const TagLib::Tag *tag = f.tag();
        if (tag) {
            const TagLib::String pArtist, pTitle, pAlbum, pGenre, pComment;
            if (genre != NULL) {
                f.tag()->setGenre(genre);
                f.save();
            }
        }
    }
}

void TagLibsetYear(const char* url, unsigned int year){
    TagLib::FileRef f(url, false);
    if (!f.isNull()) {
        const TagLib::Tag *tag = f.tag();
        if (tag) {
            if (year >= 0) {
                f.tag()->setYear(year);
                f.save();
            }
        }
    }
}

void TagLibsetTrack(const char* url, unsigned int track)
{
    TagLib::FileRef f(url, false);
    if (!f.isNull()) {
        const TagLib::Tag *tag = f.tag();
        if (tag) {
            if (track >= 0) {
                f.tag()->setTrack(track);
                f.save();
            }
        }
    }
}
