#include <AudioUnit/AudioUnit.h>
#include <AudioToolbox/AudioToolbox.h>

// for HTTPWaveFormOSX
#import "TPCircularBuffer.h"

#ifdef __cplusplus
extern "C"
{
#endif
    float sliderToMixer(float slider) { return slider * slider * slider; }
    
    void registerSFB();
    void printAUGraph();
    
    // Units & AUGraph
    AUGraph sfbAUGraph();
    AUNode sfbConverterNode();
    AudioUnit kMeter();
    AudioUnit mixerUnit();
    AudioUnit fxUnit();
    
    // AudioDevice
    AudioDeviceID currentDeviceID();
    bool setOutputDeviceTo(const char* deviceID);
    
    // Volume
    bool setMixerVolume(float volume);
    bool muteFileConverter(bool mute);
    bool getMeterLevels(Float32 *levels);
    
    // Samplerate
    int getDeviceSampleRates(Float64 *sampleRates);
    bool setDeviceSampleRate(Float64 samplerate);
    Float64 deviceSampleRate();
    Float64 sampleRateFromMetaDataForURL(const char* url);
    bool setupForFormat(AudioStreamBasicDescription format);
    
    // Time / Progress
    CFTimeInterval SFBcurrentTime();
    CFTimeInterval SFBtotalTime();
    Float32 SFBprogress();
    
    // Transport
    bool SFBplayURL(const char *url);
    bool SFBenqueue(const char *url);
    bool enqueueCueURL(const char *url, SInt64 startingFrame, UInt32 frames);
    bool playCueURL   (const char *url, SInt64 startingFrame, UInt32 frames);
    bool playDSDURL(const char *url);
    void SFBplayPause();
    void SFBstop();
    void SFBseek(double position);
    bool SFBseekForward(double seconds);
    bool SFBseekBackward(double seconds);
    bool SFBisPlaying();
    bool SFBisPaused();
    bool SFBisStopped();
    
    // Effects
    bool removeEffectWhilePlaying();
    bool removeEffect();
    bool addEffect(AudioComponentDescription inDescription);
    bool addMeter(AudioComponentDescription inDescription);
    
    // Properties
    struct phonixProperties {
        int channels;
        int bitspersample;
        float samplerate;
        int totalframes;
        const char *fileformat;
    };
    struct phonixProperties SFBpropertiesForURL(CFURLRef url);
    // Metadata
    struct phonixMetadata {
        const char *artist;
        const char *title;
        const char *album;
        const char *genre;
        const char *year;
        int track;
        const UInt8 *artwork;
        long imagelength;
    };
    struct phonixMetadata SFBmetadataForURL(CFURLRef url);
    //
    struct phonixMetaPropsData {
        const char *artist;
        const char *title;
        const char *album;
        const char *genre;
        const char *year;
        int track;
        const UInt8 *artwork;
        long imagelength;
        int channels;
        int bitspersample;
        float samplerate;
        int totalframes;
        const char *fileformat;
    };
    struct phonixMetaPropsData SFBmetaAndPropsForURL(CFURLRef url);
    
    // TagLib
    void TagLibsetTitle(const char* url, const char* title);
    void TagLibsetArtist(const char* url, const char* artist);
    void TagLibsetAlbum(const char* url, const char* album);
    void TagLibsetGenre(const char* url, const char* genre);
    void TagLibsetYear(const char* url, unsigned int year);
    void TagLibsetTrack(const char* url, unsigned int track);
    void TagLibremoveEmbeddedArtwork(const char* url, const char* extension);
    const char* TagLibsetArtwork(const char* url,const char* extension, const UInt8 *image, int imagelength);
    
#ifdef __cplusplus
}
#endif

