/*
 * Copyright (c) 2010 - 2018 Stephen F. Booth <me@sbooth.org>
 * See https://github.com/sbooth/SFBAudioEngine/blob/master/LICENSE.txt for license information
 */

#include <stdexcept>

#include "SFBSemaphore.h"
#include "Logger.h"

SFB::SFBSemaphore::SFBSemaphore()
	: mSemaphore(nullptr)
{
	mSemaphore = dispatch_semaphore_create(0);

	if(nullptr == mSemaphore) {
		LOGGER_CRIT("org.sbooth.AudioEngine.Semaphore", "dispatch_semaphore_create failed");
		throw std::runtime_error("Unable to create the semaphore");
	}
}

SFB::SFBSemaphore::~SFBSemaphore()
{
	dispatch_release(mSemaphore);
	mSemaphore = nullptr;
}

bool SFB::SFBSemaphore::Signal()
{
	return dispatch_semaphore_signal(mSemaphore);
}

bool SFB::SFBSemaphore::Wait()
{
	return TimedWait(DISPATCH_TIME_FOREVER);
}

bool SFB::SFBSemaphore::TimedWait(dispatch_time_t duration)
{
	return !dispatch_semaphore_wait(mSemaphore, duration);
}
