import AppKit

class ArtPanelController: NSController, NSWindowDelegate, HoverViewDelegate {
    
    @IBOutlet var artPanel          : NSPanel!
    
    @IBOutlet var centerBtn         : NSButton?
    @IBOutlet var upLeftBtn         : NSButton?
    @IBOutlet var upRightBtn        : NSButton?
    @IBOutlet var dnLeftBtn         : NSButton?
    @IBOutlet var dnRightBtn        : NSButton?
    
    @IBOutlet var hoverView         : HoverView?
        
    @IBOutlet var panelCoverArt     : NSImageView!
    @IBOutlet var panelTitleFld     : NSTextField!
    @IBOutlet var panelArtistFld    : NSTextField!
    @IBOutlet var panelAlbumFld     : NSTextField!
    @IBOutlet var panelLengthFld    : NSTextField!
    @IBOutlet var panelYearFld      : NSTextField!
    @IBOutlet var panelFormatFld    : NSTextField!
    @IBOutlet var panelInfoBG       : NSTextField!
    
    override func awakeFromNib() {
        super.awakeFromNib()
        hoverView?.delegate = self
        panelTitleFld.textColor     = MeterBarColor
        panelArtistFld.textColor    = MeterBarColor
        panelAlbumFld.textColor     = MeterBarColor
        panelLengthFld.textColor    = MeterBarColor
        panelYearFld.textColor      = MeterBarColor
        panelFormatFld.textColor    = MeterBarColor
        panelInfoBG.backgroundColor = MidGrayColor
        initArtPanel()
    }
    
    func initArtPanel() {
        if let position = UserDefaults.standard.value(forKey: "artPanelPositon") as? [CGFloat] {
            var pos = CGPoint()
            pos.x = position[0]
            pos.y = position[1]
            artPanel.setFrameOrigin(pos)
        } else {
            artPanel.center()
        }
        artPanel.isMovable = true
        artPanel.isFloatingPanel = true
        mouseInside(false) // hoverView init
        if UserDefaults.standard.bool(forKey: "showArtPanel") {
            self.artPanel.orderFront(self)
        }
    }
    
    @IBAction func positionCenter(_ sender: NSButton) {
        artPanel.center()
    }
    
    @IBAction func postionUpLeft(_ sender: NSButton) {
        artPanel.setFrameTopLeftPoint(NSPoint(x: NSScreen.main!.visibleFrame.origin.x,
                                              y: NSScreen.main!.frame.size.height))
    }
    
    @IBAction func postionDnLeft(_ sender: NSButton) {
        artPanel.setFrameOrigin(NSPoint(x: NSScreen.main!.visibleFrame.origin.x,
                                        y: NSScreen.main!.visibleFrame.origin.y))
    }
    
    @IBAction func postionUpRight(_ sender: NSButton) {
        artPanel.setFrameTopLeftPoint(NSPoint(x: NSScreen.main!.visibleFrame.origin.x + NSScreen.main!.visibleFrame.size.width - artPanel.frame.size.width,
                                              y: NSScreen.main!.frame.size.height))
    }
    
    @IBAction func postionDnRight(_ sender: NSButton) {
        artPanel.setFrameOrigin(NSPoint(x: NSScreen.main!.visibleFrame.origin.x + NSScreen.main!.visibleFrame.size.width - artPanel.frame.size.width,
                                        y: NSScreen.main!.visibleFrame.origin.y))
    }
    
    func windowDidMove(_ notification: Notification) {
        var origin = artPanel.frame.origin        
        if origin.x >= NSScreen.main!.visibleFrame.size.width - artPanel.frame.size.width {
            origin.x = NSScreen.main!.visibleFrame.size.width - artPanel.frame.size.width
        } else if origin.x <= 0.0 {
            origin.x = 0.0
        }
        if origin.y <= 0.0 {
            origin.y = 0.0
        }
        artPanel.setFrameOrigin(origin)
        UserDefaults.standard.set([origin.x, artPanel.frame.origin.y], forKey: "artPanelPositon")
    }
    
    func mouseInside(_ state: Bool) {
        let hidden = !state
        centerBtn? .animator().isHidden = hidden
        upLeftBtn? .animator().isHidden = hidden
        upRightBtn?.animator().isHidden = hidden
        dnLeftBtn? .animator().isHidden = hidden
        dnRightBtn?.animator().isHidden = hidden
    }
}
