
class FileItem {
    
    var url: URL!
    var parent: FileItem?
    
    static let fileManager = FileManager()
    static let options: FileManager.DirectoryEnumerationOptions = [.skipsHiddenFiles, .skipsPackageDescendants, .skipsSubdirectoryDescendants]
    static let resourceKeys : [URLResourceKey] = [.nameKey, .isDirectoryKey]
    
    lazy var children: [FileItem]? = {
        guard let directoryURL = self.url else { return nil }
        
        if let directoryEnumerator = FileItem.fileManager.enumerator(at: directoryURL,
                                                                     includingPropertiesForKeys: FileItem.resourceKeys,
                                                                     options: FileItem.options)
        {
            var fileItems : [FileItem] = []
            
            for case let fileURL as NSURL in directoryEnumerator {
                guard let resourceValues = try? fileURL.resourceValues(forKeys: FileItem.resourceKeys),
                    let isDirectory = resourceValues[URLResourceKey.isDirectoryKey] as? Bool
                    else {
                        continue
                }
                
                if isDirectory || acceptableFileTypes().contains(fileURL.pathExtension!.lowercased()) {
                    fileItems.append(FileItem(url: fileURL as URL, parent: self))
                    //directoryEnumerator!.skipDescendants()
                } else if let aliasURL = resolveFinderAlias(at: fileURL as URL) {
                    fileItems.append(FileItem(url: aliasURL, parent: self))
                }
            }
            fileItems.sort { $0.url.lastPathComponent.compare(
                $1.url.lastPathComponent, options:  String.CompareOptions.caseInsensitive ) == .orderedAscending
            }
            return fileItems
        }
        return nil
    }()
    
    init(url:URL, parent: FileItem?) {
        self.url = url
        self.parent = parent
    }
    
    var displayName: String {
        return self.url.lastPathComponent
    }
    
    var count: Int {
        return (self.children?.count)!
    }
    
    func childAtIndex(_ n: Int) -> FileItem? {
        return self.children![n]
    }
}
