import AppKit

class FileBrowserOutlineView: NSOutlineView {
    @IBOutlet var fileBrowserController : FileBrowserController!
    @IBOutlet var filebrowserDataSource : FileBrowserDataSource!
    
    override func awakeFromNib() {
        backgroundColor = BackgroundColorA
        fileBrowserController.initControls()
        registerForDraggedTypes([NSPasteboard.PasteboardType(rawValue: PhonixUrlsPboardType)])
        
        //print("FTV autosaveExpandedItems:", autosaveExpandedItems)
        doubleAction = #selector(fileBrowserController.doubleClickItem)
        target = fileBrowserController
    }
    // enables right-click selection for contextual menu
    override func menu(for event: NSEvent) -> NSMenu? {
        // Find which row is under the cursor
        window?.makeFirstResponder(self)
        let menuPoint = (convert(event.locationInWindow, from: nil))
        let pointedRow: Int = row(at: menuPoint)
        
        let filebrowserMenu: NSMenu? = menu
        var enabled = true
        /* Update the file tree selection before showing menu
         Preserves the selection if the row under the mouse is selected (to allow for
         multiple items to be selected), otherwise selects the row under the mouse */
        let currentRowIsSelected: Bool = selectedRowIndexes.contains(pointedRow)
        if pointedRow == -1 {
            deselectAll(self)
            enabled = false
            return nil // don't display menu, all entries are disabled anyway
        } else if !currentRowIsSelected {
            let indexSet = NSIndexSet(index: pointedRow)
            selectRowIndexes(indexSet as IndexSet, byExtendingSelection: false)
            enabled = true
        }
        for item in filebrowserMenu!.items {
            item.isEnabled = enabled
        }
        return filebrowserMenu
    }
    
}
