import AppKit

class FileBrowserSegmentedCell: NSSegmentedCell {
    
    required init(coder aDecoder: NSCoder) {
        super.init(coder: aDecoder)
    }
    
    override func draw(withFrame cellFrame: NSRect, in controlView: NSView) {
        var segCount: Int = 0
        while segCount < segmentCount {
            drawSegment(segCount, inFrame: cellFrame, with: controlView)
            segCount += 1
        }
    }
    
    override func drawSegment(_ segment: Int, inFrame frame: NSRect, with controlView: NSView) {
        // Calculate rect for this segment
        let segmentRect: NSRect = rect(forSegment: segment, inFrame: frame)
        var folderShape: NSBezierPath?
        let flapHeight: CGFloat = 2.0
        //
        //  draw foldershape
        //
        let margin: CGFloat = 2.0
        let points = NSPointArray.allocate(capacity: 6)
        points[0] = NSMakePoint(segmentRect.origin.x + margin,
                                segmentRect.origin.y + margin * 2.0)
        
        points[1] = NSMakePoint(segmentRect.origin.x + segmentRect.size.width / 2.5,
                                segmentRect.origin.y + margin * 2.0)
        
        points[2] = NSMakePoint(segmentRect.origin.x + segmentRect.size.width / 2.5 + 3.0,
                                segmentRect.origin.y + flapHeight + margin * 2.0)
        
        points[3] = NSMakePoint(segmentRect.origin.x + segmentRect.size.width - margin,
                                segmentRect.origin.y + flapHeight + margin * 2.0)
        
        points[4] = NSMakePoint(segmentRect.origin.x + segmentRect.size.width - margin,
                                segmentRect.origin.y + segmentRect.size.height - margin)
        
        points[5] = NSMakePoint(segmentRect.origin.x + margin,
                                segmentRect.origin.y + segmentRect.size.height - margin)
        
        folderShape = NSBezierPath()
        folderShape?.appendPoints(points, count: 6)
        folderShape?.close()
        var gradient: NSGradient?
        if isEnabled {
            if selectedSegment == segment {
                gradient = highlightGradient
            } else if selectedSegment != segment {
                gradient = normalGradient
            }
        } else {
            gradient = disabledNormalGradient
        }
        gradient?.draw(in: folderShape!, angle: 90)
        drawInterior(forSegment: segment, withFrame: segmentRect)
    }
    
    func drawInterior(forSegment segment: Int, withFrame rect: NSRect) {
        var newTitle: NSAttributedString?
        if label(forSegment: segment) != nil {
            var textAttributes = [AnyHashable: Any](minimumCapacity: 0)
            textAttributes[NSAttributedString.Key.font] = NSFont.controlContentFont(ofSize: NSFont.systemFontSize(for: controlSize))
            if isEnabled {
                if selectedSegment == segment {
                    textAttributes[NSAttributedString.Key.foregroundColor] = highlightTextColor
                } else {
                    textAttributes[NSAttributedString.Key.foregroundColor] = highlightTextColor
                }
            } else {
                textAttributes[NSAttributedString.Key.foregroundColor] = disabledTextColor
            }
            if label(forSegment: segment) != nil {
                newTitle = NSAttributedString(string: label(forSegment: segment) ?? "", attributes: textAttributes as? [NSAttributedString.Key: Any])
            } else {
                newTitle = NSAttributedString(string: "", attributes: textAttributes as? [NSAttributedString.Key: Any])
            }
        }
        var textRect: NSRect = rect
        // Draw text alone
        textRect.origin.y += rect.size.height / 2 - ((newTitle?.size().height)! / 2.0) + 1.0
        textRect.origin.x += rect.size.width  / 2 - ((newTitle?.size().width)! / 2.0)
        textRect.size.height = (newTitle?.size().height)!
        textRect.size.width  = (newTitle?.size().width)!
        if textRect.origin.x < 1 {
            textRect.origin.x = 1
        }
        newTitle?.draw(in: textRect)
    }

    func rect(forSegment segment: Int, inFrame frame: NSRect) -> NSRect {
        var left: CGFloat = 0
        for i in 0 ..< segmentCount {
            let w = width(forSegment: i)
            if i == segment {
                let off : CGFloat = CGFloat(i) + 0 // Account for separators and border.
                return NSRect(x: left + off, y: frame.origin.y, width: w, height: frame.size.height)
            }
            left += w
        }
        return NSZeroRect
    }
    
    var highlightGradient: NSGradient {
        let cAndL: [(NSColor, CGFloat)] = [(NSColor(deviceRed: 0.31, green: 0.56, blue: 0.69, alpha: noAlphaValue), 0.00),
                                           (NSColor(deviceRed: 0.25, green: 0.43, blue: 0.53, alpha: noAlphaValue), 0.10),
                                           (NSColor(deviceRed: 0.46, green: 0.74, blue: 0.85, alpha: noAlphaValue), 0.15),
                                           (NSColor(deviceRed: 0.31, green: 0.69, blue: 0.85, alpha: noAlphaValue), 0.20),
                                           (NSColor(deviceRed: 0.43, green: 0.73, blue: 0.84, alpha: noAlphaValue), 0.80),
                                           (NSColor(deviceRed: 0.29, green: 0.59, blue: 0.72, alpha: noAlphaValue), 0.90),
                                           (NSColor(deviceRed: 0.15, green: 0.15, blue: 0.15, alpha: alphaValue), 0.95)]
        
        return NSGradient(colorsAndLocations: cAndL[0], cAndL[1], cAndL[2], cAndL[3], cAndL[4], cAndL[5], cAndL[6])!
    }
    
    var normalGradient: NSGradient {
        let cAndL: [(NSColor, CGFloat)] = [(NSColor(deviceRed: 0.35, green: 0.35, blue: 0.35, alpha: alphaValue), 0.00),
                                           (NSColor(deviceRed: 0.30, green: 0.30, blue: 0.30, alpha: alphaValue), 0.10),
                                           (NSColor(deviceRed: 0.41, green: 0.41, blue: 0.41, alpha: alphaValue), 0.15),
                                           (NSColor(deviceRed: 0.36, green: 0.36, blue: 0.36, alpha: alphaValue), 0.20),
                                           (NSColor(deviceRed: 0.38, green: 0.38, blue: 0.38, alpha: alphaValue), 0.80),
                                           (NSColor(deviceRed: 0.33, green: 0.33, blue: 0.33, alpha: alphaValue), 0.90),
                                           (NSColor(deviceRed: 0.15, green: 0.15, blue: 0.15, alpha: alphaValue), 0.95)]
        
        return NSGradient(colorsAndLocations: cAndL[0], cAndL[1], cAndL[2], cAndL[3], cAndL[4], cAndL[5], cAndL[6])!
    }
    
    var disabledNormalGradient: NSGradient {
        return NSGradient(starting: NSColor(deviceRed: 0.251, green: 0.251, blue: 0.255, alpha: disabledAlphaValue),
                          ending: NSColor(deviceRed: 0.118, green: 0.118, blue: 0.118, alpha: disabledAlphaValue))!
    }
    
    var pushedGradient: NSGradient {
        return NSGradient(starting: NSColor(deviceRed: 0.451, green: 0.451, blue: 0.455, alpha: alphaValue),
                          ending: NSColor(deviceRed: 0.318, green: 0.318, blue: 0.318, alpha: alphaValue))!
    }
    
    var noAlphaValue: CGFloat {
        return 1.0
    }
    
    var alphaValue: CGFloat {
        return 0.7
    }
    
    var disabledAlphaValue: CGFloat {
        return 0.2
    }
    
    var strokeColor: CGColor? {
        return (NSColor(deviceRed: 0.749, green: 0.761, blue: 0.788, alpha: 0.7).cgColor)
    }
    
    var textColor: NSColor! {
        return NSColor(deviceRed: 0.8, green: 0.8, blue: 0.8, alpha: alphaValue)
    }
    
    var highlightTextColor: NSColor! {
        return NSColor(deviceRed: 0.0, green: 0.0, blue: 0.0, alpha: alphaValue)
    }
    
    var disabledTextColor: NSColor! {
        return NSColor(deviceRed: 1, green: 1, blue: 1, alpha: disabledAlphaValue)
    }
}

