import AppKit

class FileBrowserStepperCell: NSStepperCell {
    
    var isTopPressed = false
    var isBottomPressed = false
    
    override func draw(withFrame cellFrame: NSRect, in controlView: NSView) {
        drawPlusMinus(in: cellFrame)
        controlView.layer?.backgroundColor = BackgroundColorA.cgColor
    }
    
    func drawPlusMinus(in frame: NSRect) {
        let signSize : CGFloat = 7.0
        let topOffset: CGFloat = 0.0
        let btmOffset: CGFloat = 0.0
        let margin: CGFloat = 0.0
        let topRect: NSRect = NSMakeRect(frame.origin.x,
                                         frame.origin.y + (frame.size.height / 2.0),
                                         frame.size.width,
                                         frame.size.height / 2.0)
        
        let bottomRect: NSRect = NSMakeRect(frame.origin.x,
                                            frame.origin.y,
                                            frame.size.width,
                                            frame.size.height / 2.0)
        //
        //  draw +
        //
        var hLine = NSBezierPath()
        let vLine = NSBezierPath()
        
        let hPoints = NSPointArray.allocate(capacity: 2)
        hPoints[0] = NSMakePoint(topRect.origin.x + margin + (topRect.size.width / 2.0) - (signSize / 2.0),
                                 topRect.origin.y + (topRect.size.height / 2.0) + topOffset)
        
        hPoints[1] = NSMakePoint(topRect.origin.x + margin + (topRect.size.width / 2.0) + (signSize / 2.0),
                                 topRect.origin.y + (topRect.size.height / 2.0) + topOffset)
        
        let vPoints = NSPointArray.allocate(capacity: 2)
        vPoints[0] = NSMakePoint(topRect.origin.x + margin + (topRect.size.width / 2.0),
                                 topRect.origin.y + (topRect.size.height / 2.0) - (signSize / 2.0) + topOffset)
        
        vPoints[1] = NSMakePoint(topRect.origin.x + margin + (topRect.size.width / 2.0),
                                 topRect.origin.y + (topRect.size.height / 2.0) + (signSize / 2.0) + topOffset)
        
        hLine.appendPoints(hPoints, count: 2)
        vLine.appendPoints(vPoints, count: 2)
        MidGrayColor.setFill()
        if isEnabled {
            if isTopPressed {
                pressedFillColor.setFill()
                pressedLineColor.setStroke()
            } else {
                MidGrayColor.setFill()
                lineColor.setStroke()
            }
        } else {
            disabledStrokeColor.setStroke()
        }
        topRect.fill()
        hLine.stroke()
        vLine.stroke()
        //
        //  draw -
        //
        hLine = NSBezierPath()
        hPoints[0] = NSMakePoint(bottomRect.origin.x + margin + (bottomRect.size.width / 2.0) - (signSize / 2.0),
                                 bottomRect.origin.y + (bottomRect.size.height / 2.0) + btmOffset)
        
        hPoints[1] = NSMakePoint(bottomRect.origin.x + margin + (bottomRect.size.width / 2.0) + (signSize / 2.0),
                                 bottomRect.origin.y + (bottomRect.size.height / 2.0) + btmOffset)
        
        hLine.appendPoints(hPoints, count: 2)
        if isEnabled {
            if isBottomPressed {
                pressedFillColor.setFill()
                pressedLineColor.setStroke()
            } else {
                MidGrayColor.setFill()
                lineColor.setStroke()
            }
        } else {
            disabledStrokeColor.setStroke()
        }
        bottomRect.fill()
        hLine.stroke()
    }
    
    override func startTracking(at startPoint: NSPoint, in controlView: NSView) -> Bool {
        let frame:      NSRect = controlView.bounds
        let seperator: CGFloat = frame.size.height / 2.0
        let bottomRect: NSRect = NSMakeRect(frame.origin.x, frame.origin.y, frame.size.width, seperator)
        let topRect:    NSRect = NSMakeRect(frame.origin.x, frame.origin.y + seperator, frame.size.width, frame.size.height - seperator)
        if NSPointInRect(startPoint, topRect) {
            isTopPressed = true
        }
        if NSPointInRect(startPoint, bottomRect) {
            isBottomPressed = true
        }
        return super.startTracking(at: startPoint, in: controlView)
    }
    
    override func stopTracking(last lastPoint: NSPoint, current stopPoint: NSPoint, in controlView: NSView, mouseIsUp flag: Bool) {
        //
        //  mouse button is released
        //
        isTopPressed = false
        isBottomPressed = false        
        super.stopTracking(last: lastPoint, current: stopPoint, in: controlView, mouseIsUp: flag)
    }
    
    var lineColor: NSColor {
        return TextColorInactive
    }
    
    var pressedLineColor: NSColor {
        return BackgroundColorA
    }
    
    var disabledStrokeColor: NSColor {
        return NSColor(deviceRed: 0.749, green: 0.761, blue: 0.788, alpha: disabledAlphaValue())
    }
    
    var pressedFillColor : NSColor {
        return MidGrayColor.blended(withFraction: 0.3, of: .darkGray)!
    }
    
    var alphaValue: CGFloat {
        return 0.7
    }
    
    func disabledAlphaValue() -> CGFloat {
        return 0.2
    }
}
