//
//  ContainerItem.swift
//  Phonix
//
//  Created by Sander Borst on 04/05/2019.
//  Copyright © 2019 VS. All rights reserved.
//

import Foundation

struct MediaItem {
    var urlString   : String?
    var title       : String?
    var length      : Int?
    
    static func parseM3U(contents: String) -> [MediaItem]? {
        var mediaItems = [MediaItem]()
        contents.enumerateLines(invoking: { line, stop in
            if line.contains("#EXTM3U") {
                // skip this line
            } else if line.hasPrefix("#EXTINF:") {
                let infoLine    = line.replacingOccurrences(of: "#EXTINF:", with: "")
                let infos       = Array(infoLine.components(separatedBy: ","))
                if let lengthString = infos.first, let length = Int(lengthString) {
                    let mediaItem   = MediaItem(urlString: nil, title: infos.last, length: length)
                    mediaItems.append(mediaItem)
                }
            } else {
                if mediaItems.count > 0 {
                    let index = mediaItems.count - 1
                    mediaItems[index].urlString = line
                }
            }
        })
        return mediaItems
    }
    
    static func parsePLS(contents: String) -> [MediaItem]? {
        var mediaItems = [MediaItem]()
        contents.enumerateLines(invoking: { line, stop in
            var item = MediaItem()
            
            if line.hasPrefix("File") {
                //
                // [playlist]\n
                // NumberOfEntries=1\n
                // File1=http://streams.cityfm.nl:8043/\n
                //
                item.urlString = line.components(separatedBy: "=")[1]
                mediaItems.append(item)
            } else if line.hasPrefix("Title") {
                let item = mediaItems.count - 1
                if mediaItems[item].urlString != nil { mediaItems[item].title = line.components(separatedBy: "=")[1] }
            } else if line.hasPrefix("Length") {
                let item = mediaItems.count - 1
                if mediaItems[item].urlString != nil { mediaItems[item].length = Int(line.components(separatedBy: "=")[1]) }
            }
        })
        return mediaItems
    }
    
    static func parseASX(contents: String) -> [MediaItem]? {
        var mediaItems = [MediaItem]()
        let content = contents.replacingOccurrences(of: "\t", with: "")
        content.enumerateLines(invoking: { line, stop in
            //print("CI parseASX line:", line)
            if line.lowercased().contains("<ref href") {
                
                let parts = line.components(separatedBy: "=")
                var url = parts[1].replacingOccurrences(of: " \"", with: "").replacingOccurrences(of: "\"", with: "")
                url = url.replacingOccurrences(of: "</entry>", with: "")
                    .replacingOccurrences(of: " />", with: "")
                    .replacingOccurrences(of: "/>", with: "")
                //print("CI parseASX url:", url)
                let mediaItem = MediaItem(urlString: url, title: nil, length: nil)
                mediaItems.append(mediaItem)
            }
        })
        return mediaItems
    }
}

