
class CueSheet: NSObject {
    var tracks      : [CueSheetTrack] = []
    var refFileName : URL?
    
    func cueSheetWithFile(file: URL) -> CueSheet
    {
        return CueSheet(cueFile: file)
    }

    convenience init(cueFile: URL)
    {
        self.init()
        parseCue(file: cueFile)
    }

    func cueSheet(file: URL?) -> Any?
    {
        return CueSheet(cueFile: file!)
    }

    func urlForRef(fileName: NSString, reletiveTo: URL) -> URL
    {
        //
        // the purpose of this function:
        // replace last path part               : Massive%20Attack%20-%20Heligoland.cue#9
        // with fileName of the reference file  : Massive%20Attack%20-%20Heligoland.wav
        // file: file:///Users/you/Music/2010%20-%20Heligoland/Massive%20Attack%20-%20Heligoland.cue#9
        // base: file:///Users/you/Music/2010%20-%20Heligoland/Massive%20Attack%20-%20Heligoland.wav
        //
        var refURL: URL = reletiveTo
        refURL.deleteLastPathComponent()
        refURL = URL(fileURLWithPath:refURL.relativePath.appending("/").appending(fileName as String))
        return refURL
    }

    func parseCue(file: URL) {
        var contents: String?
        do {
            contents = try String(contentsOf: file, encoding: .utf8)
        } catch {
//            print("CUS_CUE File is not readable as utf8: \(error)")
        }
        if contents == nil {
            do {
                contents = try String(contentsOf: file, encoding: .windowsCP1251)
            } catch {
//                print("CUS_CUE File is not readable as windowsCP1251: \(error)")
            }
        }
        if contents == nil {
            do {
                contents = try String(contentsOf: file, encoding: .isoLatin1)
            } catch {
//                print("CUS_CUE File is not readable as isoLatin1: \(error)")
            }
        }
        if contents == nil {
            return
        }
        let entries:    NSMutableArray = []
        var track:      Int?
        var path:       NSString?
        var artist:     NSString?
        var album:      NSString?
        var title:      NSString?
        var genre:      NSString?
        var year:       NSString?
        var begin:      Double = 0
        let end:        Double = 0
        var trackAdded = false
        let whitespace = CharacterSet.whitespacesAndNewlines
        var scanner: Scanner?
        
        for line in contents!.components(separatedBy: "\n") {
            scanner = Scanner(string: line)
            var command: NSString?
            if !scanner!.scanUpToCharacters(from: whitespace, into: &command) {
                continue
            }
            if command == "FILE" {
                trackAdded = false
                if !scanner!.scanString("\"", into: nil) {
                    continue
                }
                // Read in the path of refFile
                if !scanner!.scanUpTo("\"", into: &path) {
                    continue
                }
            } else if command == "TRACK" {
                trackAdded = false
                var trackStr : NSString?
                if !scanner!.scanUpToCharacters(from: whitespace, into: &trackStr) {
                    continue
                }
                else {
                    track = trackStr?.integerValue
                }
                var type: NSString?
                if !scanner!.scanUpToCharacters(from: whitespace, into: &type) || !(type == "AUDIO") {
                    continue
                }
            } else if command == "INDEX" {
                if trackAdded {
                    continue
                }
                if !(path != nil) {
                    continue
                }
                var index: NSString?
                if !scanner!.scanUpToCharacters(from: whitespace, into: &index) || index?.intValue != 1 {
                    continue
                }
                scanner!.scanCharacters(from: whitespace, into: nil)
                var time: NSString?
                if !scanner!.scanUpToCharacters(from: whitespace, into: &time) {
                    continue
                }
                let msf: NSArray = time!.components(separatedBy: ":") as NSArray
                if msf.count != 3 {
                    continue
                }
                let pos0 = (msf.object(at: 0) as! NSString).doubleValue
                let pos1 = (msf.object(at: 1) as! NSString).doubleValue
                let pos2 = (msf.object(at: 2) as! NSString).doubleValue
                begin = (60 * pos0) + pos1 + (pos2 / 75)
                if track == nil {
                    track = 1
                }
                // Need to add basePath, and convert to URL
                let url: URL = urlForRef(fileName: path!, reletiveTo: file)
                entries.add(CueSheetTrack(url   : url,
                                          track : track,
                                          begin : begin,
                                          end   : end,
                                          artist: artist,
                                          album : album,
                                          title : title,
                                          genre : genre,
                                          year  : year))
                //
                // can we add end to previous object?
                //
                // this works for al track but the last one, we don't know totaltime
                // or do we? we could also make an exeption, if( end == 0.0)  end = totalTime
                //
                if entries.count > 1 {
                    ((entries.object(at: entries.count - 2)) as! CueSheetTrack).setEnd(begin)
                }
                //
                trackAdded = true
            } else if command == "PERFORMER" {
                if !scanner!.scanString("\"", into: nil) {
                    continue
                }
                // Read in the path
                if !scanner!.scanUpTo("\"", into: &artist) {
                    continue
                }
            } else if command == "TITLE" {
                var titleDest: NSString?
                if !scanner!.scanString("\"", into: nil) {
                    continue
                }
                // Read in the path
                if !scanner!.scanUpTo("\"", into: &titleDest) {
                    continue
                }
                if !(path != nil) {
                    album = titleDest
                } else {
                    title = titleDest
                }
            } else if command == "REM" {
                var type: NSString?
                if !scanner!.scanUpToCharacters(from: whitespace, into: &type) {
                    continue
                }
                if type == "GENRE" {
                    if scanner!.scanString("\"", into: nil) {
                        if !scanner!.scanUpTo("\"", into: &genre) {
                            continue
                        }
                    } else {
                        if !scanner!.scanUpToCharacters(from: whitespace, into: &genre) {
                            continue
                        }
                    }
                } else if type == "DATE" {
                    if !scanner!.scanUpToCharacters(from: whitespace, into: &year) {
                        continue
                    }
                }
            }
        }
        tracks = entries.copy() as! [CueSheetTrack]
        //dump(tracks,  name: "CUS_entries: ")
    }
}
    
class CueSheetTrack: NSObject {
    var url     : URL!
    var track   : Int?
    var artist  : NSString?
    var album   : NSString?
    var title   : NSString?
    var genre   : NSString?
    var year    : NSString?
    var begin   : Double? //Starting time for the track
    var end     : Double?
    
    class func trackWithURL(url     u: URL?,
                            track   t: Int?,
                            begin   s: Double,
                            end     e: Double,
                            artist  a: NSString?,
                            album   b: NSString?,
                            title   l: NSString?,
                            genre   g: NSString?,
                            year    y: NSString?) -> CueSheetTrack { return CueSheetTrack(url       :u,
                                                                                          track     :t,
                                                                                          begin     :s,
                                                                                          end       :e,
                                                                                          artist    :a,
                                                                                          album     :b,
                                                                                          title     :l,
                                                                                          genre     :g,
                                                                                          year      :y )
    }
    
    init(url    u: URL?,
         track  t: Int?,
         begin  s: Double,
         end    e: Double,
         artist a: NSString?,
         album  b: NSString?,
         title  l: NSString?,
         genre  g: NSString?,
         year   y: NSString?)
    {
        super.init()
        
        if t != nil {
            track = t
        }
        if u!.absoluteString != "" {
            url = u!//.copy() as? NSURL
        }
        if l != "" {
            title = l?.copy() as? NSString
        }
        if a != "" {
            artist = a?.copy() as? NSString
        }
        if b != "" {
            album = b?.copy() as? NSString
        }
        if g != "" {
            genre = g?.copy() as? NSString
        }
        if y != "" {
            year = y?.copy() as? NSString
        }
        begin = s
        end = e
    }

    func setEnd(_ endCue: Double) { end = endCue }
}
