import AppKit

var bookmarks = [URL: Data]()

func bookmarkPath() -> String
{
    var url = PhonixPrefsFolder
    url = url.appendingPathComponent("PhonixPlaylist.dict")
    return url.path
}

func loadBookmarks()
{
    let path = bookmarkPath()
    do {
        if let data = try NSKeyedUnarchiver.unarchiveTopLevelObjectWithData(path.data(using: String.Encoding.utf8)!) {
            bookmarks = data as! [URL : Data]
            for bookmark in bookmarks
            {
                restoreBookmark(bookmark)
            }
        }
    } catch {
        //         print("Couldn't read file.")
    }    
}

func saveBookmarks()
{
    let path = bookmarkPath()
    do {
        let data = try NSKeyedArchiver.archivedData(withRootObject: bookmarks, requiringSecureCoding: false)
        try data.write(to: URL(fileURLWithPath: path))
    } catch {
        //print("Couldn't write file")
    }
}

func storeBookmark(url: URL)
{
    do
    {
        let data = try url.bookmarkData(options: NSURL.BookmarkCreationOptions.withSecurityScope, includingResourceValuesForKeys: nil, relativeTo: nil)
        bookmarks[url] = data
    }
    catch
    {
        //        Swift.print ("Error storing bookmarks")
    }    
}

func restoreBookmark(_ bookmark: (key: URL, value: Data))
{
    let resourceAvailable = (try? bookmark.key.checkResourceIsReachable()) ?? false
    if resourceAvailable {
        let restoredUrl: URL?
        var isStale = false
        //    Swift.print ("Restoring \(bookmark.key)")
        do
        {
            restoredUrl = try URL.init(resolvingBookmarkData: bookmark.value, options: NSURL.BookmarkResolutionOptions.withSecurityScope, relativeTo: nil, bookmarkDataIsStale: &isStale)
        }
        catch
        {
            //        Swift.print ("Error restoring bookmarks")
            restoredUrl = nil
        }
        if let url = restoredUrl
        {
            if isStale
            {
                //            Swift.print ("URL is stale")
            }
            else
            {
                if !url.startAccessingSecurityScopedResource()
                {
                    //                Swift.print ("Couldn't access: \(url.path)")
                }
            }
        }
    }
}

func resolveFinderAlias(at url: URL) -> URL? {
    do {
        let resourceValues = try url.resourceValues(forKeys: [.isAliasFileKey])
        if resourceValues.isAliasFile! {
            let original = try URL(resolvingAliasFileAt: url)
            return original
        }
    } catch  {
        print(error)
    }
    return nil
}

let PhonixFileManager = FileManager.default

var userHomeDirectoryPath : String {
    //
    // not used
    //
    let pw = getpwuid(getuid())
    let home = pw?.pointee.pw_dir
    let homePath = FileManager.default.string(withFileSystemRepresentation: home!, length: Int(strlen(home!)))
    return homePath
}

var HomeFolder : URL {
    let homeFolder = FileManager.default.homeDirectoryForCurrentUser
    return homeFolder
}

var MusicFolder : URL {
    let pw = getpwuid(getuid());
    let home = pw?.pointee.pw_dir
    let homePath = FileManager.default.string(withFileSystemRepresentation: home!, length: Int(strlen(home!)))
    let musicPath = (homePath as NSString).appendingPathComponent("Music")
    return URL(fileURLWithPath: musicPath)
}

var PhonixFolder : URL {
    let phonixFolder = MusicFolder.appendingPathComponent("Phonix")
    return phonixFolder
}

var PhonixTabsFolder : URL {
    let tabsFolder = PhonixFolder.appendingPathComponent("Tabs")
    return tabsFolder
}

var PhonixRecordingFolder : URL {
    let recFolder = PhonixFolder.appendingPathComponent("Recordings")
    return recFolder
}

var PhonixPrefsFolder : URL {
    let prefsFolder = HomeFolder.appendingPathComponent("Library/Preferences/Phonix")
    return prefsFolder
}

func createPhonixFolders() {
    //
    // create /Phonix
    //
    if !PhonixFileManager.fileExists(atPath: PhonixFolder.path) {
        do {
            try PhonixFileManager.createDirectory(atPath: PhonixFolder.path, withIntermediateDirectories: false, attributes: nil)
        } catch {
            //            print("PhonixFileManager: ERROR: unable to create:", PhonixFolder.path)
        }
    }
    //
    // create /Phonix/Tabs
    //
    if !PhonixFileManager.fileExists(atPath: PhonixTabsFolder.path) {
        do {
            try PhonixFileManager.createDirectory(atPath: PhonixTabsFolder.path, withIntermediateDirectories: false, attributes: nil)
        } catch {
            //            print("PhonixFileManager: ERROR: unable to create:", PhonixTabsFolder.path)
        }
    }
    //
    // create /Phonix/Recordings
    //
    if !PhonixFileManager.fileExists(atPath: PhonixRecordingFolder.path) {
        do {
            try PhonixFileManager.createDirectory(atPath: PhonixRecordingFolder.path, withIntermediateDirectories: false, attributes: nil)
        } catch {
            //            print("PhonixFileManager: ERROR: unable to create:", PhonixRecordingFolder.path)
        }
    }
    //
    // create Preferences/Phonix
    //
    if !PhonixFileManager.fileExists(atPath: PhonixPrefsFolder.path) {
        do {
            try PhonixFileManager.createDirectory(atPath: PhonixPrefsFolder.path, withIntermediateDirectories: false, attributes: nil)
        } catch {
            //            print("PhonixFileManager: ERROR: unable to create:", PhonixPrefsFolder.path)
        }
    }
}

func removeFiles(_ regex: NSRegularExpression, inPath path: NSString) {
    let filesEnumerator = PhonixFileManager.enumerator(atPath: path as String)
    while let file: NSString = filesEnumerator?.nextObject() as? NSString {
        let match = regex.numberOfMatches(in: file as String, options: [], range: NSMakeRange(0, file.length))
        
        if match > 0 {
            try! PhonixFileManager.removeItem(atPath: path.appendingPathComponent(file as String))
        }
    }
}

func initDefaults() {
    let userDefaults = UserDefaults.standard
    
    for index in 0..<6 {
        if userDefaults.value(forKey: "fileBrowserRoot\(index)") == nil {
//            do {
//                let bookmark = try (MusicFolder as NSURL).bookmarkData(options: [.securityScopeAllowOnlyReadAccess, .withSecurityScope], includingResourceValuesForKeys: nil, relativeTo: nil)
         
                UserDefaults.standard.set("~/Music/", forKey: "fileBrowserRoot\(index)")
            }
//            catch
//            {
//            }
//        }
    }
    
    if userDefaults.value(forKey: "numberOfRootOptions") == nil {
        userDefaults.set(2, forKey: "numberOfRootOptions") }
    
    if userDefaults.value(forKey: "FileBrowserDoubleClick") == nil {
        userDefaults.set(true, forKey: "FileBrowserDoubleClick") }
    
    if userDefaults.value(forKey: "SideViewDividerPosition") == nil {
        userDefaults.set(256.0, forKey: "SideViewDividerPosition") }
    if userDefaults.value(forKey: "ShowSideView") == nil {
        userDefaults.set(true, forKey: "ShowSideView") }
    if userDefaults.value(forKey: "showAlbumArt") == nil {
        userDefaults.set(true, forKey: "showAlbumArt") }
    // mediaDetails is deprecated
    if userDefaults.value(forKey: "showMediaDetails") != nil {
        userDefaults.removeObject(forKey: "showMediaDetails") }
    
    if userDefaults.value(forKey: "ShowMenuBarButtons") == nil {
        userDefaults.set(true, forKey: "ShowMenuBarButtons") }
    if userDefaults.value(forKey: "ShowMenuBarLeveldB") == nil {
        userDefaults.set(false, forKey: "ShowMenuBarLeveldB") }
    
    if userDefaults.value(forKey: "volume") == nil {
        userDefaults.set(0.5, forKey: "volume") }
    if userDefaults.value(forKey: "repeat") == nil {
        userDefaults.set(0, forKey: "repeat") }
    if userDefaults.value(forKey: "shuffle") == nil {
        userDefaults.set(0, forKey: "shuffle") }
    
    if userDefaults.value(forKey: "readCueSheetsInFolders") == nil {
        userDefaults.set(true, forKey: "readCueSheetsInFolders") }
    if userDefaults.value(forKey: "drawAlbumSeparator") == nil {
        userDefaults.set(true, forKey: "drawAlbumSeparator") }
    
    if userDefaults.value(forKey: "fontSize") == nil {
        userDefaults.set(12.0, forKey: "fontSize") }
    
    if userDefaults.value(forKey: "kMeterOn") == nil {
        userDefaults.set(false, forKey: "kMeterOn") }
    if userDefaults.value(forKey: "levelMeterEnabled") == nil {
        userDefaults.set(false, forKey: "levelMeterEnabled") }
    if userDefaults.value(forKey: "spectrumAnalyzerEnabled") == nil {
        userDefaults.set(false, forKey: "spectrumAnalyzerEnabled") }
    if userDefaults.value(forKey: "waveFormPositionSlider") == nil {
        userDefaults.set(true, forKey: "waveFormPositionSlider") }
    
    if userDefaults.value(forKey: "recordFormat") == nil {
        userDefaults.set("wav", forKey: "recordFormat") }
    if userDefaults.value(forKey: "gaplessplayback") == nil {
        userDefaults.set(true, forKey: "gaplessplayback") }
    
    if userDefaults.value(forKey: "DisplayHours") == nil {
        userDefaults.set(true, forKey: "DisplayHours") }
    if userDefaults.value(forKey: "BypassFX") == nil {
        userDefaults.set(true, forKey: "BypassFX") }
    
    if userDefaults.value(forKey: "fixedSampleRate") == nil {
        userDefaults.set(44100.0, forKey: "fixedSampleRate") }
    if userDefaults.value(forKey: "sampleRateFixed") == nil {
        userDefaults.set(true, forKey: "sampleRateFixed") }
    
    if userDefaults.value(forKey: "useAUeffects") == nil {
        userDefaults.set(false, forKey: "useAUeffects") }
    
    if userDefaults.value(forKey: "autoplay") == nil {
        userDefaults.set(false, forKey: "autoplay") }
    if userDefaults.value(forKey: "selectedTabIndex") == nil {
        userDefaults.set(0, forKey: "selectedTabIndex") }
    if userDefaults.value(forKey: "lastTrackPlaying") == nil {
        userDefaults.set(-1, forKey: "lastTrackPlaying") }
    if userDefaults.value(forKey: "lastTrackPosition") == nil {
        userDefaults.set(0.0, forKey: "lastTrackPosition") }
    
    if userDefaults.value(forKey: "lastPlaybackStatus") == nil {
        userDefaults.set(PhonixStatus.phonixStopped.rawValue, forKey: "lastPlaybackStatus") }
    
    if userDefaults.value(forKey: "numberOfRootOptions") == nil {
        userDefaults.set(2, forKey: "numberOfRootOptions") }
    //
    // deprecated defaults: should be removed
    //
    // fileBrowserRootURL (non bookmarked)
    for index in 0..<6 {
        if userDefaults.object(forKey: "fileBrowserRootURL\(index)") != nil {
            userDefaults.removeObject(forKey: "fileBrowserRootURL\(index)")
        }
    }
    // iTunesArtWork
    if userDefaults.value(forKey: "iTunesArtwork") != nil {
        userDefaults.removeObject(forKey: "iTunesArtwork") }
    // remote control
    if userDefaults.value(forKey: "remoteControl") != nil {
        userDefaults.removeObject(forKey: "remoteControl") }
    // kMeterPosition should be one entry, convert and remove .x and .y
    if userDefaults.value(forKey: "kMeter.origin.x") != nil || userDefaults.value(forKey: "kMeter.origin.y") != nil {
        var origin = CGPoint()
        origin.x = userDefaults.value(forKey: "kMeter.origin.x") as! CGFloat
        origin.y = userDefaults.value(forKey: "kMeter.origin.y") as! CGFloat
        userDefaults.removeObject(forKey: "kMeter.origin.x")
        userDefaults.removeObject(forKey: "kMeter.origin.y")
        
        userDefaults.set([origin.x, origin.y], forKey: "kMeterPosition")
    }
    if userDefaults.value(forKey: "openingFilesBehavior") != nil {
        userDefaults.removeObject(forKey: "openingFilesBehavior") }
    if userDefaults.value(forKey: "openingFilesAlteredBehavior") != nil {
        userDefaults.removeObject(forKey: "openingFilesAlteredBehavior") }
    
    if userDefaults.value(forKey: "MediaKeys") == nil {
        userDefaults.set(false,forKey: "MediaKeys") }
    
    userDefaults.synchronize()
}

var timeDisplayFont : NSFont?

func registerFont() {
    guard let fontData = NSDataAsset(name: "OratorStd")?.data else { return }
    let dataProvider = CGDataProvider(data: fontData as CFData)
    let cgFont = CGFont(dataProvider!)
    var error: Unmanaged<CFError>?
    if !CTFontManagerRegisterGraphicsFont(cgFont!, &error)
    {
        //        print("Error loading Font!")
    } else {
        timeDisplayFont = NSFont(name: String(cgFont!.postScriptName!), size: 12.0)
    }
}

func directoryExistsAtPath(_ path: String) -> Bool {
    var isDirectory = ObjCBool(true)
    let exists = PhonixFileManager.fileExists(atPath: path, isDirectory: &isDirectory)
    return exists && isDirectory.boolValue
}

func fileHandleForWritingAtPath(path: String, create: Bool) -> FileHandle {
    if !PhonixFileManager.fileExists(atPath: path) {
        PhonixFileManager.createFile(atPath: path, contents: nil, attributes: nil)
    }
    return FileHandle(forWritingAtPath: path)!
}

func relativePath(from filename: String, to entryURL: URL) -> String {
    let basePath: String = URL(fileURLWithPath: URL(fileURLWithPath: filename).standardized.absoluteString).deletingLastPathComponent().absoluteString + ("/")
    if entryURL.isFileURL {
        // We want relative paths.
        var entryPath: String = URL(fileURLWithPath: entryURL.path).standardized.absoluteString
        if let subRange = Range<String.Index>(NSRange(location: 0, length: entryPath.count), in: entryPath) { entryPath = entryPath.replacingOccurrences(of: basePath, with: "", options: [String.CompareOptions.anchored, String.CompareOptions.literal, String.CompareOptions.caseInsensitive], range: subRange)
        }
        if entryURL.fragment != nil {
            entryPath += "#"
            entryPath += entryURL.fragment ?? ""
        }
        return entryPath
    } else {
        // Write [entryURL absoluteString] to file
        return entryURL.absoluteString
    }
}

func allFileURLs(atPath path: String) -> [URL]?
{
    let manager = FileManager.default
    var urls = [URL]()
    let subpaths = manager.subpaths(atPath: path)
    
    for subpath: String in subpaths!
    {
        let absoluteSubpath = NSString.path(withComponents: [path, subpath])
        // Only let directories be Set as Root
        var isDir: ObjCBool = ObjCBool(false)
        //
        // we should do this with a quicker method, see fileItem.swift
        //
        let exists: Bool = PhonixFileManager.fileExists(atPath: absoluteSubpath, isDirectory: &isDir)
        
        if exists && isDir.boolValue
        {
            // Exists. Directory.
        }
        else if exists && isDir.boolValue == false
        {
            if URL(fileURLWithPath: absoluteSubpath).pathExtension.caseInsensitiveCompare("cue") != .orderedSame
                || UserDefaults.standard.bool(forKey: "readCueSheetsInFolders")
            {
                urls.append(URL(fileURLWithPath: absoluteSubpath))
            }
        }
    }
    return urls
}

func getNextIndex(ofFileWith prefix: String, at pathURL: URL) -> UInt {
    var nextIndex: UInt = 0
    var fileName: String
    let fileList = try! PhonixFileManager.contentsOfDirectory(at: pathURL,
                                                              includingPropertiesForKeys: [],
                                                              options: [.skipsHiddenFiles, .skipsPackageDescendants, .skipsPackageDescendants])
    
    for url in fileList
    {
        if recordingFileTypes().contains(url.pathExtension)
        {
            fileName = url.deletingPathExtension().lastPathComponent
            
            if      fileName.contains(prefix)
                &&  fileName.lengthOfBytes(using: .utf8) == prefix.lengthOfBytes(using: .utf8) + 4  {
                //
                // extract index, last 3 chars
                //
                let indexStr = fileName.suffix(3)
                let index = (indexStr as NSString).intValue
                nextIndex = max(UInt(index), nextIndex)
            }
        }
    }
    nextIndex += 1
    
    return nextIndex
}

func cacheKmeter() {    
    let cacheDirectory = PhonixFileManager.urls(for: .cachesDirectory, in: .userDomainMask)[0]
    let kmeterBinary = cacheDirectory.appendingPathComponent("K-Meter.component/Contents/MacOS/K-Meter")
    if !PhonixFileManager.fileExists(atPath: kmeterBinary.path) {
        
        let folder1 = cacheDirectory.appendingPathComponent("K-Meter.component/Contents/MacOS")
        let folder2 = cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/include/dtd")
        let folder3a = cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/backgrounds/itu")
        let folder3b = cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/backgrounds/rms")
        let folder4 = cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/buttons")
        let folder5 = cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/labels")
        let folder6 = cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/meters")
        do
        {
            try PhonixFileManager.createDirectory(atPath: folder1.path, withIntermediateDirectories: true, attributes: nil)
        }
        catch let error as NSError
        {
            NSLog("Unable to create directory \(error.debugDescription)")
        }
        do
        {
            try PhonixFileManager.createDirectory(atPath: folder2.path, withIntermediateDirectories: true, attributes: nil)
        }
        catch let error as NSError
        {
            NSLog("Unable to create directory \(error.debugDescription)")
        }
        do
        {
            try PhonixFileManager.createDirectory(atPath: folder3a.path, withIntermediateDirectories: true, attributes: nil)
        }
        catch let error as NSError
        {
            NSLog("Unable to create directory \(error.debugDescription)")
        }
        do
        {
            try PhonixFileManager.createDirectory(atPath: folder3b.path, withIntermediateDirectories: true, attributes: nil)
        }
        catch let error as NSError
        {
            NSLog("Unable to create directory \(error.debugDescription)")
        }
        do
        {
            try PhonixFileManager.createDirectory(atPath: folder4.path, withIntermediateDirectories: true, attributes: nil)
        }
        catch let error as NSError
        {
            NSLog("Unable to create directory \(error.debugDescription)")
        }
        do
        {
            try PhonixFileManager.createDirectory(atPath: folder5.path, withIntermediateDirectories: true, attributes: nil)
        }
        catch let error as NSError
        {
            NSLog("Unable to create directory \(error.debugDescription)")
        }
        do
        {
            try PhonixFileManager.createDirectory(atPath: folder6.path, withIntermediateDirectories: true, attributes: nil)
        }
        catch let error as NSError
        {
            NSLog("Unable to create directory \(error.debugDescription)")
        }
        
        let infoplist = NSDataAsset(name: "Contents/Info")?.data
        PhonixFileManager.createFile(atPath: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Info.plist").path, contents: infoplist, attributes: nil)
        
        let pkginfo = NSDataAsset(name: "Contents/PkgInfo")?.data
        PhonixFileManager.createFile(atPath: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/PkgInfo").path, contents: pkginfo, attributes: nil)
        
        let kmeterbinary = NSDataAsset(name: "Contents/MacOS/K-Meter")?.data
        PhonixFileManager.createFile(atPath: kmeterBinary.path, contents: kmeterbinary, attributes: nil)
        
        let defaultskinini = NSDataAsset(name: "Contents/Resources/kmeter/skins/default_skin")?.data
        PhonixFileManager.createFile(atPath: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/default_skin.ini").path, contents: defaultskinini, attributes: nil)
        
        let defaultskin = NSDataAsset(name: "Contents/Resources/kmeter/skins/skin")?.data
        PhonixFileManager.createFile(atPath: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default.skin").path, contents: defaultskin, attributes: nil)
        
        let dtd = NSDataAsset(name: "Contents/Resources/kmeter/skins/include/dtd/kmeter-skin")?.data
        PhonixFileManager.createFile(atPath: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/include/dtd/kmeter-skin.dtd").path, contents: dtd, attributes: nil)
        
        let license = NSDataAsset(name: "Contents/Resources/kmeter/skins/Default/LICENSE")?.data
        PhonixFileManager.createFile(atPath: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/LICENSE").path, contents: license, attributes: nil)
        
        // backgrounds/itu
        let ituimage1 = NSImage(named: "Contents/Resources/kmeter/skins/Default/background_itu_stereo_no_peaks")
        if ituimage1!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/backgrounds/itu/background_itu_stereo_no_peaks.png"))  {}
        
        let ituimage2 = NSImage(named: "Contents/Resources/kmeter/skins/Default/background_itu_stereo_peaks")
        if ituimage2!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/backgrounds/itu/background_itu_stereo_peaks.png"))  {}
        
        let ituimage3 = NSImage(named: "Contents/Resources/kmeter/skins/Default/background_itu_surround_no_peaks")
        if ituimage3!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/backgrounds/itu/background_itu_surround_no_peaks.png"))  {}
        
        let ituimage4 = NSImage(named: "Contents/Resources/kmeter/skins/Default/background_itu_surround_peaks")
        if ituimage4!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/backgrounds/itu/background_itu_surround_peaks.png"))  {}
        
        let ituimage5 = NSImage(named: "Contents/Resources/kmeter/skins/Default/graduation_itu_k12_expanded")
        if ituimage5!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/backgrounds/itu/graduation_itu_k12_expanded.png"))  {}
        
        let ituimage6 = NSImage(named: "Contents/Resources/kmeter/skins/Default/graduation_itu_k12")
        if ituimage6!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/backgrounds/itu/graduation_itu_k12.png"))  {}
        
        let ituimage7 = NSImage(named: "Contents/Resources/kmeter/skins/Default/graduation_itu_k14_expanded")
        if ituimage7!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/backgrounds/itu/graduation_itu_k14_expanded.png"))  {}
        
        let ituimage8 = NSImage(named: "Contents/Resources/kmeter/skins/Default/graduation_itu_k14")
        if ituimage8!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/backgrounds/itu/graduation_itu_k14.png"))  {}
        
        let ituimage9 = NSImage(named: "Contents/Resources/kmeter/skins/Default/graduation_itu_k20_expanded")
        if ituimage9!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/backgrounds/itu/graduation_itu_k20_expanded.png")) {}
        
        let ituimage10 = NSImage(named: "Contents/Resources/kmeter/skins/Default/graduation_itu_k20")
        if ituimage10!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/backgrounds/itu/graduation_itu_k20.png")) {}
        
        let ituimage11 = NSImage(named: "Contents/Resources/kmeter/skins/Default/graduation_itu_normal_expanded")
        if ituimage11!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/backgrounds/itu/graduation_itu_normal_expanded.png")) {}
        
        let ituimage12 = NSImage(named: "Contents/Resources/kmeter/skins/Default/graduation_itu_normal")
        if ituimage12!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/backgrounds/itu/graduation_itu_normal.png")) {}
        
        // backgrounds/rms
        let rmsimage1 = NSImage(named: "Contents/Resources/kmeter/skins/Default/background_rms_stereo_no_peaks")
        if rmsimage1!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/backgrounds/rms/background_rms_stereo_no_peaks.png")) {}
        
        let rmsimage2 = NSImage(named: "Contents/Resources/kmeter/skins/Default/background_rms_stereo_peaks")
        if rmsimage2!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/backgrounds/rms/background_rms_stereo_peaks.png")) {}
        
        let rmsimage3 = NSImage(named: "Contents/Resources/kmeter/skins/Default/background_rms_surround_no_peaks")
        if rmsimage3!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/backgrounds/rms/background_rms_surround_no_peaks.png")) {}
        
        let rmsimage4 = NSImage(named: "Contents/Resources/kmeter/skins/Default/background_rms_surround_peaks")
        if rmsimage4!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/backgrounds/rms/background_rms_surround_peaks.png")) {}
        
        let rmsimage5 = NSImage(named: "Contents/Resources/kmeter/skins/Default/graduation_rms_k12_expanded")
        if rmsimage5!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/backgrounds/rms/graduation_rms_k12_expanded.png")) {}
        
        let rmsimage6 = NSImage(named: "Contents/Resources/kmeter/skins/Default/graduation_rms_k12")
        if rmsimage6!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/backgrounds/rms/graduation_rms_k12.png")) {}
        
        let rmsimage7 = NSImage(named: "Contents/Resources/kmeter/skins/Default/graduation_rms_k14_expanded")
        if rmsimage7!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/backgrounds/rms/graduation_rms_k14_expanded.png")) {}
        
        let rmsimage8 = NSImage(named: "Contents/Resources/kmeter/skins/Default/graduation_rms_k14")
        if rmsimage8!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/backgrounds/rms/graduation_rms_k14.png")) {}
        
        let rmsimage9 = NSImage(named: "Contents/Resources/kmeter/skins/Default/graduation_rms_k20_expanded")
        if rmsimage9!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/backgrounds/rms/graduation_rms_k20_expanded.png")) {}
        
        let rmsimage10 = NSImage(named: "Contents/Resources/kmeter/skins/Default/graduation_rms_k20")
        if rmsimage10!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/backgrounds/rms/graduation_rms_k20.png")) {}
        
        let rmsimage11 = NSImage(named: "Contents/Resources/kmeter/skins/Default/graduation_rms_normal_expanded")
        if rmsimage11!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/backgrounds/rms/graduation_rms_normal_expanded.png")) {}
        
        let rmsimage12 = NSImage(named: "Contents/Resources/kmeter/skins/Default/graduation_rms_normal")
        if rmsimage12!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/backgrounds/rms/graduation_rms_normal.png")) {}
        
        // buttons
        let button1 = NSImage(named: "Contents/Resources/kmeter/skins/Default/button_about_off")
        if button1!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/buttons/button_about_off.png")) {}
        let button2 = NSImage(named: "Contents/Resources/kmeter/skins/Default/button_about_on")
        if button2!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/buttons/button_about_on.png")) {}
        let button3 = NSImage(named: "Contents/Resources/kmeter/skins/Default/button_about_over")
        if button3!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/buttons/button_about_over.png")) {}
        let button4 = NSImage(named: "Contents/Resources/kmeter/skins/Default/button_discrete_off")
        if button4!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/buttons/button_discrete_off.png")) {}
        let button5 =  NSImage(named: "Contents/Resources/kmeter/skins/Default/button_discrete_on")
        if button5!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/buttons/button_discrete_on.png")) {}
        let button6 = NSImage(named: "Contents/Resources/kmeter/skins/Default/button_discrete_over")
        if button6!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/buttons/button_discrete_over.png")) {}
        let button7 = NSImage(named: "Contents/Resources/kmeter/skins/Default/button_expand_off")
        if button7!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/buttons/button_expand_off.png")) {}
        let button8 = NSImage(named: "Contents/Resources/kmeter/skins/Default/button_expand_on")
        if button8!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/buttons/button_expand_on.png")) {}
        let button9 = NSImage(named: "Contents/Resources/kmeter/skins/Default/button_expand_over")
        if button9!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/buttons/button_expand_over.png")) {}
        let button10 = NSImage(named: "Contents/Resources/kmeter/skins/Default/button_hold_off")
        if button10!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/buttons/button_hold_off.png")) {}
        let button11 = NSImage(named: "Contents/Resources/kmeter/skins/Default/button_hold_on")
        if button11!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/buttons/button_hold_on.png")) {}
        let button12 = NSImage(named: "Contents/Resources/kmeter/skins/Default/button_hold_over")
        if button12!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/buttons/button_hold_over.png")) {}
        let button13 = NSImage(named: "Contents/Resources/kmeter/skins/Default/button_itu_off")
        if button13!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/buttons/button_itu_off.png")) {}
        let button14 = NSImage(named: "Contents/Resources/kmeter/skins/Default/button_itu_on")
        if button14!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/buttons/button_itu_on.png")) {}
        let button15 = NSImage(named: "Contents/Resources/kmeter/skins/Default/button_itu_over")
        if button15!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/buttons/button_itu_over.png")) {}
        let button16 = NSImage(named: "Contents/Resources/kmeter/skins/Default/button_k12_off")
        if button16!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/buttons/button_k12_off.png")) {}
        let button17 = NSImage(named: "Contents/Resources/kmeter/skins/Default/button_k12_on")
        if button17!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/buttons/button_k12_on.png")) {}
        let button18 = NSImage(named: "Contents/Resources/kmeter/skins/Default/button_k12_over")
        if button18!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/buttons/button_k12_over.png")) {}
        let button19 = NSImage(named: "Contents/Resources/kmeter/skins/Default/button_k14_off")
        if button19!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/buttons/button_k14_off.png")) {}
        let button20 = NSImage(named: "Contents/Resources/kmeter/skins/Default/button_k14_on")
        if button20!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/buttons/button_k14_on.png")) {}
        let button21 = NSImage(named: "Contents/Resources/kmeter/skins/Default/button_k14_over")
        if button21!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/buttons/button_k14_over.png")) {}
        let button22 = NSImage(named: "Contents/Resources/kmeter/skins/Default/button_k20_off")
        if button22!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/buttons/button_k20_off.png")) {}
        let button23 = NSImage(named: "Contents/Resources/kmeter/skins/Default/button_k20_on")
        if button23!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/buttons/button_k20_on.png")) {}
        let button24 = NSImage(named: "Contents/Resources/kmeter/skins/Default/button_k20_over")
        if button24!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/buttons/button_k20_over.png")) {}
        let button25 = NSImage(named: "Contents/Resources/kmeter/skins/Default/button_mono_off")
        if button25!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/buttons/button_mono_off.png")) {}
        let button26 = NSImage(named: "Contents/Resources/kmeter/skins/Default/button_mono_on")
        if button26!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/buttons/button_mono_on.png")) {}
        let button27 = NSImage(named: "Contents/Resources/kmeter/skins/Default/button_mono_over")
        if button27!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/buttons/buton_mono_over.png")) {}
        let button28 = NSImage(named: "Contents/Resources/kmeter/skins/Default/button_normal_off")
        if button28!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/buttons/button_normal_off.png")) {}
        let button29 = NSImage(named: "Contents/Resources/kmeter/skins/Default/button_normal_on")
        if button29!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/buttons/button_normal_on.png")) {}
        let button30 = NSImage(named: "Contents/Resources/kmeter/skins/Default/button_normal_over")
        if button30!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/buttons/button_normal_over.png")) {}
        let button31 = NSImage(named: "Contents/Resources/kmeter/skins/Default/button_peaks_off")
        if button31!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/buttons/button_peaks_off.png")) {}
        let button32 = NSImage(named: "Contents/Resources/kmeter/skins/Default/button_peaks_on")
        if button32!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/buttons/button_peaks_on.png")) {}
        let button33 = NSImage(named: "Contents/Resources/kmeter/skins/Default/button_peaks_over")
        if button33!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/buttons/button_peaks_over.png")) {}
        let button34 = NSImage(named: "Contents/Resources/kmeter/skins/Default/button_reset_off")
        if button34!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/buttons/button_reset_off.png")) {}
        let button35 = NSImage(named: "Contents/Resources/kmeter/skins/Default/button_reset_on")
        if button35!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/buttons/button_reset_on.png")) {}
        let button36 = NSImage(named: "Contents/Resources/kmeter/skins/Default/button_reset_over")
        if button36!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/buttons/button_reset_over.png")) {}
        let button37 = NSImage(named: "Contents/Resources/kmeter/skins/Default/button_rms_off")
        if button37!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/buttons/button_rms_off.png")) {}
        let button38 = NSImage(named: "Contents/Resources/kmeter/skins/Default/button_rms_on")
        if button38!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/buttons/button_rms_on.png")) {}
        let button39 = NSImage(named: "Contents/Resources/kmeter/skins/Default/button_rms_over")
        if button39!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/buttons/button_rms_over.png")) {}
        
        // labels
        let label1 = NSImage(named: "Contents/Resources/kmeter/skins/Default/label_over_off")
        if label1!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/labels/label_over_off.png")) {}
        let label2 = NSImage(named: "Contents/Resources/kmeter/skins/Default/label_over_on")
        if label2!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/labels/label_over_on.png")) {}
        
        let label3 = NSImage(named: "Contents/Resources/kmeter/skins/Default/label_peak_off")
        if label3!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/labels/label_peak_off.png")) {}
        
        let label4 = NSImage(named: "Contents/Resources/kmeter/skins/Default/label_peak_on")
        if label4!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/labels/label_peak_on.png")) {}
        
        let label5 = NSImage(named: "Contents/Resources/kmeter/skins/Default/label_true_peak_off")
        if label5!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/labels/label_true_peak_off.png")) {}
        
        let label6 = NSImage(named: "Contents/Resources/kmeter/skins/Default/label_true_peak_on")
        if label6!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/labels/label_true_peak_on.png")) {}
        
        // meters
        let meter1 = NSImage(named: "Contents/Resources/kmeter/skins/Default/meter_kmeter_itu_k12_expanded")
        if meter1!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/meters/meter_kmeter_itu_k12_expanded.png")) {}
        let meter2 = NSImage(named: "Contents/Resources/kmeter/skins/Default/meter_kmeter_itu_k12")
        if meter2!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/meters/meter_kmeter_itu_k12.png")) {}
        let meter3 = NSImage(named: "Contents/Resources/kmeter/skins/Default/meter_kmeter_itu_k14_expanded")
        if meter3!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/meters/meter_kmeter_itu_k14_expanded.png")) {}
        let meter4 = NSImage(named: "Contents/Resources/kmeter/skins/Default/meter_kmeter_itu_k14")
        if meter4!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/meters/meter_kmeter_itu_k14.png")) {}
        let meter5 = NSImage(named: "Contents/Resources/kmeter/skins/Default/meter_kmeter_itu_k20_expanded")
        if meter5!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/meters/meter_kmeter_itu_k20_expanded.png")) {}
        let meter6 = NSImage(named: "Contents/Resources/kmeter/skins/Default/meter_kmeter_itu_k20")
        if meter6!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/meters/meter_kmeter_itu_k20.png")) {}
        let meter7 = NSImage(named: "Contents/Resources/kmeter/skins/Default/meter_kmeter_itu_normal_expanded")
        if meter7!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/meters/meter_kmeter_itu_normal_expanded.png")) {}
        let meter8 = NSImage(named: "Contents/Resources/kmeter/skins/Default/meter_kmeter_itu_normal")
        if meter8!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/meters/meter_kmeter_itu_normal.png")) {}
        let meter9 = NSImage(named: "Contents/Resources/kmeter/skins/Default/meter_kmeter_rms_k12_expanded")
        if meter9!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/meters/meter_kmeter_rms_k12_expanded.png")) {}
        let meter10 = NSImage(named: "Contents/Resources/kmeter/skins/Default/meter_kmeter_rms_k12")
        if meter10!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/meters/meter_kmeter_rms_k12.png")) {}
        let meter11 = NSImage(named: "Contents/Resources/kmeter/skins/Default/meter_kmeter_rms_k14_expanded")
        if meter11!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/meters/meter_kmeter_rms_k14_expanded.png")) {}
        let meter12 = NSImage(named: "Contents/Resources/kmeter/skins/Default/meter_kmeter_rms_k14")
        if meter12!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/meters/meter_kmeter_rms_k14.png")) {}
        let meter13 = NSImage(named: "Contents/Resources/kmeter/skins/Default/meter_kmeter_rms_k20_expanded")
        if meter13!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/meters/meter_kmeter_rms_k20_expanded.png")) {}
        let meter14 = NSImage(named: "Contents/Resources/kmeter/skins/Default/meter_kmeter_rms_k20")
        if meter14!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/meters/meter_kmeter_rms_k20.png")) {}
        let meter15 = NSImage(named: "Contents/Resources/kmeter/skins/Default/meter_kmeter_rms_normal_expanded")
        if meter15!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/meters/meter_kmeter_rms_normal_expanded.png")) {}
        let meter16 = NSImage(named: "Contents/Resources/kmeter/skins/Default/meter_kmeter_rms_normal")
        if meter16!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/meters/meter_kmeter_rms_normal.png")) {}
        let meter17 = NSImage(named: "Contents/Resources/kmeter/skins/Default/meter_phase_correlation_needle")
        if meter17!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/meters/meter_phase_correlation_needle.png")) {}
        let meter18 = NSImage(named: "Contents/Resources/kmeter/skins/Default/meter_phase_correlation")
        if meter18!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/meters/meter_phase_correlation.png")) {}
        let meter19 = NSImage(named: "Contents/Resources/kmeter/skins/Default/meter_stereo_needle")
        if meter19!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/meters/meter_stereo_needle.png")) {}
        let meter20 = NSImage(named: "Contents/Resources/kmeter/skins/Default/meter_stereo")
        if meter20!.savePNG(to: cacheDirectory.appendingPathComponent("K-Meter.component/Contents/Resources/kmeter/skins/Default/meters/meter_stereo.png")) {}
    }
}

extension NSBitmapImageRep {
    var png: Data? {
        return representation(using: .png, properties: [:])
    }
}
extension Data {
    var bitmap: NSBitmapImageRep? {
        return NSBitmapImageRep(data: self)
    }
}

extension NSImage {
    var png: Data? {
        return tiffRepresentation?.bitmap?.png
    }
    func savePNG(to url: URL) -> Bool {
        do {
            try png?.write(to: url)
            return true
        } catch {
            print(error)
            return false
        }
        
    }
}

