import AppKit

@objcMembers
class RepeatModeTransformer: ValueTransformer {
    var repeatMode: RepeatMode?

    override class func transformedValueClass() -> AnyClass {
        return NSNumber.self
    }

    override class func allowsReverseTransformation() -> Bool {
        return true
    }

    init(mode r: RepeatMode) {
        super.init()
        repeatMode = r
    }

    override func transformedValue(_ value: Any?) -> Any? {
        guard value != nil else {
            return nil
        }
        let mode = (value as! NSNumber).intValue
        if repeatMode!.rawValue == mode {
            return NSNumber(value: true)
        }
        return NSNumber(value: false)
    }

    override func reverseTransformedValue(_ value: Any?) -> Any? {
        guard value != nil else {
            return nil
        }
        let enabled = (value as! NSNumber).boolValue
        if enabled {
            return NSNumber(value: (repeatMode?.rawValue)!)
        } else if repeatMode == RepeatMode.RepeatNone {
            return NSNumber(value: RepeatMode.RepeatAll.rawValue)
        } else {
            return NSNumber(value: RepeatMode.RepeatNone.rawValue)
        }
    }
}

@objcMembers
class RepeatModeImageTransformer: ValueTransformer {
    override class func transformedValueClass() -> AnyClass {
        return NSImage.self
    }

    override class func allowsReverseTransformation() -> Bool {
        return false
    }

    override func transformedValue(_ value: Any?) -> Any? {
        guard value != nil else {
            return nil
        }
        let mode = (value as! NSNumber).intValue
        if mode == RepeatMode.RepeatNone.rawValue {
            return NSImage(named: NSImage.Name("RepeatModeOffTemplate"))
        } else if mode == RepeatMode.RepeatOne.rawValue {
            return NSImage(named: NSImage.Name("RepeatModeOneTemplate"))
        } else if mode == RepeatMode.RepeatAlbum.rawValue {
            return NSImage(named: NSImage.Name("RepeatModeAlbumTemplate"))
        } else if mode == RepeatMode.RepeatAll.rawValue {
            return NSImage(named: NSImage.Name("RepeatModeAllTemplate"))
        }
        return NSImage()
    }
}
