import AppKit

@objcMembers
class ShuffleImageTransformer: ValueTransformer {
    override class func transformedValueClass() -> AnyClass {
        return NSImage.self
    }

    override class func allowsReverseTransformation() -> Bool {
        return false
    }

    override func transformedValue(_ value: Any?) -> Any? {
        guard value != nil else {
            return nil
        }
        let mode = (value as! NSNumber).intValue
        if mode == ShuffleMode.ShuffleOff.rawValue {
            return NSImage(named: NSImage.Name("ShuffleOffTemplate"))
        } else if mode == ShuffleMode.ShuffleAlbums.rawValue {
            return NSImage(named: NSImage.Name("ShuffleAlbumTemplate"))
        } else if mode == ShuffleMode.ShuffleAll.rawValue {
            return NSImage(named: NSImage.Name("ShuffleOnTemplate"))
        }
        return NSImage()
    }
}

@objcMembers
class ShuffleModeTransformer: ValueTransformer {
    var shuffleMode: ShuffleMode?

    override class func transformedValueClass() -> AnyClass {
        return NSNumber.self
    }

    override class func allowsReverseTransformation() -> Bool {
        return true
    }

    init(mode s: ShuffleMode) {
        super.init()
        shuffleMode = s
    }

    override func transformedValue(_ value: Any?) -> Any? {
        guard value != nil else {
            return nil
        }
        let mode = (value as! NSNumber).intValue
        if shuffleMode?.rawValue == mode {
            return NSNumber(value: true)
        }
        return NSNumber(value: false)
    }

    override func reverseTransformedValue(_ value: Any?) -> Any? {
        guard value != nil else {
            return nil
        }
        let enabled = (value as! NSNumber).boolValue
        if enabled {
            return NSNumber(value: (shuffleMode?.rawValue)!)
        } else if shuffleMode == ShuffleMode.ShuffleOff {
            return NSNumber(value: ShuffleMode.ShuffleAll.rawValue)
        } else {
            return NSNumber(value: ShuffleMode.ShuffleOff.rawValue)
        }
    }
}
