import SystemConfiguration
import AppKit

func isConnectedToNetwork() -> Bool {

    var zeroAddress = sockaddr_in()
    zeroAddress.sin_len = UInt8(MemoryLayout<sockaddr_in>.size)
    zeroAddress.sin_family = sa_family_t(AF_INET)
    
    guard let defaultRouteReachability = withUnsafePointer(to: &zeroAddress, {
        $0.withMemoryRebound(to: sockaddr.self, capacity: 1) {
            SCNetworkReachabilityCreateWithAddress(nil, $0)
        }
    }) else {
        return false
    }
    
    var flags: SCNetworkReachabilityFlags = []
    if !SCNetworkReachabilityGetFlags(defaultRouteReachability, &flags) {
        return false
    }
    
    let isReachable = flags.contains(.reachable)
    let needsConnection = flags.contains(.connectionRequired)
    
    return (isReachable && !needsConnection)
}

func changeFontSize(_ size: Float) {
    let defaults = UserDefaults.standard
    let fCurrentSize: Float = defaults.float(forKey: "fontSize")
    let newSize = fCurrentSize + size
    defaults.set(newSize, forKey: "fontSize")
}

func imageResize(_ anImage: NSImage?, newSize: NSSize) -> NSImage? {
    let sourceImage: NSImage? = anImage
    //        sourceImage?.setScalesWhenResized(true)
    // Report an error if the source isn't a valid image
    if sourceImage?.isValid == nil {
    } else {
        let smallImage = NSImage(size: newSize)
        smallImage.lockFocus()
        sourceImage?.size = newSize
        NSGraphicsContext.current?.imageInterpolation = .high
        sourceImage?.draw(at: NSZeroPoint, from: CGRect(x: 0, y: 0, width: newSize.width, height: newSize.height), operation: .copy, fraction: 1.0)
        smallImage.unlockFocus()
        return smallImage
    }
    return nil
}

func transFormers() {
    let fontSizetoLineHeightTransformer = FontSizetoLineHeightTransformer()
    ValueTransformer.setValueTransformer(fontSizetoLineHeightTransformer, forName: NSValueTransformerName("FontSizetoLineHeightTransformer"))
    // Repeatmode
    let repeatNoneTransformer = RepeatModeTransformer(mode: RepeatMode.RepeatNone)
    ValueTransformer.setValueTransformer(repeatNoneTransformer, forName: NSValueTransformerName("RepeatNoneTransformer"))
    let repeatOneTransformer = RepeatModeTransformer(mode: RepeatMode.RepeatOne)
    ValueTransformer.setValueTransformer(repeatOneTransformer, forName: NSValueTransformerName("RepeatOneTransformer"))
    let repeatAlbumTransformer = RepeatModeTransformer(mode: RepeatMode.RepeatAlbum)
    ValueTransformer.setValueTransformer(repeatAlbumTransformer, forName: NSValueTransformerName("RepeatAlbumTransformer"))
    let repeatAllTransformer = RepeatModeTransformer(mode: RepeatMode.RepeatAll)
    ValueTransformer.setValueTransformer(repeatAllTransformer, forName: NSValueTransformerName("RepeatAllTransformer"))
    let repeatModeImageTransformer = RepeatModeImageTransformer()
    ValueTransformer.setValueTransformer(repeatModeImageTransformer, forName: NSValueTransformerName("RepeatModeImageTransformer"))
    // Shufflemode
    let shuffleOffTransformer = ShuffleModeTransformer(mode: ShuffleMode.ShuffleOff)
    ValueTransformer.setValueTransformer(shuffleOffTransformer, forName: NSValueTransformerName("ShuffleOffTransformer"))
    let shuffleAlbumsTransformer = ShuffleModeTransformer(mode: ShuffleMode.ShuffleAlbums)
    ValueTransformer.setValueTransformer(shuffleAlbumsTransformer, forName: NSValueTransformerName("ShuffleAlbumsTransformer"))
    let shuffleAllTransformer = ShuffleModeTransformer(mode: ShuffleMode.ShuffleAll)
    ValueTransformer.setValueTransformer(shuffleAllTransformer, forName: NSValueTransformerName("ShuffleAllTransformer"))
    let shuffleImageTransformer = ShuffleImageTransformer()
    ValueTransformer.setValueTransformer(shuffleImageTransformer, forName: NSValueTransformerName("ShuffleImageTransformer"))
    let statusImageTransformer = StatusImageTransformer()
    ValueTransformer.setValueTransformer(statusImageTransformer, forName: NSValueTransformerName("StatusImageTransformer"))
}

func makeSafeTabName(for unsafeName: String) -> String {
    var charSet = CharacterSet.urlPathAllowed
    charSet.remove(charactersIn: "/")
    return unsafeName.components(separatedBy: charSet.inverted).joined(separator: "_")
}

extension String {
    public func OSType() -> OSType {
        return UTGetOSTypeFromString(self as CFString)
    }
}

func log2u(_ inU: UInt32) -> UInt32 {
    var v : UInt32 = (UInt32)(inU)  // 32-bit word to find the log base 2 of
    var r : UInt32 = 0              // r will be lg(v)
    
    while (v > 1)                   // unroll for more speed...
    {
        v >>= 1
        r += 1
    }
    return UInt32(r)
}

func printLog(_ log: Any?) {
    let formatter = DateFormatter()
    formatter.dateFormat = "yyyy-MM-dd HH:mm:ss.SSS "
    print(formatter.string(from:Date()), terminator: "")
    let printStr = formatter.string(from:Date())
    
    if log == nil {
        print("nil")
    }
    else {
        print(printStr, log!)
    }
}
