import AppKit

class MainWindow: NSWindow, NSTabViewDelegate {
    @IBOutlet var tabView            : LYTabView!
    @IBOutlet var playlistController : PlaylistController!
    @IBOutlet var playlistLoader     : PlaylistLoader!
    
    var tabBarView     : LYTabBarView!
    var playlistPerTab : [Int : NSMutableArray]!
    var playlistNewKey : Int!
    var initWindow     : Bool = false
    var playingTabId   : Int = 0

    override func awakeFromNib() {
        backgroundColor = BackgroundColorA
        tabView.delegate = self
        for item: NSTabViewItem in tabView.tabViewItems {
            tabView.removeTabViewItem(item)
        }
        tabBarView = tabView.tabBarView
        //
        // init singleton as mutable
        //
        playlistPerTab = [:]
        playlistNewKey = 0
        //
        // set tabBar style
        //
        tabBarView.minTabHeight = 28
        tabBarView.minTabItemWidth = 120
        tabView.tabBarView.hideIfOnlyOneTabExists = false
        tabBarView.addNewTabButtonAction = #selector(addNewTab)
        tabBarView.addNewTabButtonTarget = self
        initWindow = true
        //
        // scan Phonix/tabs dir for m3u files
        //
        var nrTitles: NSMutableArray = []
        let regex = try? NSRegularExpression(pattern: "^.*.m3u$", options: .caseInsensitive)
        
        if directoryExistsAtPath(PhonixTabsFolder.path) {
            nrTitles = readTabFiles(regex!, inPath: PhonixTabsFolder.path as NSString)
              for index in 0 ..< nrTitles.count {
                let item = NSTabViewItem(identifier: index)
                item.label = nrTitles[index] as! String
                DispatchQueue.main.async {
                    self.tabView.tabView.addTabViewItem(item)
                }
            }
        }
        if nrTitles.count == 0 {
            //
            // load the content into singleton.tabplaylist of index
            //
            DispatchQueue.main.async {
                (self.playlistController.content as! NSMutableArray).removeAllObjects()
                
                self.addViewWithLabel("Untitled-01", aTabBarView: self.tabBarView, fromTabView: true)
            }
        }
        initWindow = false
        //
        //  select the tab we want selected
        //
        DispatchQueue.main.async {
        self.selectTab(UserDefaults.standard.integer(forKey: "selectedTabIndex"))
        }        
        self.orderFrontRegardless()
    }
    
    func addViewWithLabel(_ label: String, aTabBarView: LYTabBarView, fromTabView: Bool = false) {
        let item = NSTabViewItem(identifier: playlistNewKey)
        item.label = label
        playlistController.content = nil
        let array = ((playlistController.content as! [PlaylistEntry]) as NSArray).mutableCopy() as! NSMutableArray
        playlistPerTab[playlistNewKey] = array
        playlistNewKey += 1
        tabView.tabView.addTabViewItem(item)
        tabView.selectTabViewItem(item)
    }
    
    @IBAction func addNewTab(_ sender: AnyObject?) {
        if let tabBarView = (sender as? LYTabBarView) ?? self.tabBarView {
            let count = tabBarView.tabViewItems.count
            let label = String(format: "Untitled-%02i", count + 1)
            addViewWithLabel(label, aTabBarView: tabBarView)
        }
    }
    
    func tabView(_ tabView: NSTabView, willSelect tabViewItem: NSTabViewItem?) {
        setPlayingTab(withId: playingTabId)
    }
    
    func tabViewDidChangeNumberOfTabViewItems(_ tabView: NSTabView) {
        if !initWindow
        {
            
            // This is does not work and messes up the tabarrays
            
//            if tabView.tabViewItems.count != playlistPerTab.count {
//                let keys = Array(playlistPerTab.keys)
//                for key in keys {
//                    var found: Bool = false
//                    for itemView: NSTabViewItem in tabView.tabViewItems {
//                        let id: Int = itemView.identifier as! Int
//                        if key == id {
//                            found = true
//                            break
//                        }
//                    }
//                    if !found {
//                        playlistPerTab.removeValue(forKey: key)
//                    }
//                }
//            }
            setPlayingTab(withId: playingTabId)
        }
    }
    
    func selectTab(withId id: Int) {
        for itemView: NSTabViewItem in tabView.tabViewItems {
            if id == itemView.identifier as! Int {
                tabView.selectTabViewItem(itemView)
            }
        }
    }
    
    func selectTab(_ tabindex: Int) {
        let tabIndex = tabindex
   
        if tabIndex < tabView.numberOfTabViewItems {
            tabView.selectTabViewItem(at: tabindex)
        }
    }
    
    func selectPlayingTab() {
        if selectedTabId() != playingTabId {
            selectTab(withId: playingTabId)
        }
    }
    
    func setPlayingTab(withId tabId: Int, playing: Bool = true ) {
        playingTabId = tabId
        for tabViewItem in tabView.tabViewItems {
            let id = tabViewItem.identifier as! Int
            let itemView = tabBarView.itemViewForItem(tabViewItem)
            if id == tabId {
                itemView?.nowPlaying.isHidden = !playing
            } else {
                itemView?.nowPlaying.isHidden = true
            }
        }
    }
    
    func tabView(_ tabView: NSTabView, didSelect tabViewItem: NSTabViewItem?) {
        let id = tabViewItem?.identifier as! Int
        playlistController.content = playlistPerTab[id]
    }
    
    func tabNames() -> [Any] {
        var tabs = [Any]()
        for i in 0 ..< tabView.numberOfTabViewItems {
            tabs.append(tabView.tabViewItem(at: i).label)
        }
        return tabs
    }
    
    func selectedTabId() -> Int {
        return tabView.selectedTabViewItem?.identifier as! Int
    }
    
    func selectedTabName() -> String? {
        let tabName = tabView.selectedTabViewItem?.label
        return tabName
    }
    
    func selectedTabIndex() -> Int {
        return tabView.indexOfTabViewItem(tabView.selectedTabViewItem!)
    }
    
    func playingTabIndex() -> Int {
//        var index = 0
//        tabView.tabView.selectedTabViewItem?.identifier as! Int
//        for itemView: NSTabViewItem in tabView.tabViewItems {
//
//            if playingTabId == itemView.identifier as! Int {
//                return index
//            }
//            index += 1
//        }
        return tabView.tabView.selectedTabViewItem?.identifier as! Int // 0
    }
    
    func getTabName(forTabIndex index: Int) -> NSString {
        let tab: NSTabViewItem = tabView.tabViewItem(at: index)
        return tab.label as NSString
    }
    
    func getPlayListIndex(forTabIndex index: Int) -> Int {
        let tab: NSTabViewItem = tabView.tabViewItem(at: index)
        return tab.identifier as! Int
    }
    
    func addTab(withName name: String) {
        var label: String = ""
        if name.isEmpty {
            label = String(format: "Untitled-%02i", tabBarView.tabViewItems.count + 1)
        } else {
            label = name
            for tabIndex in 0 ..< tabView.numberOfTabViewItems {
                let tabViewItem: NSTabViewItem = tabView.tabViewItem(at: tabIndex)
                if tabViewItem.label == name {
                    label = name + String(format: "-%02i", tabBarView.tabViewItems.count + 1)
                    break
                }
            }
        }
        addViewWithLabel(label, aTabBarView: tabBarView)
    }
    
    func readTabFiles(_ regex: NSRegularExpression, inPath path: NSString) -> NSMutableArray {
        // read the stored tabs (m3u playlists)
        // hfs reads in alphabethical order,
        // apfs reads in random order
        //
        // we need to read in alphabethical order.
        //
        var file: NSString = ""
        var index: Int = 0
        let tabTitles: NSMutableArray = []
        let fileList = try! PhonixFileManager.contentsOfDirectory(at: URL(fileURLWithPath: path as String), includingPropertiesForKeys: [.fileResourceIdentifierKey], options: [])
        let m3uList: NSMutableArray = []
        //
        // put all m3u files in /tab in an array
        //
        if fileList.count != 0 {
            for fileURL in fileList {
                var i: Int = 0
                file = (fileURL as AnyObject).lastPathComponent as NSString
                if regex.numberOfMatches(in: file as String, options: [], range: NSRange(location: 0, length: file.length)) != 0 {
                    m3uList.add(fileURL as NSURL)
                    i += 1
                }
            }
            //
            // sort the files alphanumerically, hfs+ did that automatically, apfs does not
            //
            let sd_path = NSSortDescriptor(key: "path", ascending: true)
            let sortedList = NSMutableArray(array: m3uList.sortedArray(using: [sd_path]))
            
            while index < sortedList.count {
                let fileName: NSURL = sortedList.object(at: index) as! NSURL
                file = fileName.lastPathComponent! as NSString
                //
                // load files into tabs, the files wil be put into tabs at their numerological and then alphabetical order
                //
                // get filename without extension and path
                //
                var tabTitle: NSString = ((file.lastPathComponent) as NSString).deletingPathExtension as NSString
                //
                // remove the index if present
                // put the parts seperated by | in an array
                //
                let components: NSArray = tabTitle.components(separatedBy: "|") as NSArray
                //
                // divide the name in 2 parts (if the divider exists) take the last part
                //
                tabTitle = components.lastObject as! NSString
                //
                // put the names in array
                //
                tabTitles.add(tabTitle)
                //
                // empty playlist controller
                //
                (playlistController.content as! NSMutableArray).removeAllObjects()
                //
                // load file to playlist controller
                //
                _ = playlistLoader.add(fileName as URL)
           
                let array = ((playlistController.content as! [PlaylistEntry]) as NSArray).mutableCopy() as! NSMutableArray
                playlistPerTab[index] = array
                //
                // next file
                //
                index += 1
            }
        }
        playlistNewKey = index
        return tabTitles
        //
        // we want to return an array with tabTitles
        //
    }
    
    func saveTabFiles(toPath path: NSString) {
        for tabIndex in 0 ..< tabView.numberOfTabViewItems {
            let tabViewItem: NSTabViewItem = tabView.tabViewItem(at: tabIndex)
            let id: Int = tabViewItem.identifier as! Int
            let fileName = String(format: "%02i|%@.m3u", tabIndex, tabViewItem.label)
            let file = path.appendingPathComponent(fileName)
            //
            // load the playlist per tab in the playlist controller and save the playlist
            //
            if tabView.numberOfTabViewItems > 1 {
                playlistController.content = playlistPerTab[id]
            }
            _ = playlistLoader!.saveM3u(file)
        }
    }
}

class SearchFieldCell: NSSearchFieldCell {
    override func select(withFrame rect: NSRect, in controlView: NSView, editor textObj: NSText, delegate: Any?, start selStart: Int, length selLength: Int) {
        var newRect = rect
        if !isBordered || isBezeled {
            let cancelButtonWidth = NSWidth(cancelButtonRect(forBounds: rect))
            let searchButtonWidth = NSWidth(searchButtonRect(forBounds: rect))
            newRect.size.width -= (cancelButtonWidth + searchButtonWidth)
            newRect.origin.x += searchButtonWidth
        }
        super.select(withFrame: newRect, in: controlView, editor: textObj, delegate: delegate, start: selStart, length: selLength)
    }
}
