import Foundation
import AppKit

class MenuVolumeTextField : NSTextField {
    var volumeFloat : Float?
    
    override init(frame frameRect: NSRect) {
        super.init(frame: frameRect)
        textColor       = MeterBarColor
        font            = NSFont.systemFont(ofSize: 12.0)
        alignment       = .right
        bezelStyle      = .roundedBezel
        isBezeled       = false
        isEditable      = false
        isSelectable    = false
        drawsBackground = true
        usesSingleLineMode = true
    }
    
    required init?(coder: NSCoder) {
        super.init(coder: coder)
    }
    
    private func mixerTodB(value: Float) -> String {
        return String(format: "%0.1f dB ", 20 * log10(value))
    }
    
    private func sliderToMixer(slider: Float) -> Float {
        return slider * slider * slider
    }
    
    func setVolumeText(_ volume: Float) {
        stringValue = mixerTodB(value: sliderToMixer(slider: volume))
    }
    
    var volume : Float? {
        get {
            return volumeFloat
        }
        set (newVolume) {
            setVolumeText(newVolume!)
            volumeFloat = newVolume
        }
    }
}
