import Foundation
import AppKit

protocol StatusBarDelegate {
    func statusBarAppIconLeftMouse()
    func statusBarAppIconRightMouse()
    func statusBarPausePlay()
    func statusBarNext()
    func statusBarPrev()
    func statusBarSetVolume(_ volume: Float)
}

class StatusBar : NSObject {
    
    @IBOutlet var menuVolumeSlider      : MenuVolumeSlider?
    @IBOutlet var menuVolumeTextfield   : MenuVolumeTextField?
    @IBOutlet var playbackController    : PlaybackController!
    
    var testItemController, menuBarAppIcon, menuBarPrev, menuBarPlay, menuBarNext, menuBarVolumeSlider, menuBarVolumeText: StatusItemController?
    
    var delegate : StatusBarDelegate?
    
    convenience init(volumeDisplay: Bool) {
        self.init()
        //
        // in order of appearance, from right to left...
        //
        if volumeDisplay == true {
            createItemVolumeText()
        }
        createItemVolumeSlider()
        createItemNext()
        createItemPlay()
        createItemPrev()
        createItemAppIcon()
    }
    
    func createItemPrev() {
        menuBarPrev = StatusItemController(buttonWithSelector: #selector(prev),
                                           target: self,
                                           title: "Prev",
                                           imageName: "PreviousTemplate")
    }
    
    func createItemPlay() {
        menuBarPlay = StatusItemController(buttonWithSelector: #selector(play),
                                           target: self,
                                           title: "Play",
                                           imageName: "PlayTemplate",
                                           altImageName: "PauseTemplate")
    }
    
    func createItemNext() {
        menuBarNext = StatusItemController(buttonWithSelector: #selector(next),
                                           target: self,
                                           title: "Next",
                                           imageName: "NextTemplate")
    }
    
    func createItemAppIcon() {
        menuBarAppIcon = StatusItemController(buttonWithSelector: #selector(appIconAction),
                                              target: self,
                                              title: "LeftMouse Show\nRightMouse Hide",
                                              imageName: "icon_status",
                                              rightMouseAction: true)
    }
    
    func createItemVolumeSlider() {
        menuBarVolumeSlider = StatusItemController(volumeControlSelector: #selector(setVolume), target: self, title: "Volume")
        menuVolumeSlider = menuBarVolumeSlider?.menuVolumeSlider()
    }

    func createItemVolumeText() {
        menuBarVolumeText = StatusItemController(volumeTextWithTitle: "Volume Display")
        menuVolumeTextfield = menuBarVolumeText?.menuVolumeText()
    }

    func removeItemVolumeText() {
        menuBarVolumeText?.removeItem()
    }
    
    func removeAllItems() {
        testItemController?.removeItem()
        menuBarAppIcon?.removeItem()
        menuBarPrev?.removeItem()
        menuBarPlay?.removeItem()
        menuBarNext?.removeItem()
        menuBarVolumeSlider?.removeItem()
        menuBarVolumeText?.removeItem()
    }
    //
    // set controls
    //
    func set(playStatus: Bool) {
        menuBarPlay?.set(status: playStatus)
    }
    
    func set(volume: Float) {
        menuVolumeSlider?.floatValue = volume
        menuVolumeTextfield?.volume  = volume
    }
    
    @IBAction func prev(_ sender: NSStatusBarButton ) { delegate?.statusBarPrev() }
    @IBAction func play(_ sender: NSStatusBarButton ) { delegate?.statusBarPausePlay() }
    @IBAction func next(_ sender: NSStatusBarButton ) { delegate?.statusBarNext() }
    
    @IBAction func setVolume(_ sender: MenuVolumeSlider) {
        let volume = sender.floatValue
        delegate?.statusBarSetVolume(volume)
    }
    
    @IBAction func appIconAction(_ sender: NSStatusBarButton) {
        let event = NSApp.currentEvent!
        if event.type == NSEvent.EventType.rightMouseDown {
            // Right button click
            delegate?.statusBarAppIconRightMouse()
        } else if event.type == NSEvent.EventType.leftMouseDown {
            // Left button click
            delegate?.statusBarAppIconLeftMouse()
        }
    }
}
