import Foundation
import AppKit

protocol StatusItemButtonDelegate: NSObjectProtocol {
    func statusItemButtonMouseMoved(_ button: StatusItemButton)
    func statusItemButtonLeftClick(_ button: StatusItemButton)
    func statusItemButtonRightClick(_ button: StatusItemButton)
}

class StatusItemButton: NSView {
    var image: NSImage?
    weak var delegate: StatusItemButtonDelegate?
    
    init(image: NSImage) {
        super.init(frame: NSMakeRect(0, 0, image.size.width, image.size.height))
        self.image = image
    }
    
    required init?(coder aDecoder: NSCoder) {
        super.init(coder: aDecoder)
    }
    
    func setImage(_ image: NSImage) {
        self.image = image
        let rect = CGRect(x: 0, y: 0, width: image.size.width, height: image.size.height)
        let trackingArea = NSTrackingArea(rect: rect, options: [.mouseEnteredAndExited, .activeAlways], owner: self, userInfo: nil)
        addTrackingArea(trackingArea)
        needsDisplay = true
    }
    
    override func draw(_ dirtyRect: NSRect) {
        let imageSize: NSSize = image!.size
        let x: CGFloat = (bounds.size.width - imageSize.width) / 2
        let y: CGFloat = (bounds.size.height - imageSize.height) / 2
        let drawnRect: NSRect = NSMakeRect(x, y, imageSize.width, imageSize.height)
        image?.draw(in: drawnRect, from: NSZeroRect, operation: .sourceOver, fraction: 1.0)
    }
    
    override func mouseDown(with theEvent: NSEvent) {
//        print("SIB mouseDown")
    }
    
    override func rightMouseDown(with theEvent: NSEvent) {
//        print("SIB rightMouseDown")
    }
    
    override func scrollWheel(with event: NSEvent) {
//        print("SIB scrollWheel")
    }
}
