import AppKit

class StatusItemController: NSObject {
    
    var menu                        : NSMenu?
    var image                       : NSImage?
    var imageAlt                    : NSImage?
    
    var item                        : NSStatusItem?
    var volumeSlider                : MenuVolumeSlider?
    var volumeTxtFld                : MenuVolumeTextField?
    
    let menuBarItemWidth            : CGFloat = NSStatusBar.system.thickness
    let menuBarItemHeight           : CGFloat = NSStatusBar.system.thickness
    let menubarStatusItemWidth      : CGFloat = NSStatusBar.system.thickness + 4.0
    let menubarVolumeTextItemWidth  : CGFloat = NSStatusBar.system.thickness * 3.0 - 8.0
    let menubarVolumeSliderItemWidth: CGFloat = NSStatusBar.system.thickness * 4.0
    
    override init() {
        super.init()
    }
    
    init(image: NSImage) {
        super.init()
        self.image = image
    }
    
    init(image: NSImage, menu: NSMenu) {
        super.init()
        self.image = image
        self.menu  = menu
    }
    
    init(buttonWithSelector: Selector,
         target: AnyObject,
         title: String,
         imageName: String,
         altImageName: String = "",
         rightMouseAction: Bool = false) {
        
        super.init()
        
        item = NSStatusBar.system.statusItem(withLength: menubarStatusItemWidth)
        setImage(name: imageName)
        setAltImage(name: altImageName)
        if rightMouseAction { item?.button?.sendAction(on: [.leftMouseDown, .rightMouseDown]) }
        item?.highlightMode = true
        item?.button?.target = target
        item?.button?.action = buttonWithSelector
        item?.button?.toolTip = title
    }
    
    init(volumeControlSelector: Selector, target: AnyObject, title: String) {
        super.init()
        let rect                = NSMakeRect(0.0, 0.0, menubarVolumeSliderItemWidth, menuBarItemHeight)
        volumeSlider            = MenuVolumeSlider(frame: rect)
        volumeSlider?.cell      = VolumeSliderCell()
        volumeSlider?.action    = volumeControlSelector
        volumeSlider?.target    = target
        
        if volumeSlider != nil {
            item = NSStatusBar.system.statusItem(withLength: menubarVolumeSliderItemWidth)
            // Fix for 10.15 SDK not allowing Custom Views on a statusbarbutton anymore
            let image = NSImage(named: "Transparantvoid")
            image!.size = NSMakeSize(menuBarItemWidth, menuBarItemHeight)
            item?.button?.image = image
            self.imageAlt = image
            //
            item?.button?.addSubview(volumeSlider!)
            item?.button?.isEnabled = false
            item?.button?.toolTip = title
        }
    }
    
    init(volumeTextWithTitle: String) {
        super.init()
        let txtRect  = NSMakeRect(0.0, 3.0, menubarVolumeTextItemWidth, menuBarItemHeight)
        volumeTxtFld = MenuVolumeTextField(frame: txtRect)
        
        if volumeTxtFld != nil {
            item = NSStatusBar.system.statusItem(withLength: menubarVolumeTextItemWidth)
            // Fix for 10.15 SDK not allowing Custom Views on a statusbarbutton anymore
            let image = NSImage(named: "Transparantvoid")
            image!.size = NSMakeSize(menubarVolumeTextItemWidth, menuBarItemHeight)
            item?.button?.image = image
            self.imageAlt = image
            //
            item?.button?.addSubview(volumeTxtFld!)
            item?.button?.isEnabled = false
            item?.button?.toolTip = volumeTextWithTitle
        }
    }
    
    func menuVolumeSlider() -> MenuVolumeSlider {
        return volumeSlider!
    }
    
    func menuVolumeText() -> MenuVolumeTextField {
        return volumeTxtFld!
    }
    
    func setImage(name: String) {
        if let image = NSImage(named: NSImage.Name(name)) {
            image.size = NSMakeSize(menuBarItemWidth, menuBarItemHeight)
            self.image = image
            item?.button?.image = image
        }
    }
    
    func setAltImage(name: String) {
        if let image = NSImage(named: NSImage.Name(name)) {
            image.size = NSMakeSize(menuBarItemWidth, menuBarItemHeight)
            self.imageAlt = image
            item?.button?.image = image
        }
    }
    
    func set(status: Bool) {
        if status {
            item?.button?.image = imageAlt
        } else {
            item?.button?.image = image
        }
    }
    
    func removeItem() {
        if item != nil {
        NSStatusBar.system.removeStatusItem(item!)
        item = nil
        }
    }
    
}

