//
//  RepeatModeTransformer.swift
//  Phonix
//
//  Created by Viktor on 18/11/2017.
//  Copyright © 2017 Viktor Vos. All rights reserved.
//

import Foundation
import Cocoa

class RepeatModeTransformer: ValueTransformer {
    var repeatMode: RepeatMode?
    
    override class func transformedValueClass() -> AnyClass {
        return NSNumber.self
    }
    
    override class func allowsReverseTransformation() -> Bool {
        return true
    }
    
    init(mode r: RepeatMode) {
        super.init()
        repeatMode = r
    }
    
    // Convert from RepeatMode to BOOL
    func transformedValue(_ value: Any) -> Any {
        if type(of: value) is AnyClass {
            let mode = Int((value as! RepeatMode).rawValue) //as? RepeatMode
            if repeatMode!.rawValue == mode {
                return 1
            }
        }
        return 0
    }
    
    func reverseTransformedValue(_ value: Any) -> Any {
        if type(of: value) is AnyClass {
            let enabled: Bool = value as! Int != 0
            if enabled {
                return repeatMode as Any
            }
            else if repeatMode == RepeatMode.RepeatNone {
                return RepeatMode.RepeatAll
            }
            else {
                return RepeatMode.RepeatNone
            }
        }
        return 0
    }
}

class RepeatModeImageTransformer: ValueTransformer {
    
    override class func transformedValueClass() -> AnyClass {
        return NSImage.self
    }
    
    override class func allowsReverseTransformation() -> Bool {
        return false
    }
    
    // Convert from string to RepeatMode
    func transformedValue(_ value: Any) -> Any {
        if type(of: value) is AnyClass {
            let mode = Int((value as! RepeatMode).rawValue) //as? RepeatMode
            //        let mode = Int(value) as? RepeatMode
            if mode == RepeatMode.RepeatNone.rawValue {
                return NSImage(named: NSImage.Name("repeatModeOffTemplate")) ?? NSImage()
            }
            else if mode == RepeatMode.RepeatOne.rawValue {
                return NSImage(named: NSImage.Name("repeatModeOneTemplate")) ?? NSImage()
            }
            else if mode == RepeatMode.RepeatAlbum.rawValue {
                return NSImage(named: NSImage.Name("repeatModeAlbumTemplate")) ?? NSImage()
            }
            else if mode == RepeatMode.RepeatAll.rawValue {
                return NSImage(named: NSImage.Name("repeatModeAllTemplate")) ?? NSImage()
            }
        }
        return 0
    }
}
