//
//  ShuffleImageTransformer.swift
//  Phonix
//
//  Created by Viktor on 18/11/2017.
//  Copyright © 2017 Viktor Vos. All rights reserved.
//

import Foundation
import Cocoa

class ShuffleImageTransformer: ValueTransformer {
    
    override class func transformedValueClass() -> AnyClass {
        return NSImage.self
    }
    
    override class func allowsReverseTransformation() -> Bool {
        return false
    }
    
    // Convert from string to RepeatMode
    func transformedValue(_ value: Any) -> Any {
        if type(of: value) is AnyClass {
            
            let mode = Int((value as! ShuffleMode).rawValue) //as? RepeatMode
            
            //            let mode = Int(value) as? ShuffleMode
            if mode == ShuffleMode.ShuffleOff.rawValue {
                return NSImage(named: NSImage.Name("shuffleOffTemplate")) ?? NSImage()
            }
            else if mode == ShuffleMode.ShuffleAlbums.rawValue {
                return NSImage(named: NSImage.Name("shuffleAlbumTemplate")) ?? NSImage()
            }
            else if mode == ShuffleMode.ShuffleAll.rawValue {
                return NSImage(named: NSImage.Name("shuffleOnTemplate")) ?? NSImage()
            }
        }
        
        return 0
    }
    
}

class ShuffleModeTransformer: ValueTransformer {
    var shuffleMode: ShuffleMode?
    
    override class func transformedValueClass() -> AnyClass {
        return NSNumber.self
    }
    
    override class func allowsReverseTransformation() -> Bool {
        return true
    }
    
    init(mode s: ShuffleMode) {
        super.init()
        
        shuffleMode = s
        
    }
    
    // Convert from RepeatMode to BOOL
    func transformedValue(_ value: Any) -> Any {
        if type(of: value) is AnyClass {
            let mode = Int((value as! ShuffleMode).rawValue) //as? RepeatMode
            //        let mode = Int(value) as? ShuffleMode
            if shuffleMode!.rawValue == mode {
                return 1
            }
        }
        return 0
    }
    
    func reverseTransformedValue(_ value: Any) -> Any {
        if type(of: value) is AnyClass {
            
            let enabled: Bool = value as! Int != 0
            if enabled {
                return shuffleMode as Any
            }
            else if shuffleMode == ShuffleMode.ShuffleOff {
                return ShuffleMode.ShuffleAll
            }
            else {
                return ShuffleMode.ShuffleOff
            }
        }
        return 0
    }
}

