import AppKit

class TimeField: NSTextField {
    var showTotalTime : Bool   = false
    var value         : Double = 0.0
    var maxValue      : Double = 0.0

    override func awakeFromNib() {
        showTotalTime = UserDefaults.standard.bool(forKey: "timerShowTimeRemaining")
        update()
    }

    func update() {
        var timeDisplay   : String = ""
        var timeElapsed   : String = ""
        var timeRemaining : String = ""
        var timeTotal     : String = ""
        var timeType      : String = ""
        var secTotal      : Int    = 0
        
        if !maxValue.isNaN {
            secTotal = Int(round(maxValue))
        }
        let secElapsed = Int(round(value))
        let secRemaining: Int = secTotal - secElapsed
        if UserDefaults.standard.bool(forKey: "DisplayHours") {
            timeType = "%i.%02i.%02i"
            timeElapsed = String(format: timeType, secElapsed / 3600, (secElapsed / 60) % 60, secElapsed % 60)
            timeRemaining = String(format: timeType, secRemaining / 3600, (secRemaining / 60) % 60, secRemaining % 60)
            timeTotal = String(format: timeType, secTotal / 3600, (secTotal / 60) % 60, secTotal % 60)

            if showTotalTime == true {
                if secTotal <= 0 || secRemaining <= 0 {
                    timeDisplay = "\(timeElapsed)|\n-.--.--|total"
                } else {
                    timeDisplay = "\(timeElapsed)|\n\(timeTotal)|total"
                }
            } else if showTotalTime == false {
                if secTotal <= 0 || secRemaining <= 0 {
                    timeDisplay = "\(timeElapsed)|\n-.--.--|remain"
                } else {
                    timeDisplay = "\(timeElapsed)|\n\(timeRemaining)|remain"
                }
            }
        } else {
            var threeDigits: Bool
            let threeDigitLimit: Int = 6000
            if secTotal >= threeDigitLimit || secElapsed >= threeDigitLimit {
                threeDigits = true
            } else {
                threeDigits = false
            }
            if threeDigits {
                timeType = TimeMinutes3
            } else {
                timeType = TimeMinutes2
            }
            timeElapsed = String(format: timeType, (secElapsed / 60), secElapsed % 60)
            timeRemaining = String(format: timeType, (secRemaining / 60), secRemaining % 60)
            timeTotal = String(format: timeType, (secTotal / 60), secTotal % 60)
            if showTotalTime == true {
                if secTotal <= 0 || secRemaining <= 0 {
                    if threeDigits {
                        timeDisplay = " \(timeElapsed)| \n ---.--|total"
                    } else {
                        timeDisplay = "  \(timeElapsed)|\n  --.--|total"
                    }
                } else {
                    if threeDigits {
                        timeDisplay = " \(timeElapsed)|  \n \(timeTotal)|total"
                    } else {
                        timeDisplay = "  \(timeElapsed)|\n  \(timeTotal)|total"
                    }
                }
            } else if showTotalTime == false {
                if secTotal <= 0 || secRemaining <= 0 {
                    if threeDigits {
                        timeDisplay = " \(timeElapsed)| \n ---.--|remain"
                    } else {
                        timeDisplay = "  \(timeElapsed)|\n  --.--|remain"
                    }
                } else {
                    if threeDigits {
                        timeDisplay = " \(timeElapsed)|  \n \(timeRemaining)|remain"
                    } else {
                        timeDisplay = "  \(timeElapsed)|\n  \(timeRemaining)|remain"
                    }
                }
            }
        }
        stringValue = timeDisplay
    }

    override func mouseDown(with event: NSEvent) {
        showTotalTime = !showTotalTime
        UserDefaults.standard.set(showTotalTime, forKey: "timerShowTimeRemaining")
        update()
    }

    func setMaxDoubleValue(_ v: Double) {
        maxValue = v
        update()
    }

    func setDoubleValue(_ v: Double) {
        value = v
        update()
    }
}
