import AppKit

class ToolTipWindow: NSPanel {
    // returns the approximate window size needed to display the tooltip string.
    var closeTimer     : RepeatingTimer?
    var tooltipObject  : Any?
    var textAttributes : NSDictionary?
    
    func suggestedSize(forTooltip tooltip: Any) -> NSSize {
        var tipSize: NSSize = NSZeroSize
        if tooltip is NSAttributedString {
            tipSize = (tooltip as AnyObject).size()
        } else if tooltip is String {
            tipSize = (tooltip as AnyObject).size(withAttributes: textAttributes as? [NSAttributedString.Key: Any])
        }
        
        if !NSEqualSizes(tipSize, NSZeroSize) {
            tipSize.width += 8
            tipSize.height += 2
        }
        return tipSize
    }
    
    init() {
        super.init(contentRect: NSMakeRect(0, 0, 0, 0), styleMask: .hudWindow, backing: .buffered, defer: false)
        isOpaque = false
        hasShadow = true
        level = .popUpMenu
        hidesOnDeactivate = true
        ignoresMouseEvents = true
        isReleasedWhenClosed = false
        // textfield setup...
        let field = NSTextField(frame: NSMakeRect(0, 0, 0, 0))
        if #available(OSX 10.14, *) {
            field.backgroundColor = NSColor.windowBackgroundColor
            field.textColor = NSColor.white
        }
        else {
            field.backgroundColor = BackgroundColorA
            field.textColor = MeterBarColor
        }
        field.isBordered = true
        
        field.isEditable = false
        field.isSelectable = false
        field.isBezeled = true
        field.drawsBackground = true
        field.font = NSFont.systemFont(ofSize: CGFloat(11.0))
        field.usesSingleLineMode = false
        contentView = field
        setFrame(frameRect(forContentRect: field.frame), display: false)
        field.stringValue = " "
        // Just having at least 1 char to allow the next message...
        textAttributes = field.attributedStringValue.attributes(at: 0, effectiveRange: nil) as NSDictionary
    }
    
    func setToolTip(_ tip: Any) {
        let contentView = self.contentView
        tooltipObject = tip
        if contentView is NSTextField {
            if tip is String {
                (contentView as! NSTextField).stringValue = tip as! String
            } else if tip is NSAttributedString {
                (contentView as! NSTextField).attributedStringValue = tip as! NSAttributedString
            }
        }
    }
    
    func orderFront(forDuration duration: TimeInterval) {
        orderFront(nil)
        closeTimer = RepeatingTimer(timeInterval: duration)
        closeTimer!.eventHandler = {
            DispatchQueue.main.async {
                // only back on the main thread, may you access UI:
                self.close()
            }
        }
        closeTimer!.resume()
    }
    
    func orderFront() {
        if closeTimer != nil {
            closeTimer?.suspend()
        }
        super.orderFront(nil)
    }
}

class ToolTipTextField: NSTextField {
    func setObjectValue(_ obj: Any) {
        var string: String = (obj as AnyObject).description
        let formatter: Formatter? = cell?.formatter
        if nil != formatter {
            string = (formatter?.string(for: obj))!
        }
        toolTip = string
        super.objectValue = obj
    }
}
