import AppKit

class VolumeSlider: NSSlider {
    
    var ToolTip: ToolTipWindow!

    override init(frame: NSRect) {
        super.init(frame: frame)
    }

    required init?(coder aDecoder: NSCoder) {
        super.init(coder: aDecoder)
        ToolTip = ToolTipWindow()
    }

    override func sendAction(_ action: Selector?, to target: Any?) -> Bool {
        showToolTip(forDuration: 1.0)
        return super.sendAction(action, to: target)
    }

    override func scrollWheel(with event: NSEvent) {
        let change = Double((event.deltaY / 30.0))
        doubleValue += change
        _ = sendAction(action, to: target)
        showToolTip(forDuration: 1.0)
    }
    
    func sliderToMixer(slider: Double) -> Double {
        return slider * slider * slider
    }
    
    func mixerTodB(value: Double) -> String {
        return String(format: "%0.1f dB", 20 * log10(value))
    }

    func updateToolTip() {
        let dBtext = mixerTodB(value: sliderToMixer(slider: doubleValue))
        let size: NSSize = ToolTip.suggestedSize(forTooltip: dBtext)

        let mouseLocation: NSPoint = NSEvent.mouseLocation
        ToolTip.setToolTip(dBtext)
        ToolTip.setFrame(NSMakeRect(mouseLocation.x, mouseLocation.y + 14.0, size.width, size.height), display: true)
    }

    func showToolTip() {
        updateToolTip()
        ToolTip.orderFront()
    }

    func showToolTip(forDuration duration: TimeInterval) {
        updateToolTip()
        ToolTip.orderFront(forDuration: duration)
    }

    func hideToolTip() {
        ToolTip.close()
    }
}
