import Foundation
import AppKit

class VolumeSliderCell: NSSliderCell {
    
    var trackColor  : NSColor = MeterScaleColor.blended(withFraction: 0.4, of: NSColor.clear)!
    var sliderColor : NSColor = MeterScaleColor
    
    required init(coder aDecoder: NSCoder) {
        super.init(coder: aDecoder)
    }
    
    override init() {
        super.init()
    }
    
    override func knobRect(flipped: Bool) -> NSRect {
        return NSRect.zero
    }
    
    override func drawBar(inside aRect: NSRect, flipped: Bool) {
        let trackRect = aRect
        
        let barRadius = trackRect.size.width / 2.0
        let value = CGFloat((doubleValue - minValue) / (maxValue - minValue))
        //
        // we have a vertical slider and it's flipped
        //
        if isVertical {
            // not used
            if flipped {
                let activeLength = CGFloat(value * trackRect.size.height)
                var activeRect = trackRect
                activeRect.size.height = activeLength
                activeRect.origin.y = trackRect.origin.y + trackRect.size.height - activeLength
                
                let track = NSBezierPath(roundedRect: trackRect, xRadius: barRadius, yRadius: barRadius)
                trackColor.setFill()
                track.fill()
                
                let active = NSBezierPath(roundedRect: activeRect, xRadius: barRadius, yRadius: barRadius)
                sliderColor.setFill()
                active.fill()
            } else {
                let activeLength = CGFloat(value * trackRect.size.height)
                var activeRect = trackRect
                activeRect.size.height = activeLength
                
                let track = NSBezierPath(roundedRect: trackRect, xRadius: barRadius, yRadius: barRadius)
                trackColor.setFill()
                track.fill()
                
                let active = NSBezierPath(roundedRect: activeRect, xRadius: barRadius, yRadius: barRadius)
                sliderColor.setFill()
                active.fill()
            }
        } else {
            let barRadius = trackRect.size.height / 2.0
            let activeLength = CGFloat(value * trackRect.size.width)
            var activeRect = trackRect
            activeRect.size.width = activeLength
            
            let track = NSBezierPath(roundedRect: trackRect, xRadius: barRadius, yRadius: barRadius)
            
            trackColor.setFill()
            track.fill()
            
            let active = NSBezierPath(roundedRect: activeRect, xRadius: barRadius, yRadius: barRadius)
            sliderColor.setFill()
            active.fill()
        }
    }
}
