import AppKit

class WaveSlider: NSSlider {
    @IBOutlet var positionTextField: TimeField!

    required init?(coder aDecoder: NSCoder) {
        super.init(coder: aDecoder)
    }

    override var doubleValue: Double {
        get {
            return super.doubleValue
        }
        set {
            super.doubleValue = newValue
            positionTextField.setDoubleValue(newValue)
        }
    }

    override var maxValue: Double {
        get {
            return super.maxValue
        }
        set {
            super.maxValue = newValue
            positionTextField.setMaxDoubleValue(newValue)
        }
    }

    convenience init() {
        self.init()
    }

    convenience init(knobImage knob: NSImage?) {
        self.init()
        cell = WaveSliderCell(knobImage: knob)
    }

    func setKnobImage(_ image: NSImage?) {
        (cell as? WaveSliderCell)?.knobImage = image
    }
}

class WaveSliderCell: NSSliderCell {
    var knobImage: NSImage?

    convenience init?(knobImage knob: NSImage?) {
        if nil == knob {
            return nil
        }
        self.init()
        knobImage = knob
    }

    override func drawKnob(_ knobRect: NSRect) {}

    override func drawBar(inside cellFrame: NSRect, flipped: Bool) {}

    override func knobRect(flipped: Bool) -> NSRect {
        let WIDTH = knobImage!.size.width
        let HEIGHT = knobImage!.size.height
        let value = CGFloat((doubleValue - Double(minValue)) / Double((maxValue - minValue)))
        let defaultRect: NSRect = super.knobRect(flipped: flipped)
        var myRect: NSRect = NSMakeRect(0, 0, 0, 0)
        if (controlView as? NSSlider)?.isVertical == true {
            // ...
        } else {
            myRect.size.width = CGFloat(WIDTH)
            myRect.size.height = CGFloat(HEIGHT)
            myRect.origin.x = value * ((controlView?.frame.size.width ?? 0.0) - CGFloat(WIDTH))
            myRect.origin.y = (defaultRect.origin.y - 2) + defaultRect.size.height / 2.0 - myRect.size.height / 2.0 // Fixed the position
        }
        return myRect
    }
}
