import AppKit

class Shuffle: NSObject {
    override init() {
        var initialized = false
        if !initialized {
            // Call srandom() exactly once.
            srandom(UInt32(UInt(time(nil))))
            initialized = true
        }
    }

    class func shuffleList(_ l: NSArray) -> NSMutableArray {
        let randomLongs = NSArray.randomLongs(count: l.count)
        // randomLongs is an array of random integers, equal in length to l.
        let pairs = NSArray.zip(randomLongs, withArray: l)
        // randomLongs and l are paired.
        let shuffledPairs = pairs.sortedArray {
            (obj1, obj2) -> ComparisonResult in
            let p1 = obj1 as! NSArray
            let p2 = obj2 as! NSArray
            let result = (p1.object(at: 0) as AnyObject).compare(p2.object(at: 0))
            return result
        }
        // The numbers from randomLongs are sorted in ascending order; the tracks from l
        // are in random order.
        // Peel the tracks off and return them.
        return NSArray.unzipArray(shuffledPairs as NSArray).object(at: 1) as! NSMutableArray
    }
}

extension NSArray {
    /*
     * Generates an array of random long integers in the range
     * 0 to (2**31) - 1.  The length of the array is determined
     * by the count parameter.
     */
    class func randomLongs(count: Int) -> NSArray {
        let randomLongs = NSMutableArray(capacity: count)
        for _ in 0 ..< count {
            randomLongs.add(arc4random())
        }
        return randomLongs
    }

    class func zip(_ x: NSArray, withArray y: NSArray) -> NSArray {
        let xCount: Int = x.count
        let yCount: Int = y.count
        let minCount: Int = (xCount < yCount) ? xCount : yCount
        let pairs = NSMutableArray(capacity: minCount)
        for i in 0 ..< minCount {
            let p = [x[i], y[i]]
            pairs.add(p as Any)
        }
        return pairs
    }

    class func unzipArray(_ pairs: NSArray) -> NSArray {
        let first = NSMutableArray(capacity: pairs.count)
        let second = NSMutableArray(capacity: pairs.count)
        for pair in pairs {
            first.add((pair as! NSArray).first())
            second.add((pair as! NSArray).second())
        }
        return [first, second]
    }

    func first() -> Any {
        return object(at: 0)
    }

    func second() -> Any {
        return object(at: 1)
    }
}
