import AppKit

class CuteRadioController: NSController, NSWindowDelegate, NSTableViewDataSource, NSTableViewDelegate, CuteRadioSearchDelegate {
    
    @IBOutlet var playlistLoader               : PlaylistLoader!
    @IBOutlet var playbackController           : PlaybackController!
    @IBOutlet var cuteRadioSearchPanel         : NSPanel!
    @IBOutlet var cuteRadioCountrySelector     : NSPopUpButton!
    @IBOutlet var cuteRadioGenreSelector       : NSPopUpButton!
    @IBOutlet var stopSearchBtn                : NSButton!
    @IBOutlet var cuteRadioSearchTable         : NSTableView!
    @IBOutlet var cuteRadioButton              : NSButton?
    
    var cuteradioSearch : CuteradioSearch!
    var closeOnSearch   = true
    
    override func awakeFromNib() {
        initCuteRadioSearch()
        cuteRadioButton?.isEnabled = false
        if isConnectedToNetwork() {
        requestGenres()
        }
    }
    
    func initCuteRadioSearch() {
        cuteradioSearch = CuteradioSearch()
        cuteradioSearch.delegate = self
    }
    
    func requestGenres() {
        self.cuteradioSearch.getGenreList()
    }
    
    func requestCoutries() {
        self.cuteradioSearch.getCountryList()
    }
    
    @IBAction func openCloseSearchPanel(_ sender: AnyObject) {
        if cuteRadioSearchPanel.isVisible {
            cuteRadioSearchPanel.close()
        } else {
            cuteRadioSearchPanel.orderFront(sender)
            if cuteRadioSearchTable.numberOfRows == 0
               && cuteRadioGenreSelector.selectedCell()?.title == "All" {
                cuteRadioCountrySelector.selectItem(withTitle: "Netherlands")
                cuteRadioGenreSelector.selectItem(withTitle: "All")
            }
            if closeOnSearch {
                cuteRadioGenreAndCountrySearch(sender)
            }
        }
    }
    
    @IBAction func cuteRadioGenreAndCountrySearch(_ sender: Any) {
        var country: String = cuteRadioCountrySelector.titleOfSelectedItem ?? ""
        if country == "All" {
            country = ""
        }
        var genre = cuteRadioGenreSelector.titleOfSelectedItem ?? ""
        if genre == "All" {
            genre = ""
        }
        stopSearchBtn.isEnabled = true
        cuteradioSearch.searchBy(genre: genre, country: country)
    }
    
    @IBAction func stopSearch(_ sender: AnyObject) {
        closeOnSearch = cuteradioSearch.stopSearch()
    }
    
    func stopSearchButton(enabled: Bool) {
        stopSearchBtn.isEnabled = enabled
    }
    
    @IBAction func addCuteStationToPlaylist(_ sender: AnyObject) {
        let row: Int = sender.selectedRow!
        if row > -1 {
            var urls = [URL]()
            if let station = cuteradioSearch[row] {
                if let stationURL = URL(string: station.url!) {
                    urls.append(stationURL)
                    DispatchQueue.main.async {
                        let insertedEntries = self.playlistLoader.addURLs(urls, sort: false)
                        //print("CRP addCuteStationToPlaylist:", insertedEntries as Any )
                        if !insertedEntries!.isEmpty {
                            _ = self.playbackController.play(entry: insertedEntries![0])
                        }
                    }
                }
            }
        }
    }
    //
    // TableView Delegate funtions
    //
    func numberOfRows(in tableView: NSTableView) -> Int {
        return cuteradioSearch.stationsCount()
    }
    
    func tableView(_ tableView: NSTableView, objectValueFor tableColumn: NSTableColumn?, row: Int) -> Any? {
        if let station = cuteradioSearch[row] {
            switch tableColumn?.identifier.rawValue {
            case "index"    : return row
            case "title"    : return station.title
            case "url"      : return station.url
            case "genre"    : return station.genre
            case "country"  : return station.country
            default: return nil
            }
        }
        return nil
    }
    //
    // widowdelegate functions
    //
    func windowWillClose(_ notification: Notification) {
        closeOnSearch = cuteradioSearch.stopSearch()
    }
    //
    // searchDelagete functions
    //
    func searchInProgress(_ state: Bool) {
        stopSearchBtn.isEnabled = state
    }
    
    func genresLoaded(_ genres: [String]) {
        cuteRadioGenreSelector.removeAllItems()
        cuteRadioGenreSelector.addItem(withTitle: "All")
        cuteRadioGenreSelector.addItems(withTitles: genres)
        requestCoutries()
    }
    
    func countriesLoaded(_ countries: [String]) {
        cuteRadioCountrySelector.removeAllItems()
        cuteRadioCountrySelector.addItem(withTitle: "All")
        cuteRadioCountrySelector.addItems(withTitles: countries)
        cuteRadioButton?.isEnabled = true
    }
    
    func reloadData() {
        cuteRadioSearchTable.reloadData()
        //
        // scroll to end is annoying
        //
        //cuteRadioSearchTable.scrollToEndOfDocument(self)
    }
}

