protocol ShoutcastSearchDelegate {
    func searchInProgress(_ state: Bool)
    func reloadData()
}

class ShoutcastSearch {
    // get genrelist
    // http://api.shoutcast.com/legacy/genrelist?k=[Your Dev ID]
    //
    private let searchStationUrl    : String = "https://api.shoutcast.com/legacy/stationsearch?k=%@&search=%@"
    private let searchAdvancedXml   : String = "https://api.shoutcast.com/station/advancedsearch?f=xml&k=%@"
    
    var SHOUTcastAPIkey : String
    private var stations: NSMutableArray = []
    private var selectedBitrate     : String = "All"
    var delegate: ShoutcastSearchDelegate?
    
    init(apiKey: String) {
        SHOUTcastAPIkey = apiKey
    }
    /**
     Search for radiostations within a genre or all genres and a bitrate.
     - parameters:
     - genre: The genre to search for or All for everything.
     - bitrate: Search only stations with a specific bitrate or All.
     */
    
    func searchBy(genre: String?, bitrate: String?) {
        var search : String!
        selectedBitrate = bitrate!
        search = String(format: searchAdvancedXml, SHOUTcastAPIkey)
        if bitrate != "All" {
            search += String(format: "&br=%@" , bitrate!)
        }
        if genre != "All" {
            search += String(format: "&search=%@", genre!)
        }
        if genre == "All" && bitrate == "All" {
            search += String(format: "&search=%@", genre!)
        }
        doRequest(searchUrl: search, useAdvanced: true)
    }
    
    func searchStation(searchFor: String) {
        if searchFor.isEmpty { return }
        let encodedSearch: String = searchFor.addingPercentEncoding(withAllowedCharacters: .urlPathAllowed)!
        var search       : String = String(format: searchStationUrl, SHOUTcastAPIkey, encodedSearch)
        if selectedBitrate != "All" {
            search += String(format: "&br=%@" , selectedBitrate)
        }
        doRequest(searchUrl: search)
    }
    
    func stationsCount() -> Int {
        return stations.count
    }
    
    subscript(index: Int) -> ShoutCastRadioStation? {
        if index >= stations.count { return nil }
        return stations[index] as? ShoutCastRadioStation
    }
    
    private func doRequest(searchUrl: String, useAdvanced: Bool = false) {
        let temp_stations: NSMutableArray = []
        let url = URL(string: searchUrl)!
        URLSession.shared.dataTask(with: url, completionHandler:
            { (data, response, error) -> Void in
                if error != nil { return }
                
                if let data = data {
                    let xml = XML.parse(data)
                    let stationlist = useAdvanced ? xml["response"]["data"]["stationlist"]["station"] : xml["stationlist"]["station"]
                    if stationlist.all != nil {
                        for (_, node) in (stationlist.all?.enumerated())! {
                            let shoutcastRadioStation: ShoutCastRadioStation = ShoutCastRadioStation()
                            if let name = node.attributes["name"] {
                                shoutcastRadioStation.title = name
                            }
                            if let currentplaying = node.attributes["ct"] {
                                shoutcastRadioStation.currentlyPlaying = currentplaying
                            }
                            if let genre = node.attributes["genre"] {
                                shoutcastRadioStation.genre = genre
                            }
                            if let id = node.attributes["id"] {
                                shoutcastRadioStation.id = id
                            }
                            if let bitrate = node.attributes["br"] {
                                shoutcastRadioStation.bitrate = bitrate
                            }
                            if let logo = node.attributes["logo"] {
                                shoutcastRadioStation.logo = logo
                            }
                            temp_stations.add(shoutcastRadioStation)
                        }
                        temp_stations.sort(comparator: { (lhr, rhr) -> ComparisonResult in
                            let l = lhr as! ShoutCastRadioStation
                            let r = rhr as! ShoutCastRadioStation
                            return l.title!.lowercased().compare(r.title!.lowercased())
                        })
                    } else {
                        //                        print("SCS_no stations found for search parameters")
                    }
                } else {
                    //                    print("SCS_Error: no data in response")
                }
                self.stations = temp_stations
                DispatchQueue.main.async {
                    self.delegate?.reloadData()
                }
        }).resume()
    }
}
