import AudioUnit.AUCocoaUIView
import CoreAudioKit

@objcMembers
class AUController: NSPanel {
    @IBOutlet var fxBypassButton: NSButton!
    @IBOutlet var fxSelector: NSPopUpButton!
    @IBOutlet var loadButton: NSButton!
    @IBOutlet var saveButton: NSButton!
    
    var auDescList = [AudioComponentDescription]()
    var effectUnit: AudioUnit?
    var AUView: NSView!
    var auNameList: NSMutableArray = []
    
    init(title: String) {
        super.init(contentRect: NSMakeRect(0, 0, 265, 63), styleMask: [.utilityWindow, .hudWindow, .closable, .titled], backing: .buffered, defer: true)
        
        self.title = title
        initFxWindow()
    }
    
    func initFxWindow() {
        //
        //  selector window
        //
        isReleasedWhenClosed = false
        isFloatingPanel = true
        hasShadow = true
        center()
        //
        //  bypass button
        //
        fxBypassButton = NSButton(frame: NSMakeRect(232, 13, 22, 22))
        fxBypassButton.setButtonType(NSButton.ButtonType.switch)
        
        fxBypassButton.imageScaling = .scaleProportionallyDown
        fxBypassButton.image = NSImage(named: NSImage.Name("Bypass_on"))
        fxBypassButton.alternateImage = NSImage(named: NSImage.Name("Bypass_off"))
        fxBypassButton.allowsMixedState = false
        // no effect
        fxBypassButton.isBordered = false
        fxBypassButton.target = self
        fxBypassButton.action = #selector(fxBypassBtnAction(_:))
        fxBypassButton.state = UserDefaults.standard.bool(forKey: "BypassFX") ? .on : .off
        fxBypassButton.toolTip = fxBypasButtonTooltip
        //
        //  fx selector
        //
        fxSelector = NSPopUpButton(frame: NSMakeRect(11, 11, 216, 22))
        //
        // Load settings button
        loadButton = NSButton(frame: NSMakeRect(256, 11, 48, 22))
        loadButton.setButtonType(NSButton.ButtonType.momentaryPushIn)
        loadButton.title = "Load"
        loadButton.bezelStyle = .smallSquare
        loadButton.isBordered = false
        loadButton.target = self
        loadButton.action = #selector(load)
        loadButton.toolTip = loadButtonTooltip
        // Save settings button
        saveButton = NSButton(frame: NSMakeRect(306, 11, 48, 22))
        saveButton.setButtonType(NSButton.ButtonType.momentaryPushIn)
        saveButton.title = "Save"
        saveButton.bezelStyle = .smallSquare
        saveButton.isBordered = false
        saveButton.target = self
        saveButton.action = #selector(save)
        saveButton.toolTip = saveButtonTooltip
        //
        //  init AU
        //
        getAuList(fxSelector)
        fxSelector.target = self
        fxSelector.action = #selector(fxSelectorAction(_:))
        fxSelector.toolTip = fxSelectorTooltip
        contentView?.addSubview(fxSelector)
        contentView?.addSubview(fxBypassButton)
        contentView?.addSubview(loadButton)
        contentView?.addSubview(saveButton)
        resizeFxWindow()
        loadButton.isEnabled = false
        saveButton.isEnabled = false
    }
    
    func load() {
        loadAuSettings()
        // Fix for Apple AU FX loading settings and not updating UI
        if title.contains("Apple") {
            AUView.removeFromSuperview()
            AUView = nil
            initAuView()
        }
    }
    
    func save() {
        saveAuSettings()
    }
    
    func fxBypassBtnAction(_ sender: NSButton) {
        //
        //  selector of fxBypassBtn
        //  on state : Active
        //  off state: Bypass
        //
        fxBypass(!(sender.state.rawValue as NSNumber).boolValue)
    }
    
    func fxBypassBtnEnable(_ state: Bool) {
        fxBypassButton.isEnabled = state
        if state {
            fxBypassButton.image = NSImage(named: NSImage.Name("Bypass_on"))
        } else {
            fxBypassButton.image = NSImage(named: NSImage.Name("Bypass_on_noFX"))
        }
    }
    
    func fxBypass(_ state: Bool) {
        UserDefaults.standard.set(state, forKey: "BypassFX")
        //
        //  on state : Active  // off state: Bypass
        //
        fxBypassButton.state = NSControl.StateValue(rawValue: Int(truncating: NSNumber(value: !state)))
        var Onvalue = UInt32(Int(truncating: NSNumber(value: state)))
        if fxUnit() != nil {
            AudioUnitSetProperty(fxUnit()!,
                                 kAudioUnitProperty_BypassEffect,
                                 kAudioUnitScope_Global,
                                 0,
                                 &Onvalue,
                                 UInt32(MemoryLayout.size(ofValue: Onvalue)))
        }
    }
    
    // MARK: AUPop
    func bannedAUComponents() -> NSArray {
        return ["Apple: AUNetSend", "Apple: AURoundTripAAC", "Apple: AURogerBeep", "Apple: AUSampleDelay", "Martin Zuther: K-Meter", "Apple: AUNewPitch"]
    }
    
    func toggleFxWindow() {
        if !isVisible {
            orderFrontRegardless()
        } else {
            close()
        }
    }
    
    func componentCount(forAuType inAuType: OSType) -> UInt32 {
        var desc = AudioComponentDescription(componentType: inAuType, componentSubType: 0, componentManufacturer: 0, componentFlags: 0, componentFlagsMask: 0)
        return AudioComponentCount(&desc)
    }
    
    func getAuComponentName(_ inDesc: AudioComponentDescription) -> String {
        var inDesc = inDesc
        let auComponent: AudioComponent = AudioComponentFindNext(nil, &inDesc)!
        var name: Unmanaged<CFString>?
        AudioComponentCopyName(auComponent, &name)
        return (name?.takeUnretainedValue() as String?)!
    }
    
    func getStereoComponents(forAuType inAuType: OSType, count: UInt32) {
        var last: AudioComponent?
        var name: Unmanaged<CFString>?
        let waves: OSType = "ksWV".OSType()
        var manufacturer: OSType?
        //
        // temporarily commented out stuff to filter out waves, because waves seems to have a problem loading the GUI.
        //
        var subType: OSType?
        var lastCharOfSubType: Int8 = 0
        let stereo = 83
        // ASCII 83 = S for stereo
        let Lim1: OSType = "LIM1".OSType() // the only waves plugin that does not comply to the last letter = S standard
        var desc = AudioComponentDescription()
        var fxDesc: AudioComponentDescription = AudioComponentDescription(componentType: inAuType, componentSubType: 0, componentManufacturer: 0, componentFlags: 0, componentFlagsMask: 0)
        for _ in 0 ..< Int(count) {
            last = AudioComponentFindNext(last, &fxDesc)
            AudioComponentGetDescription(last!, &desc)
            //
            //  we don't want Waves plugins other than Stereo.
            //
            subType = desc.componentSubType
            let subtype = UTCreateStringForOSType(subType!)
            lastCharOfSubType = Int8(CFStringGetCharacterAtIndex(subtype.takeUnretainedValue(), 3))
            manufacturer = desc.componentManufacturer
            //
            // manufacturer: WAVES & stereo only: subType ends with S
            //
            if (manufacturer == waves && (Int(lastCharOfSubType) == stereo || subType == Lim1)) || manufacturer != waves {
                //            if manufacturer != waves {
                AudioComponentCopyName(last!, &name)
                let namestring: NSString = (name?.takeUnretainedValue() as NSString?)!
                if !(bannedAUComponents().contains(namestring)) {
                    auDescList.append(desc)
                    auNameList.add(namestring)
                }
            }
        }
    }
    
    func sortAuListByName() {
        //
        //  insertionSort method: http://www.knowstack.com/sorting-algorithms-in-objective-c/
        //
        let unsortedDataArray: NSMutableArray = auNameList
        let count: Int = unsortedDataArray.count
        var tempDesc: AudioComponentDescription = AudioComponentDescription()
        var j: Int = 0
        for i in 1 ..< count {
            j = i
            while j > 0 && ((unsortedDataArray.object(at: j - 1) as AnyObject).caseInsensitiveCompare((unsortedDataArray[j] as? String)!) == .orderedDescending) {
                unsortedDataArray.exchangeObject(at: j, withObjectAt: j - 1)
                tempDesc = auDescList[j]
                auDescList[j] = auDescList[j - 1]
                auDescList[j - 1] = tempDesc
                j -= 1
            }
        }
        auNameList = unsortedDataArray
    }
    
    func getAuList(_ inPop: NSPopUpButton) {
        let auType: OSType = kAudioUnitType_Effect
        let pluginCount = componentCount(forAuType: auType)
        getStereoComponents(forAuType: auType, count: pluginCount)
        sortAuListByName()
        let stereoPluginCount = auNameList.count
        inPop.removeAllItems()
        inPop.addItem(withTitle: "No Effect")
        inPop.menu?.addItem(NSMenuItem.separator())
        for index in 0 ..< stereoPluginCount {
            inPop.menu?.addItem(withTitle: (auNameList[index] as? String)!, action: nil, keyEquivalent: "")
        }
    }
    
    // Called from AudioEngineWrapper on init of this Class
    func restoreAu() {
        //
        //  recall last used effect and set the popselector (at startup)
        //
        if UserDefaults.standard.object(forKey: "selectedAU.componentsubType") != nil {
            let compSubType = UTGetOSTypeFromString(UserDefaults.standard.object(forKey: "selectedAU.componentsubType") as! CFString)
            let compManufacturer = UTGetOSTypeFromString(UserDefaults.standard.object(forKey: "selectedAU.componentManufacturer") as! CFString)
            //
            //  we need to find the name in auNameList, and get the index
            //
            let desc: AudioComponentDescription = AudioComponentDescription(componentType: 0, componentSubType: compSubType, componentManufacturer: compManufacturer, componentFlags: 0, componentFlagsMask: 0)
            let index: Int = auNameList.index(of: getAuComponentName(desc))
            fxSelector.selectItem(at: index + 2)
            initAndLaunchAu(inDesc: desc)
            fxBypassBtnEnable(true)
            load()
        } else {
            fxBypassBtnEnable(false)
        }
        fxBypass(UserDefaults.standard.bool(forKey: "BypassFX"))
        
        auNameList = []
    }
    
    func fxSelectorAction(_ sender: NSPopUpButton) {
        if sender.indexOfSelectedItem == 0 {
            //
            //  close loaded effect
            //
            if SFBisPlaying() || StreamPlayer.state.rawValue == 3 {
                removeEffectWhilePlaying()
                closeAndDisposeAu()
            } else {
                removeEffect()
                closeAndDisposeAu()
            }
            //
            //  the view is removed, adjust fxSelector window size
            //
            resizeFxWindow()
            UserDefaults.standard.set(nil, forKey: "selectedAU.componentsubType")
            UserDefaults.standard.set(nil, forKey: "selectedAU.componentManufacturer")
            //
            //  change bypass state to true (inactive), there is no effect selected
            //
            fxBypass(true)
            loadButton.isEnabled = false
            saveButton.isEnabled = false
        } else if sender.indexOfSelectedItem > 0 {
            //
            //  remove effect, if present
            //
            if SFBisPlaying() || StreamPlayer.state.rawValue == 3 {
                removeEffectWhilePlaying()
                closeAndDisposeAu()
            } else {
                removeEffect()
                closeAndDisposeAu()
            }
            //
            //  get componentDescription from list
            //
            let desc: AudioComponentDescription = auDescList[(sender.indexOfSelectedItem) - 2]
            //
            // lauch the new effect, and store description in userdefaults
            //
            initAndLaunchAu(inDesc: desc)
            let compSubType = UTCreateStringForOSType(desc.componentSubType)
            UserDefaults.standard.set(compSubType.takeUnretainedValue() as String, forKey: "selectedAU.componentsubType")
            
            let compManufacturer = UTCreateStringForOSType(desc.componentManufacturer)
            UserDefaults.standard.set(compManufacturer.takeUnretainedValue() as String, forKey: "selectedAU.componentManufacturer")
            //
            //  change bypass state to false, so the selected effect will be made active
            //
            fxBypass(false)
        }
    }
    
    func initAndLaunchAu(inDesc: AudioComponentDescription) {
        title = getAuComponentName(inDesc)
        fxBypassBtnEnable(true)
        if addEffect(inDesc) { initAuView() }
        if !isVisible { orderFrontRegardless() }
    }
    
    func plugInClassIsValid(_ pluginClass: AnyClass) -> Bool {
        if pluginClass.conforms(to: AUCocoaUIBase.self) {
            if pluginClass.instancesRespond(to: #selector(AUCocoaUIBase.interfaceVersion)) && pluginClass.instancesRespond(to: #selector(AUCocoaUIBase.uiView(forAudioUnit:with:))) {
                return true
            }
        }
        return false
    }
    
    func displayAuView(_ aView: NSView?) {
        if aView == nil { return }
        AUView = NSView(frame: NSMakeRect(0, 44, aView!.frame.size.width, aView!.frame.size.height))
        AUView.addSubview(aView!)
        contentView?.addSubview(AUView)
        resizeFxWindow()
    }
    
    func resizeFxWindow() {
        var frameRect : NSRect = NSMakeRect(frame.origin.x,
                                            frame.origin.y,
                                            265.0,
                                            63)
        if AUView != nil {
            frameRect = NSMakeRect(frame.origin.x,
                                   frame.origin.y,
                                   (frameRect.width  > AUView.frame.size.width  ? frameRect.width  : AUView.frame.size.width),
                                   (frameRect.height > AUView.frame.size.height ? frameRect.height : AUView.frame.size.height + 63))
        }
        setFrame(frameRect, display: false, animate: true)
    }
    
    func closeAndDisposeAu() {
        //
        // remove the view first
        //
        if AUView != nil {
            AUView.removeAllToolTips()
            AUView.removeFromSuperview()
            //
            // then nil the view, without it the View is still visible, but not responding
            //
            AUView = nil
        }
        title = "FX Selector"
        fxBypassBtnEnable(false)
    }
    
    func genericViewCreation() {
        let AUView: NSView? = AUGenericView(audioUnit: fxUnit()!)
        (AUView as? AUGenericView)?.showsExpertParameters = true
        displayAuView(AUView)
    }
    
    func initAuView() {
        // get AU's Cocoa view property
        var result              : OSStatus
        var dataSize            : UInt32 = 0
        var isWritable          : DarwinBoolean = false
        var cocoaViewBundlePath : NSURL?
        var viewFactoryName     : NSString?
        var cocoaViewInfo       : AudioUnitCocoaViewInfo?
            
            result = AudioUnitGetPropertyInfo(fxUnit()!,
                                          kAudioUnitProperty_CocoaUI,
                                          kAudioUnitScope_Global,
                                          0,
                                          &dataSize,
                                          &isWritable )
        // Does view have custom Cocoa UI?
        if (result == noErr) && (Int(dataSize) > 0) {
            //
            // AudioUnitCocoaViewInfo:  The name and number of custom Cocoa views for an audio unit.
            //
            result = AudioUnitGetProperty(fxUnit()!,
                                          kAudioUnitProperty_CocoaUI,
                                          kAudioUnitScope_Global,
                                          0,
                                          &cocoaViewInfo,
                                          &dataSize)
            if result == noErr {
                cocoaViewBundlePath = cocoaViewInfo?.mCocoaAUViewBundleLocation.takeUnretainedValue()
                
                // there is typicaly one view.
                viewFactoryName = cocoaViewInfo?.mCocoaAUViewClass.takeUnretainedValue()
            }
        }
        var auView: NSView!
        var wasAbleToLoadCustomView = false
        //
        // [A] Show custom UI if view has it
        //
        if cocoaViewBundlePath != nil && viewFactoryName != nil {
            let viewBundle = Bundle(path: (cocoaViewBundlePath?.path)!)
            if viewBundle != nil {
                
                let factoryClass: AnyClass = (viewBundle?.classNamed(viewFactoryName! as String).self)!
                
                if plugInClassIsValid(factoryClass) {
                    // make a factory
                    let factoryInstance = factoryClass.alloc() as! AUCocoaUIBase
                    // make a view, size doesn't matter
                    auView = factoryInstance.uiView(forAudioUnit: fxUnit()!, with: NSZeroSize)
                    // Display View!
                    if auView != nil {
                        wasAbleToLoadCustomView = true
                        displayAuView(auView)
                    }
                }
            }
        }
        if !wasAbleToLoadCustomView {
            // [B] Otherwise show generic Cocoa view (if needed!)
            genericViewCreation()
        }
        if cocoaViewInfo != nil {
            cocoaViewInfo = nil
        }
        loadButton.isEnabled = true
        saveButton.isEnabled = true
    }
    
    func loadAuSettings() {
        if fxUnit() == nil { return }
        let prefsSub = String(format: "Phonix-%@-settings", title.replacingOccurrences(of: ": ", with: "-"))
        let prefsPath = PhonixPrefsFolder.appendingPathComponent(prefsSub).path
        if PhonixFileManager.fileExists(atPath: prefsPath) {
            var settingsData = NSDictionary(contentsOfFile: prefsPath)
            let count = CFDictionaryGetCount(settingsData)
            if count > 0 {
                AudioUnitSetProperty(fxUnit()!,
                                     kAudioUnitProperty_ClassInfo,
                                     kAudioUnitScope_Global,
                                     0,
                                     &settingsData,
                                     UInt32(MemoryLayout<CFPropertyList?>.size))
            }
        }
    }
    
    func saveAuSettings() {
        if fxUnit() == nil { return }
        var err: OSStatus = noErr
        var settingsData: CFPropertyList?
        var size = UInt32(MemoryLayout<CFPropertyList?>.size)
        err = AudioUnitGetProperty(fxUnit()!,
                                   kAudioUnitProperty_ClassInfo,
                                   kAudioUnitScope_Global,
                                   0,
                                   &settingsData,
                                   &size)
        if settingsData != nil && err == noErr {
            // be aware, some of no-licensed plugins refuse saving!
            let prefsSub = String(format: "Phonix-%@-settings", title.replacingOccurrences(of: ": ", with: "-"))
            let prefsPath = PhonixPrefsFolder.appendingPathComponent(prefsSub).path
            (settingsData as! NSDictionary).write(toFile: prefsPath, atomically: true)
        }
    }
    
    var AUbookmarks = [URL: Data]()
    func AUbookmarkPath() -> String
    {
        var url = PhonixPrefsFolder
        url = url.appendingPathComponent("PhonixAUbookmarkdata.dict")
        return url.path
    }
    
    func loadAUBookmarks()
    {
        let path = AUbookmarkPath()
        do {
            if let data = try NSKeyedUnarchiver.unarchiveTopLevelObjectWithData(path.data(using: String.Encoding.utf8)!) {
                AUbookmarks = data as! [URL : Data]
                for bookmark in AUbookmarks
                {
                    restoreAUBookmark(bookmark)
                }
            }
        } catch {
            //            print("Couldn't read file.")
        }
    }
    
    func saveAUBookmarks()
    {
        let path = AUbookmarkPath()
        do {
            let data = try NSKeyedArchiver.archivedData(withRootObject: AUbookmarks, requiringSecureCoding: false)
            try data.write(to: URL(fileURLWithPath: path))
        } catch {
            //print("Couldn't write file")
        }
        
    }
    
    func storeAUBookmark(url: URL)
    {
        do
        {
            let data = try url.bookmarkData(options: NSURL.BookmarkCreationOptions.withSecurityScope, includingResourceValuesForKeys: nil, relativeTo: nil)
            AUbookmarks[url] = data
        }
        catch
        {
            //            print ("Error storing bookmarks")
        }
    }
    
    func restoreAUBookmark(_ bookmark: (key: URL, value: Data))
    {
        let resourceAvailable = (try? bookmark.key.checkResourceIsReachable()) ?? false
        if resourceAvailable {
            let restoredUrl: URL?
            var isStale = false
            do
            {
                restoredUrl = try URL.init(resolvingBookmarkData: bookmark.value, options: NSURL.BookmarkResolutionOptions.withSecurityScope, relativeTo: nil, bookmarkDataIsStale: &isStale)
            }
            catch
            {
                restoredUrl = nil
            }
            
            if let url = restoredUrl
            {
                if isStale
                {
                    //                    print ("URL is stale")
                }
                else
                {
                    if !url.startAccessingSecurityScopedResource()
                    {
                        //                        print ("Couldn't access: \(url.path)")
                    }
                }
            }
        }
    }
    
    /// In the panel, Select the USB key containing WAVES license and press the Open button
    private func allowPluginAccess() -> Bool {
        let panel                   = NSOpenPanel()
        panel.message = "Please choose your USB key with your WAVES licenses"
        panel.canChooseDirectories  = true
        panel.canChooseFiles        = false
        panel.canCreateDirectories  = false
        panel.directoryURL          = FileManager.default.urls(for: .userDirectory, in: .localDomainMask).first!
        if panel.runModal() == NSApplication.ModalResponse.OK {
            guard let url = panel.url else {
                return false
            }
            storeAUBookmark(url: url)
            return true
        } else {
            return false
            
        }
    }
}
