import AppKit

class HTTPWaveFormPlotHistoryInfo {
    var buffer = [Float]()
    var bufferSize = Int()
    var circularBuffer = TPCircularBuffer()
}

class HTTPWaveFormViewOSX: NSView {
    var historyInfo: HTTPWaveFormPlotHistoryInfo?
    var progress: NSProgressIndicator?
    var positionPath = CGMutablePath()
    var waveData : [CGPoint]?
    var waveLength: Int = 0
    var disabled = false
    var waveColor: NSColor?
    
    required init?(coder: NSCoder) {
        super.init(coder: coder)
        initView()
    }
    
    func initView() {
        //
        // not used
        //
        waveColor = WaveColor
        resetHistoryBuffers()
    }
    
    func resetHistoryBuffers() {
        //
        // Clear any existing data
        //
        // WFh_resetHistoryBuffers: window.width: 356.000000
        //
        let historyLength = 150 // Int(self.bounds.size.width) - 2
        if historyInfo != nil {
            clear(historyInfo!)
            freeHistoryInfo(historyInfo!)
        }
        //
        // the default length should be set to window width
        //
        historyInfo = historyInfoWithDefaultLength(defaultLength: historyLength,
                                                   maximumLength: historyLength)
        releaseSample()
        needsDisplay = true
    }
    
    func releaseSample() {
        waveData = []
        waveLength = 0
    }
    
    // MARK: Mouse Handling
    override func acceptsFirstMouse(for theEvent: NSEvent?) -> Bool {
        return false
        // org: YES;
    }
    
    // MARK: Drawing
    func isOpaque() -> Bool {
        return false
        // org: NO
    }
    
    func setDisabled(_ state: Bool) {
        //print("WfH_setDisabled(): ",  state)
        disabled = state
        needsDisplay = true
    }
    
    func waveRect() -> NSRect { return NSMakeRect(0.0, 0.0, bounds.size.width, bounds.size.height) }
    
    override func draw(_ dirtyRect: NSRect) {
        let lineSize    : CGFloat = 2.0
        let waveMarkFill: NSColor = WaveColor
        let waveRect    : NSRect  = dirtyRect
        if !waveData!.isEmpty && waveLength > 0 && !disabled {
            positionPath = .init()
            let halfWave = CGMutablePath()
            halfWave.addLines(between: waveData!, transform: .identity)
            
            let halfHeight  : CGFloat = waveRect.size.height / 2.0
            let xScale      : CGFloat = waveRect.size.width / CGFloat(waveLength)
            let yScale      : CGFloat = halfHeight - lineSize / 2.0
            
            var xf: CGAffineTransform = .identity
            //
            // transform halfWave to upper half
            //
            xf = xf.translatedBy(x: 0.0, y: halfHeight + lineSize / 2.0)
            xf = xf.scaledBy(x: xScale, y: yScale)
            
            positionPath.addPath(halfWave, transform: xf)
            //
            // transform halfWave to lower half
            //
            xf = .identity
            xf = xf.translatedBy(x: 0.0, y: halfHeight - lineSize / 2.0)
            xf = xf.scaledBy(x: xScale, y: -yScale)
            
            positionPath.addPath(halfWave, transform: xf)
            //
            // draw zero line
            //
            let zeroLine = CGPath(rect: NSMakeRect(0.0,
                                                   (waveRect.size.height - lineSize) / 2.0,
                                                   waveRect.size.width,
                                                   lineSize),
                                  transform: nil)
            
            positionPath.addPath(zeroLine, transform: .identity)
        } else {
            //
            // we draw a Flat liner when disabled, with colored playProgress
            //
            positionPath = CGPath(rect: NSMakeRect(0.0,
                                                   (waveRect.size.height - lineSize) / 2.0,
                                                   waveRect.size.width,
                                                   lineSize),
                                  transform: nil) as! CGMutablePath
        }
        //
        // draw wave
        //
        NSGraphicsContext.current?.saveGraphicsState()
        let cr = NSGraphicsContext.current?.cgContext
        waveRect.clip()
        cr?.addPath(positionPath)
        waveMarkFill.setFill()
        cr?.fillPath()
        NSGraphicsContext.current?.restoreGraphicsState()
    }
    
    func setSampleToWaveData(theSampleData: [Float], length: Int) {
        //
        // append the buffer to the history
        //
        appendConvertedBuffer(theSampleData, withBufferSize: UInt32(length), to: historyInfo!)
        //
        // copy c buffer data to waveData
        //
        copyCbuf(toWaveData: historyInfo!.buffer, length: historyInfo!.bufferSize)
        needsDisplay = true
    }
    
    func copyCbuf(toWaveData theSampleData: [Float], length: Int) {
        waveLength = 0
        //
        // length is view width - 2
        //
        let bufLength: UInt32 = UInt32(length) + 2
        var tempData = [CGPoint](repeating: CGPoint(), count: Int(bufLength))
        //
        // set startpoint
        //
        tempData[0] = CGPoint(x: 0.0, y: 0.0)
        //
        // set endpoint
        //
        tempData[Int(bufLength - 1)] = CGPoint(x: CGFloat(bufLength) - 1.0, y: 0.0)
        //
        // fill the path with sample data
        //
        for i in 0 ..< length {
            tempData[i + 1] = CGPoint(x: CGFloat(i + 1), y: CGFloat(theSampleData[i]))
        }
        waveData = tempData
        waveLength = Int(bufLength)
    }
    
    func appendConvertedBuffer(_ buffer: [Float], withBufferSize bufferSize: UInt32, to historyInfo: HTTPWaveFormPlotHistoryInfo) {
        //
        // Calculate a single RMS value of the buffer and append to buffer
        //
        let maxValue: Float = self.buffer(toMAXvalue: buffer, length: Int(bufferSize))
        var src = [Float](repeating: Float(), count: 1)
        src[0] = maxValue.isNaN ? 0.0 : maxValue
        appendBuffer(src, withBufferSize: 1, to: historyInfo)
    }
    
    func appendBuffer(_ buffer: [Float],
                      withBufferSize bufferSize: UInt32,
                      to historyInfo: HTTPWaveFormPlotHistoryInfo) {
        //
        // Do nothing if there is no buffer
        //
        if Int(bufferSize) == 0 {
            return
        }
        //
        // Update the scroll history datasource
        //
        TPCircularBufferProduceBytes(&historyInfo.circularBuffer, buffer, bufferSize * UInt32(MemoryLayout<Float>.size))
        
        let targetBytes = historyInfo.bufferSize * MemoryLayout<Float>.size
        var availableBytes: UInt32 = 0
        let historyBuffer = TPCircularBufferTail(&historyInfo.circularBuffer, &availableBytes)
        let bytes = min(targetBytes, Int(availableBytes))
        memmove(&historyInfo.buffer, historyBuffer, Int(bytes))
        if targetBytes <= availableBytes {
            TPCircularBufferConsume(&historyInfo.circularBuffer, availableBytes - UInt32(targetBytes))
        }
    }
    
    func clear(_ historyInfo: HTTPWaveFormPlotHistoryInfo) {
        memset(&historyInfo.buffer, 0, historyInfo.bufferSize * MemoryLayout<Float>.size)
        TPCircularBufferClear(&historyInfo.circularBuffer)
    }
    
    func historyInfoWithDefaultLength(defaultLength: Int,
                                      maximumLength: Int) -> HTTPWaveFormPlotHistoryInfo {
        //
        // Setup buffers
        //
        let historyInf = HTTPWaveFormPlotHistoryInfo()
        
        historyInf.bufferSize = maximumLength
        historyInf.buffer = [Float].init(repeating: Float(), count: maximumLength)
        let bufferInitalized = _TPCircularBufferInit(&historyInf.circularBuffer, UInt32(maximumLength), MemoryLayout<TPCircularBuffer>.size)
        if !bufferInitalized {  }
        //
        // Zero out circular buffer
        //
        let emptyBuffer = [Float](repeating: 0.0, count: maximumLength)
        let size: UInt32 = UInt32(maximumLength * MemoryLayout<Float>.size)
        TPCircularBufferProduceBytes(&historyInf.circularBuffer,
                                     emptyBuffer,
                                     size)
        return historyInf
    }
    
    func freeHistoryInfo(_ historyInfo: HTTPWaveFormPlotHistoryInfo) {
        TPCircularBufferCleanup(&historyInfo.circularBuffer)
    }
    
    func buffer(toMAXvalue buffer: [Float], length bufferSize: Int) -> Float {
        var max: Float = 0.0
        for i in 0 ..< bufferSize {
            let absValue = abs(buffer[i])
            if absValue > max {
                max = absValue
            }
        }
        return max
    }
}
