import AppKit

@objcMembers
class K_MeterController: NSPanel, NSWindowDelegate {
    var kMeterUnit: AudioUnit!
    var kMeterName = ""
    
    convenience init(withOptions kUnit: AudioUnit, kName: NSString) {
        self.init()
        kMeterUnit = kUnit
        kMeterName = kName as String
        //
        // kmeter Window properties
        //
        kInit()
    }
    
    func kInit() {
        //
        // kmeter Window properties
        //
        initKMeterWindow()
        //
        // init kMeter
        //
        initKmeterView()
        //
        // load settings
        //
        loadKmeterSettings()
        //
        // add observe for closing window = bypass
        //
        NotificationCenter.default.addObserver(forName: NSWindow.willCloseNotification, object: self, queue: nil, using: windowWillClose(_:))
    }
    
    func windowWillClose(_ notification: Notification) {
        //
        // store the position of kmeter window here
        //
        storeKmeterPosition()
        //
        //  deactivate kmeter, unless the app is terminating, we want to keep our current status in the UserDefaults
        //
        if phonixWillTerminate == false {
            kMeterBypass(true)
        }
    }
    
    func initKMeterWindow() {
        //
        //  we want an always on top meterWindow
        //
        level = .floating
        hidesOnDeactivate = false
        hasShadow = true
        styleMask = [.utilityWindow,
                     .hudWindow,
                     .titled,
                     .closable] 
        setKmeterPosition()
    }
    
    func initKmeterView() {
        // get AU's Cocoa view property
        var dataSize: UInt32 = 0
        var isWritable: DarwinBoolean = false
        var cocoaViewBundlePath: NSURL?
        var viewFactoryName: NSString?
        var cocoaViewInfo: AudioUnitCocoaViewInfo?
        let result: OSStatus = AudioUnitGetPropertyInfo(kMeterUnit!, kAudioUnitProperty_CocoaUI, kAudioUnitScope_Global, 0, &dataSize, &isWritable)
        // Does view have custom Cocoa UI?
        if (result == noErr) && (Int(dataSize) > 0) {
            //
            // AudioUnitCocoaViewInfo:  The name and number of custom Cocoa views for an audio unit.
            //
            // cocoaViewInfo = UnsafeMutablePointer<AudioUnitCocoaViewInfo>.allocate(capacity: Int(dataSize)).pointee
            if AudioUnitGetProperty(kMeterUnit!, kAudioUnitProperty_CocoaUI, kAudioUnitScope_Global, 0, &cocoaViewInfo, &dataSize) == noErr {
                cocoaViewBundlePath = cocoaViewInfo?.mCocoaAUViewBundleLocation.takeRetainedValue()
                // there is typicaly one view.
                viewFactoryName = cocoaViewInfo?.mCocoaAUViewClass.takeUnretainedValue()
            } else {
                if cocoaViewInfo != nil {
                    cocoaViewInfo = nil
                }
            }
        }
        var auView: NSView?
        var wasAbleToLoadCustomView = false
        // [A] Show custom UI if view has it
        if cocoaViewBundlePath != nil && viewFactoryName != nil {
            let viewBundle = Bundle(path: (cocoaViewBundlePath?.path)!)
            if viewBundle != nil {
                let factoryClass: AnyClass = (viewBundle?.classNamed(viewFactoryName! as String).self)!
                if K_MeterController().plugInClassIsValid(factoryClass) {
                    // make a factory
                    let factoryInstance = factoryClass.alloc()
                    // make a view
                    auView = (factoryInstance as! AUCocoaUIBase).uiView(forAudioUnit: kMeterUnit!, with: CGSize(width: 20, height: 30))
                    // size doesn't matter
                    if auView != nil {
                        // Display View!
                        wasAbleToLoadCustomView = true
                        displayKmeterView(auView)
                    }
                }
            }
        }
        if !wasAbleToLoadCustomView {
            // [B] Otherwise show generic Cocoa view (if needed!)
        }
    }
    
    func plugInClassIsValid(_ pluginClass: AnyClass) -> Bool {
        if pluginClass.conforms(to: AUCocoaUIBase.self) {
            if pluginClass.instancesRespond(to: #selector(AUCocoaUIBase.interfaceVersion)) && pluginClass.instancesRespond(to: #selector(AUCocoaUIBase.uiView(forAudioUnit:with:))) {
                return true
            }
        }
        return false
    }
    
    func displayKmeterView(_ kView: NSView?) {
        if kMeterName != "" {
            title = "\(kMeterName)"
        }
        //
        //  adjust window size to K-view
        //
        setFrame(NSMakeRect(frame.origin.x, frame.origin.y, kView?.frame.size.width ?? 0.0, (kView?.frame.size.height ?? 0.0) + 16), display: false, animate: true)
        if let aView = kView {
            contentView?.addSubview(aView)
        }
    }
    
    func kMeterActive(_ state: Bool) {
        if state {
            orderFrontRegardless()
            kMeterBypass(false)
        } else {
            kMeterBypass(true)
            close()
            //
            //  bypass is set in windowWillClose
            //
        }
    }
    
    func kMeterBypass(_ bypass: Bool) {
        var onValue = UInt32(truncating: NSNumber(value: bypass))
        var err: OSStatus
        err = AudioUnitSetProperty(kMeterUnit!,
                                   kAudioUnitProperty_BypassEffect,
                                   kAudioUnitScope_Global,
                                   0,
                                   &onValue,
                                   UInt32(MemoryLayout.size(ofValue: onValue)))
        if err == noErr {
            UserDefaults.standard.set(!bypass, forKey: "kMeterOn")
        }
        /*
         if bypass == false {
         //
         // reset k-Meter, looking for a way to trigger reset.
         // reset is not a parameter
         //
         print("KMC_kMeterBypass: reset levels")
         
         // var data : UInt32 = 0
         // var dataSize : UInt32 = UInt32(MemoryLayout<CFPropertyList?>.size)
         
         var dataSize: UInt32 = 0
         
         err = AudioUnitGetPropertyInfo(kMeterUnit!,
         kAudioUnitProperty_ParameterList,
         kAudioUnitScope_Global,
         0,
         &dataSize,
         nil)
         
         var parameterIDs = [AudioUnitParameterID](repeating: AudioUnitParameterID(), count: Int(dataSize))
         
         err = AudioUnitGetProperty(kMeterUnit!,
         kAudioUnitProperty_ParameterList,
         kAudioUnitScope_Global,
         0,
         &parameterIDs,
         &dataSize)
         
         debugPrint("KMC_kMeterBypass: size:", dataSize)
         // dump(parameterIDs, name: "KMC_kMeterBypass: parameterIDs:")
         debugPrint("KMC_kMeterBypass: parameterIDs:", parameterIDs)
         
         var parameterInfo_t: AudioUnitParameterInfo = .init()
         dataSize = UInt32(MemoryLayout<AudioUnitParameterInfo>.size)
         
         let parameterCount: UInt32 = dataSize / UInt32(MemoryLayout<AudioUnitParameterID>.size)
         
         for pIndex in 0 ..< parameterCount {
         AudioUnitGetProperty(kMeterUnit!,
         kAudioUnitProperty_ParameterInfo,
         kAudioUnitScope_Global,
         parameterIDs[Int(pIndex)],
         &parameterInfo_t,
         &dataSize)
         
         // debugPrint(pIndex, parameterInfo_t )
         // dump(parameterInfo_t)
         }
         }*/
    }
    
    func setKmeterPosition() {
        //
        // get the stored coordinates from userdefaults
        //
        if let position = UserDefaults.standard.value(forKey: "kMeterPosition") as? [CGFloat] {
            var pos = CGPoint()
            pos.x = position[0]
            pos.y = position[1]
            setFrameOrigin(pos)
        } else {
            setFrameOrigin(NSZeroPoint)
        }
    }
    
    func storeKmeterPosition() {
        //
        //  get the the window coordinates
        //
        var position = NSPoint()
        position.x = (NSScreen.main?.visibleFrame.origin.x)! + frame.origin.x
        position.y = (NSScreen.main?.visibleFrame.origin.y)! + frame.origin.y
        //
        //  store the coordinates to userdefaults
        //
        UserDefaults.standard.set([position.x, position.y], forKey: "kMeterPosition")
    }
    
    func loadKmeterSettings() {
        let prefsSub = "Phonix-K-Meter-settings"
        let prefsPath = PhonixPrefsFolder.appendingPathComponent(prefsSub).path
        if PhonixFileManager.fileExists(atPath: prefsPath) {
            var settingsData = NSDictionary(contentsOfFile: prefsPath)
            let count = CFDictionaryGetCount(settingsData)
            if count > 0 {
                AudioUnitSetProperty(kMeterUnit!,
                                     kAudioUnitProperty_ClassInfo,
                                     kAudioUnitScope_Global,
                                     0,
                                     &settingsData,
                                     UInt32(MemoryLayout<CFPropertyList?>.size))
            }
        }
    }
    
    func saveKmeterSettings() {
        var err: OSStatus = noErr
        var settingsData: CFPropertyList?
        var size = UInt32(MemoryLayout<CFPropertyList?>.size)
        err = AudioUnitGetProperty(kMeterUnit!,
                                   kAudioUnitProperty_ClassInfo,
                                   kAudioUnitScope_Global,
                                   0,
                                   &settingsData,
                                   &size)
        if settingsData != nil && err == noErr {
            // be aware, some of no-licensed plugins refuse saving!
            let prefsSub = "Phonix-K-Meter-settings"
            let prefsPath = PhonixPrefsFolder.appendingPathComponent(prefsSub).path
            (settingsData as! NSDictionary).write(toFile: prefsPath, atomically: true)
        }
    }
}
