import AppKit

class LevelMeter: NSView {
    var graphLevels = [CGFloat](repeating: 0.0, count: 4)
    let centerGap   : CGFloat = 6.0
    
    var scaleMarks  : [CGFloat] {
        return [
            0.0628, // -60dB
            0.1600, // -48dB
            0.3010, // -36dB
            0.4895, // -24dB
            0.7204, // -12dB
            0.8547, // -6dB
            0.9238] // -3dB
    }
    
    func setLevels(_ inLevels: [Float]) {
        // scale levels: level (-120.0 - 0.0) to (0 - 200) METERSEGMENTS = 200
        for i in 0 ..< 4 {
            //
            // limit range to -80dB to 0dB.
            //
            if inLevels[i] > -80.0 {
                //
                // (0 - 120)/80 = (0 - 1.5) >(-0.5)> (-0.5 - 1.0)
                //
                graphLevels[i] = CGFloat(( (120.0 + inLevels[i]) / 80.0 ) - 0.5)
            } else {
                graphLevels[i] = 0.0
            }
            //
            // convert to meter range (sqr)
            //
            graphLevels[i] = graphLevels[i] * graphLevels[i]
        }
        needsDisplay = true
    }

    override func draw(_ rect: CGRect) {
        let barHeight   : CGFloat = (frame.size.height - centerGap) / 2.0
        let topBarPos_y : CGFloat = (frame.size.height + centerGap) / 2.0
        let width: CGFloat = frame.size.width
        //
        // meter background
        //
        MeterBackgroundColor.setFill()
        //
        // draw lower bar background
        //
        var backGround: NSRect = NSMakeRect(0.0, 0.0, width, barHeight)
        backGround.fill()
        //
        // draw upper bar background
        //
        backGround = NSMakeRect(0.0, topBarPos_y, width, barHeight)
        backGround.fill()
        //
        // level bars
        //
        MeterBarColor.setFill()
        //
        // draw lower bar (average level right channel)
        //
        var levelBar: NSRect = NSMakeRect(0.0,
                                          0.0,
                                          width * graphLevels[1],
                                          barHeight)
        levelBar.fill()
        //
        // draw upper bar (average level left channel)
        //
        levelBar = NSMakeRect(0.0,
                              topBarPos_y,
                              width * graphLevels[0],
                              barHeight)
        levelBar.fill()
        //
        // scale lines
        //
        let scaleLine: NSBezierPath = NSBezierPath.init()
        
        for scaleMark in scaleMarks as [CGFloat] {
            let pos_x = scaleMark * width
            //
            // draw lower bar scale mark
            //
            scaleLine.move(to: NSMakePoint(pos_x, 0.0))
            scaleLine.line(to: NSMakePoint(pos_x, barHeight))
            //
            // draw upper bar scale mark
            //
            scaleLine.move(to: NSMakePoint(pos_x, topBarPos_y))
            scaleLine.line(to: NSMakePoint(pos_x, topBarPos_y + barHeight))
        }
        scaleLine.lineWidth = 1.0
        MeterScaleColor.setStroke()
        scaleLine.stroke()
        //
        // peak Level indicators
        //
        let peakLine: NSBezierPath = NSBezierPath.init()
        let topPos_x = width * graphLevels[2]
        let botPos_x = width * graphLevels[3]
        //
        // draw lower peak Level indicator
        //
        peakLine.move(to: NSMakePoint(botPos_x, 0.0))
        peakLine.line(to: NSMakePoint(botPos_x, barHeight))
        //
        // draw upper peak Level indicator
        //
        peakLine.move(to: NSMakePoint(topPos_x, topBarPos_y))
        peakLine.line(to: NSMakePoint(topPos_x, topBarPos_y + barHeight))
        
        peakLine.lineWidth = 3.0
        MeterPeakColor.setStroke()
        peakLine.stroke()
    }
}
